.wc-block-components-notices {
	display: block;
	margin-bottom: 2em;
	.wc-block-components-notices__notice {
		margin: 0;
		display: flex;
		flex-wrap: nowrap;
		.components-notice__dismiss {
			background: transparent none;
			padding: 0;
			margin: 0 0 0 auto;
			border: 0;
			outline: 0;
			color: currentColor;
			svg {
				fill: currentColor;
				vertical-align: text-top;
			}
		}
	}
	.wc-block-components-notices__notice + .wc-block-components-notices__notice {
		margin-top: 1em;
	}
}

// @todo Either move notice style fixes to Woo core, or take full control over notice component styling in blocks.
.theme-twentytwentyone,
.theme-twentytwenty {
	.wc-block-components-notices__notice {
		padding: 1.5rem 3rem;
	}
}
