img {
    max-width: 100%;
}
.full-width{
    width: 100%;
}

#dc_admin_footer {
    bottom: 24px;
    text-align: center;
    position: absolute;
    width: 100%;
    right: 0;
    left: 0;
}

#dc_admin_footer a {
    text-decoration: none;
}

#dc_admin_footer img {
    border: none;
    margin-left: 5px;
    vertical-align: bottom;
}

.required {
    color: red;
}

.img_tip {
    background: url("../../images/question_mark.png") no-repeat scroll 100% 0 rgba(0, 0, 0, 0);
    display: inline-block;
    height: 16px;
    vertical-align: -1px;
    width: 16px;
    cursor: pointer;
    /*    margin-left: 5px;*/
}

.multi_input_block {
    border: 1px solid #dfdfdf;
    border-radius: 10px;
    padding: 10px;
    margin-bottom: 10px;
}

.multi_input_block_manupulate {
    float: left;
    margin: 2px !important;
}

#choose_vendor_ajax_chosen {
    width: 90% !important;
}

#choose_vendor_ajax_chosen .chosen-results {
    padding: 3px;
    clear: both;
}

#vendor-label {
    float: none;
    margin: 0;
    padding: 0;
    width: 52px;
}

label[for=default_commission],
label[for=commission_include_coupon],
label[for=commission_type],
label[for=sold_by_catalog],
label[for=catalog_colorpicker],
label[for=catalog_hover_colorpicker],
label[for=block_vendor_desc],
label[for=sold_by_cart_and_checkout],
label[for=sold_by_text],
label[for=give_shipping],
label[for=give_tax] {
    text-decoration: line-through;
}

#default_commission,
#sold_by_texttt,
#commission_type,
#commission_include_coupon,
#sold_by_catalog,
#catalog_colorpicker,
#catalog_hover_colorpicker,
#sold_by_cart_and_checkout,
#sold_by_text,
#block_vendor_desc,
#give_shipping,
#give_tax {
    display: none;
}

.wcmp_vendors_settings .form-table tbody tr td {
    padding: 0;
    padding-top: 10px;
    padding-bottom: 10px;
}

.wcmp_vendors_settings .form-table td p {
    margin-bottom: 10px !important;
}

.wcmp_vendors_settings #to_do_list tbody tr td,
.wcmp_vendors_settings #to_do_list tbody tr th {
    padding-right: 10px;
}

.wcmp_vendors_settings #to_do_list tbody tr th {
    background: rgb(216, 216, 216);
}

.wcmp_vendors_settings #to_do_list tbody tr {
    border-bottom: 2px solid rgb(247, 245, 245)
}

.wcmp_vendors_settings #to_do_list {
    border-radius: 5px;
}

span.vendor_bulk div.chosen-container {
    width: 50% !important;
}

span.variable_commission_cls {
    font-size: 15px;
}

.wcmpsubtabadmin {
    margin: 8px 0 0 !important;
}

a.wcmp_sub_sction {
    border-left: 1px solid #999 !important;
    display: inline-block;
    line-height: 14px;
    padding: 0 8px;
}

.wcmpsubtabadmin li:last-child a.wcmp_sub_sction {
    border-left: 0 none !important;
}

.inline-edit-col span.inline-edit-categories-label:nth-of-type(2),
.inline-edit-col ul.product_shipping_class-checklist {
    display: none !important;
}


/*... One product multiple vendor ..............................................*/

#wcmp_auto_suggest_product_title {
    box-shadow: 0 0 3px 1px rgba(0, 0, 0, 0.3);
    margin-top: -16px;
    margin-bottom: 16px;
}

#wcmp_auto_suggest_product_title>ul {
    border: 1px solid #aaa;
    list-style: outside none none;
    max-height: 140px;
    overflow-y: auto;
    padding: 10px;
}

#wcmp_auto_suggest_product_title li a {
    border-bottom: 1px solid #ddd;
    color: #727272;
    display: inline-block;
    text-decoration: none;
    width: 100%;
}

.wcmp_vendors_settings select {
    width: 25em;
}

.wcmp_vendors_settings fieldset {
    position: relative;
    padding-right: 25px;
}

.wcmp_vendors_settings fieldset .hint-container {
    position: absolute;
    right: 0;
    margin: auto !important;
    top: 5px;
}

@media screen and (max-width:782px) {
    .wcmp_vendors_settings fieldset .hint-container {
        position: relative;
        top: -5px;
    }
    .wcmp_vendors_settings fieldset {
        padding-right: 0;
    }
}


/* template selection */

.wcmp_template_list {
    margin: 0 0 20px 0;
    position: relative;
}

.wcmp_template_list .dashicons {
    position: absolute;
    background: #151715;
    color: #fff;
    line-height: 34px;
    top: 0;
    width: 35px;
    left: 0;
    height: 35px;
}

.wcmp_template_list input[type="radio"]:checked+.dashicons {
    background: #0073aa;
}

.wcmp_template_list input[type="radio"]:checked+.dashicons:before {
    content: '\f160';
}


/* admin submenu design */

.wcmp_subtab_container {
    border: solid 1px #ccc;
    box-sizing: border-box;
    overflow: hidden;
}

.wcmp_subtab_content,
.vendor-preview-form {
    background: #fff;
    float: right;
    width: 100%;
    padding: 25px;
    box-sizing: border-box;
}

.wcmp_subtab_content h2, .wcmp-ui-tabs h2 {
    border-bottom: 1px solid #ccc;
    padding-top: 30px;
}

.wcmp_subtab_content h2:first-of-type, .wcmp-ui-tabs h2:first-of-type {
    padding-top: 0;
    margin-top: 0;
}

.subsubsub.wcmpsubtabadmin li a,
.wcmp-ui-tabs .ui-tabs-nav li a {
    padding: 10px;
    display: block;
    font-size: 13px;
    font-weight: 600;
    border-color: #ccc !important;
    border-width: 0 0 1px 1px !important;
    border-style: solid;
    border-bottom: 0;
    color: #7f8591;
    text-decoration: none;
}

.subsubsub.wcmpsubtabadmin li a .dashicons,
.wcmp-ui-tabs li .dashicons {
    vertical-align: middle;
    margin-left: 7px;
}

.subsubsub.wcmpsubtabadmin li a:hover,
.subsubsub.wcmpsubtabadmin li a.current,
.wcmp-ui-tabs .ui-tabs-nav li a:hover,
.wcmp-ui-tabs .ui-tabs-nav li.ui-tabs-active a {
    background: #fff;
    color: #333;
}

.subsubsub.wcmpsubtabadmin li a:hover .dashicons,
.subsubsub.wcmpsubtabadmin li a.current .dashicons,
.wcmp-ui-tabs li a:hover .dashicons,
.wcmp-ui-tabs li.ui-tabs-active .dashicons {
    color: #F86A7C;
}

.wcmp_subtab_container .wcmpsubtabadmin li a{
    white-space: normal;
}

#vendor_preview_tabs{
        padding: 0;
}
#vendor_preview_tabs .ui-tabs-nav{
        padding: 0;
        border: 0;
}
#vendor_preview_tabs .ui-tabs-nav li{
        float: none !important;
        white-space: initial;
        clear: both;
        border: 0;
}
#vendor_preview_tabs .ui-tabs-nav li a{
        float: none !important;
}
#vendor_preview_tabs .ui-widget-header {
    border: 1px solid #e78f08;
    background: #f1f1f1;
}
@media screen and (min-width: 768px){
        .subsubsub.wcmpsubtabadmin li a, .wcmp-ui-tabs .ui-tabs-nav li a {
            padding: 20px 15px !important;
    }
}

@media screen and (min-width: 768px) {
    .wcmp_subtab_container .subsubsub.wcmpsubtabadmin,
    .wcmp-ui-tabs ul.ui-tabs-nav {
        margin: 0 0 0 -1px !important;
        width: 200px;
        position: relative;
        float: right;
    }
    .subsubsub.wcmpsubtabadmin li,
    .ui-tabs-vertical .ui-tabs-nav li {
        display: block;
        padding: 0;
        margin: 0;
    }
    .subsubsub.wcmpsubtabadmin li a,
    .wcmp-ui-tabs .ui-tabs-nav li a {
        padding: 20px 15px;
        display: block;
        font-size: 14px;
        border-color: #ccc !important;
        border-width: 0 0 1px 1px !important;
        border-style: solid;
    }
    .wcmp_subtab_content,
    .vendor-preview-form {
        width: calc(100% - 200px);
        border-right: 1px solid #ccc;
    }
    #vendor_preview_tabs {
        display: -webkit-box;
        display: flex;
    }
    .wcmpsubtabadmin li:last-child a.wcmp_sub_sction {
        border-left: 1px solid #ccc !important;
    }
}

.subsubsub.wcmpsubtabadmin li a:focus {
    box-shadow: none;
}

h4.wcmp-tab-description {
    padding: 12px 20px;
    margin: 25px 0;
    border-style: solid;
    border-width: 0 4px 0 0;
    border-color: #ffb900;
    background-color: #fff8e5;
}

h4.wcmp-tab-description.success {
    border-color: #ffb900;
    background-color: #fff8e5;
}

.wcmp_tab_content {
    background: #fff;
    padding: 0 20px;
    border: solid 1px #ccc;
}

.wcmp_vendors_settings table {
    margin: 0;
}

.nav-tab-active,
.nav-tab-active:active,
.nav-tab-active:focus,
.nav-tab-active:hover {
    background: #fff;
}

.nav-tab-wrapper a.nav-tab:focus {
    box-shadow: none;
}

.wcmp_vendors_settings #poststuff {
    min-width: auto;
}

.wcmp_subtab_container .form-table tr:first-of-type th,
.wcmp_subtab_container .form-table tr:first-of-type td {
    padding-top: 0;
}

.wcmp_vendors_settings h2, .wcmp-ui-tabs h2 {
    color: #0073aa;
    padding-bottom: 10px;
}

.wcmp_vendors_settings h2+.wcmp-section-info {
    margin: -11px 0 30px 0;
}

.wcmp_page_wcmp-setting-admin .nav-tab-wrapper a.nav-tab:first-of-type {
    margin-right: 0;
}


/* settings */

.wcmp-settings-wrap h2.nav-tab-wrapper {
    /* background: #63c97d; */
    background: #8abee5;
    margin: 40px 0 25px;
    padding: 0;
    border: 0;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a {
    margin: 0;
    border: 0;
    background: transparent;
    padding: 13px 35px;
    color: #fff;
    -webkit-transition: color 0.3s;
    transition: color 0.3s;
    position: relative;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a:after {
    content: '';
    position: absolute;
    right: 0;
    left: 0;
    bottom: 10px;
    width: 0;
    height: 2px;
    background: #fff;
    margin: auto;
    -webkit-transition: all 0.3s;
    transition: all 0.3s;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a:hover:after {
    width: 25px;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a:hover {
    color: #fff;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a.nav-tab-active {
    background: #fff;
    /* color: #63c97d; */
    color: #8abee5;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a dashicons {
    color: #aff1b6;
}

.wcmp-settings-wrap h2.nav-tab-wrapper a.nav-tab-active dashicons {
    color: #24b47e;
}

.wp-core-ui .wrap.wcmp-settings-wrap .button.button-primary,
.wp-core-ui .wrap.wcmp-settings-wrap .button-primary,
.wcmp-primary-btn.button-primary {
    background: #fe6778;
    border-color: #fe6778;
    box-shadow: 0 1px 0 #d85261;
    text-shadow: none;
    width: auto;
    height: auto;
    padding: 4px 15px;
    font-size: 14px;
}

.wp-core-ui .wrap.wcmp-settings-wrap .button.button-primary:hover,
.wcmp-primary-btn.button-primary:hover,
.wcmp-primary-btn.button-primary:focus {
    background: #d85261;
    border-color: #d85261;
    box-shadow: 0 1px 0 #d85261;
}

.wcmp-action-button.pull-right {
    background: #696364;
    border-color: #696364;
    box-shadow: 0 1px 0 #696364;
    text-shadow: none;
    width: auto;
    height: auto;
    padding: 4px 15px;
    font-size: 14px;
}
.wcmp-action-button.pull-right:focus, .wcmp-action-button.pull-right:hover {
    background: #4c4848;
    border-color: #4c4848;
}

.wcmp-settings-wrap .form-table th {
    padding: 10px 0 0 10px;
}

.wcmp-settings-wrap .wcmp_payment_help {
    left: 23px;
}

.vendor-preview-form #wc-backbone-modal-dialog {
    margin-top: 25px;
}

@media screen and (max-width:1006px) {
    .vendor-preview-form .regular-text, .vendor-preview-form #display_name, body #wp-vendor_description-wrap, body .wcmp-ui-tabs form fieldset textarea{
        width: 100%;
    }
    body .wcmp-ui-tabs form fieldset p.description{
        margin: 0;
    }
    .dc-wp-fields-uploader{
        display: block;
    }
    body #wp-vendor_shipping_policy-wrap, body #wp-vendor_refund_policy-wrap, body #wp-vendor_cancellation_policy-wrap {
    width: 100%; }
}

@media screen and (max-width:1170px) {
    .wcmp-settings-wrap .wcmp_payment_help,
    form.wcmp_vendors_settings .frontend_manager_promo {
        left: auto;
        position: relative;
        padding-right: 0;
        padding-top: 0;
    }
}

#wcmp_dashboard_site_logo_display,
#wcmp_dashboard_site_logo,
#wcmp_dashboard_site_logo_button {
    vertical-align: middle !important;
}


/* for color palet */

#vendor_color_scheme_picker .color-option {
    width: 42%;
    padding: 0;
    margin: -12px 0 5px 8%;
}

#vendor_color_scheme_picker .color-option .color-palette td {
    padding-bottom: 0;
}

@media screen and (max-width: 480px) {
    #vendor_color_scheme_picker .color-option {
        width: 100%;
        margin: 0 0 5px;
    }
}

#vendor_color_scheme_picker .color-option>label {
    padding: 10px;
    margin: 0 !important;
    display: block;
}

#vendor_color_scheme_picker .color-option:last-of-type {
    margin: 0;
}

#woocommerce-coupon-data ul.wc-tabs li.vendor_icons a::before,
#woocommerce-product-data ul.wc-tabs li.vendor_icons a::before,
.woocommerce ul.wc-tabs li.vendor_icons a::before {
    font-family: Dashicons;
    content: "\f338";
}

#woocommerce-coupon-data ul.wc-tabs li.policy_icons a::before,
#woocommerce-product-data ul.wc-tabs li.policy_icons a::before,
.woocommerce ul.wc-tabs li.policy_icons a::before {
    font-family: Dashicons;
    content: "\f497";
}


/* pending vendor */

.wp-list-table.vendors th#username {
    width: 250px;
}
.wp-list-table.vendors td{
    padding: 8px 10px;
}
.wp-list-table.bordered {
    margin-top: 5px;
}

.wp-list-table.bordered,
.wp-list-table.bordered th,
.wp-list-table.bordered td {
    border: 1px solid #e6e6e6;
    border-collapse: collapse;
    text-align: center;
}

.pending-vendor-clm {
    display: inline-block;
    width: calc(100% - 80px)
}

.vendor-preview {
    float: left;
    width: 16px;
    padding: 20px 4px 4px 4px;
    height: 0;
    overflow: hidden;
    position: relative;
    border: 2px solid transparent;
    margin: 6px 0px 0 6px;
}
@media screen and (max-width:782px){
    .vendor-preview{
        margin: 0px 0 0 37px;
    }
}

.vendor-preview::before {
    font-family: WooCommerce;
    speak: none;
    font-weight: 400;
    text-transform: none;
    text-indent: 0px;
    position: absolute;
    right: 0px;
    width: 100%;
    height: 100%;
    text-align: center;
    content: "";
    line-height: 16px;
    font-size: 14px;
    vertical-align: middle;
    top: 4px;
    font-variant: normal normal;
    margin: 0px;
}

.vendor-status {
    display: inline-block;
    font-size: 17px;
    padding: 8px 10px;
    border-radius: 3px;
}

.vendor-status.pending-vendor {
    background: #f8dda7;
    color: #94660c;
}

.vendor-status.approved-vendor {
    background: #c8d7e1;
    color: #2e4453;
}

.vendor-status.rejected-vendor {
    background: #e86767;
    color: #420c0c;
}

.vendor-status.suspended-vendor {
    background: #6d6d6d;
    color: #191919;
}


/* modal */

body .wcmp-vendor-modal-content article p {
    margin: 0 0 10px;
}

.shop-link {
    float: left;
}

.wcmp-vendor-modal * {
    box-sizing: border-box;
}

.wcmp-vendor-modal .wcmp-vendor-modal-content {
    position: fixed;
    background: #fff;
    z-index: 100000;
    right: 50%;
    top: 50%;
    -webkit-transform: translate(50%, -50%);
    -ms-transform: translate(50%, -50%);
    transform: translate(50%, -50%);
    width: 590px;
    max-width: 100%;
    max-height: 90%;
    overflow: auto;
}

.wcmp-vendor-modal-content a {
    text-decoration: none;
}

.wcmp-vendor-modal .wcmp-vendor-modal-content article {
    overflow: auto;
}

.wcmp-vendor-modal.wcmp-vendor-modal-shipping-method-settings .wcmp-vendor-modal-content {
    width: 75%;
    min-width: 500px;
}

.wcmp-vendor-modal .select2-container {
    width: 100%!important;
}

@media screen and (max-width:782px) {
    .wcmp-vendor-modal .wcmp-vendor-modal-content {
        width: 100%;
        min-width: 100%
    }
}

.wcmp-vendor-modal-backdrop {
    position: fixed;
    top: 0;
    right: 0;
    left: 0;
    bottom: 0;
    min-height: 360px;
    background: #000;
    opacity: .7;
    z-index: 99900;
}

.wcmp-vendor-modal-main {
    padding-bottom: 55px;
    position: relative;
}

.wcmp-vendor-modal-main article,
.wcmp-vendor-modal-main header {
    display: block;
    position: relative;
}

.wcmp-vendor-modal-main .wcmp-vendor-modal-header {
    height: auto;
    background: #fcfcfc;
    padding: 20px;
    border-bottom: 1px solid #ddd;
    position: relative;
}

.wcmp-vendor-modal-main .wcmp-vendor-modal-header+article {
    padding: 30px 0 10px;
}
.wcmp-vendor-modal-main .wcmp-vendor-modal-header+article.pending-vendor-article {
    padding: 30px 0 110px;
}


.wcmp-vendor-modal-main .wcmp-vendor-modal-header h1 {
    margin: 0;
    font-size: 21px;
    font-weight: 700;
    line-height: 20px;
    display: inline-block;
}

.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link {
    cursor: pointer;
    color: #777;
    height: 100%;
    width: 64px;
    padding: 0;
    position: absolute;
    top: 0;
    left: 0;
    text-align: center;
    border: 0;
    border-right: 1px solid #ddd;
    background-color: transparent;
    -webkit-transition: color .1s ease-in-out, background .1s ease-in-out;
    transition: color .1s ease-in-out, background .1s ease-in-out;
}

.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link::before {
    font: normal 40px/50px dashicons!important;
    color: #666;
    display: block;
    content: '\f335';
    font-weight: 300;
}

.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link:focus,
.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link:hover {
    background: #ddd;
    border-color: #ccc;
    color: #000;
}

.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link:focus {
    outline: 0;
}

.wcmp-vendor-modal-header h1 span {
    background: #028ed9;
    padding: 2px 10px;
    font-size: 16px;
    border-radius: 3px;
    margin-right: 5px;
    color: #fff;
}

.wcmp-vendor-modal-header h1 span i {
    font-size: 14px;
    vertical-align: middle;
    margin-right: 2px;
}

.wcmp-vendor-modal-main article {
    padding: 1.5em;
    max-height: 54vh;
}

.wcmp-vendor-modal-main article p {
    margin: 1.5em 0;
}

.wcmp-vendor-modal-main article p:first-child {
    margin-top: 0;
}

.wcmp-vendor-modal-main article p:last-child {
    margin-bottom: 0;
}

.wcmp-vendor-modal-main article .pagination {
    padding: 10px 0 0;
    text-align: center;
}

.wcmp-vendor-modal-main footer {
    position: absolute;
    right: 0;
    left: 0;
    bottom: 0;
    z-index: 100;
    padding: 15px 25px;
    background: #fcfcfc;
    border-top: 1px solid #dfdfdf;
    box-shadow: 0 -4px 4px -4px rgba(0, 0, 0, .1);
}

.wcmp-vendor-modal-main footer .inner {
    text-align: left;
    line-height: 23px;
}

.wcmp-vendor-modal-main footer .inner .button {
    margin-bottom: 0;
}

.wcmp-vendor-modal-main footer .inner .button.wcmp-action-button {
    box-shadow: none;
    margin-left: 5px;
    text-shadow: none;
}

.wcmp-vendor-modal-main footer .inner .button.pending-vendor {
    border-color: #eed093;
    background: #f8dda7;
    color: #94660c;
}

.wcmp-vendor-modal-main footer .inner .button.approve-vendor,
.wcmp-vendor-modal-main footer .inner .button.activate-vendor {
    border-color: #b8ccda;
    background: #c8d7e1;
    color: #2e4453;
}

.wcmp-vendor-modal-main footer .inner .button.reject-vendor {
    border-color: #eb6363;
    background: #e86767;
    color: #420c0c;
}

.wcmp-vendor-modal-main footer .inner .button.suspend-vendor {
    border-color: #5f5d5d;
    background: #6d6d6d;
    color: #191919;
}

.wcmp-vendor-modal h2 {
    margin: 0;
}

.wcmp-vendor-modal form {
    margin-top: 20px;
}

.wcmp-vendor-modal form textarea {
    width: 100%;
    height: 80px;
}

.wcmp-vendor-modal .vendor-status-header {
    margin: 1px 0 0 70px;
    float: left;
    font-size: 16px;
}
@media screen and (max-width:480px){
    .wcmp-vendor-modal .vendor-status-header span{
        display: none;
    }
    .wcmp-vendor-modal .vendor-status-header {
        margin: 1px 0 0 45px;
    }
}


.wcmp-vendor-modal-main article .vendor-info-row h3 {
    margin: 0 0 10px;
    color: #484848;
}

@media screen and (max-width:768px){
    .wcmp-vendor-modal-main article{
        max-height: initial;
    }
    .wcmp-vendor-modal-main .wcmp-vendor-modal-header+article.pending-vendor-article {
        padding: 10px 0;
    }
    .wcmp-vendor-modal-main footer {
        position: initial;
    }
    .wcmp-vendor-modal-main {
        padding-bottom: 0;
    }
}


/* single vendor review */

.wcmp-ui-tabs {
    border: 1px solid #ccc;
    border-radius: 0;
}

.wcmp-ui-tabs ul li:focus,
.wcmp-ui-tabs ul li a:focus {
    outline: none;
    box-shadow: none;
}

.wcmp-ui-tabs .ui-widget-content {
    border-width: 1px 0px 1px 1px;
}

.wcmp-ui-tabs ul li,
.wcmp-ui-tabs ul li a,
.wcmp-ui-tabs .ui-widget-content,
.wcmp-ui-tabs ul {
    border-color: #ccc !important;
}

.wcmp-ui-tabs li.ui-state-default {
    background: #f1f1f1;
}

.ui-tabs .ui-tabs-nav li.ui-tabs-active {
    margin-bottom: 0;
    padding: 0;
}

.ui-tabs-vertical {
    overflow: hidden;
}

.ui-tabs-vertical * {
    box-sizing: border-box;
}

.ui-tabs-vertical .ui-tabs-panel {
    float: left;
    width: 100%;
    border-radius: 0;
    border: 0;
    min-height: 108px;
}


/* form element */

.wcmp-ui-tabs form fieldset p {
    display: inline-block;
    width: 200px;
    vertical-align: middle;
}


.wcmp-ui-tabs form fieldset p.description {
    display: block;
    margin: -7px 200px 5px 0;
}

.wcmp-ui-tabs table {
    width: 100%;
}

.wcmp-ui-tabs form fieldset textarea{
    width: 25em;
}

.wcmp-ui-tabs table td,
.wcmp-ui-tabs table th {
    padding-bottom: 15px;
    vertical-align: top;
}

.wcmp-ui-tabs table th {
    width: 200px;
    text-align: right;
}

.wcmp-ui-tabs input.form-control {
    width: 25em;
}

.vendor-review-btn.button.button-primary {
    margin: 20px 0 0;
    background: #fe6778;
    border-color: #fe6778;
    box-shadow: 0 1px 0 #d85261;
    text-shadow: none;
    width: auto;
    height: auto;
    padding: 4px 15px;
    font-size: 14px;
}

.vendor-top-info-holder .pull-left {
    width: 150px;
}

.pull-left {
    float: right;
}

.pull-right {
    float: left;
}

.user-social-link {
    padding: 0;
    margin: 0;
    overflow: hidden;
}

.user-social-link li {
    float: right;
    margin: 7px 0 0 10px;
}

.user-social-link li:nth-child(4n+4) {
    margin-left: 0;
}

.vendor-top-info-holder .pull-right {
    width: calc(100% - 185px);
}

@media screen and (max-width:768px){
    .vendor-top-info-holder .pull-right{
        width: 100%;
    }
    .vendor-img-holder img{
        max-width: 100px;
        max-height: 100px;
        margin-bottom: 15px;
    }
}

.vendor-info-row {
    display: inline-block;
    width: 100% !important;
    border-bottom: 1px solid #ddd;
    padding-bottom: 15px;
    margin-bottom: 15px;
}

.vendor-info-row h2 {
    margin-bottom: 9px;
}

.vendor-info-row .dashicons-arrow-right-alt {
    font-size: 14px;
    margin: 1px 0 0;
}

article .vendor-info-row {
    border: 0;
    padding: 0 30px;
}

article .vendor-info-row.payment-info-row {
    background: #e6e6e6;
    padding: 15px 30px;
    position: relative;
    width: 100%;
    margin: 8px 0 20px;
}

.payment-info-row:after,
.payment-info-row:before {
    content: '';
    position: absolute;
    width: 100%;
    height: 1px;
    top: -3px;
    right: 0;
    background: #e6e6e6;
}

.payment-info-row:before {
    bottom: -3px;
    top: auto;
}

.vendor-personal-info-table td {
    padding: 4px 0 2px 20px;
    font-size: 15px;
    color: #484848;
}

body .wcmp-vendor-modal-content article .payment-info-row p {
    float: right;
    font-size: 16px;
    margin: 0;
}

body .wcmp-vendor-modal-content article .payment-info-row p:nth-child(2) {
    float: left;
    margin: 0 12px 0 0;
}

body .wcmp-vendor-modal-content article .payment-info-row p mark {
    margin-right: 7px;
    background: #fff;
    padding: 3px 10px;
    font-size: 13px;
    color: #028ed9;
}

.payment-info-row img {
    vertical-align: middle;
    margin-right: 8px;
    max-width: 100px;
    max-height: 50px;
}

.wcmp-vendor-modal-main footer .button {
    height: 40px;
}

.wcmp-vendor-modal-main footer .button {
    height: 35px !important;
    line-height: 35px !important;
    font-size: 15px;
    padding: 0 20px !important;
}
@media screen and (max-width:480px){
    .wcmp-vendor-modal-main footer .button {
        height: 32px !important;
        line-height: 32px !important;
        padding: 0 10px !important;
        font-size: 13px !important;
    }
    .wcmp-vendor-modal-main .wcmp-vendor-modal-header h1{
        font-size: 18px;
    }
}

body .wcmp-vendor-modal-content article p span {
    vertical-align: middle;
}
body .wcmp-vendor-modal-content article p span.woocommerce-Price-amount {
    vertical-align: baseline;
}
body .wcmp-vendor-modal-content article p span.dashicons{
    vertical-align: sub;
}
body .wcmp-vendor-modal-content article p span.woocommerce-Price-currencySymbol {
    vertical-align: baseline;
}

.vendor-profile p {
    width: 100%;
    margin: 0;
    padding: 10px;
    background: #fff2c7;
    text-align: center;
    font-size: 18px;
}

.vendor-profile p.vendor-profile-complete {
    background: #dcf7e0;
}

#wp-vendor_description-wrap {
    width: calc(100% - 225px);
    display: inline-block;
    vertical-align: top;
    margin: 12px 0;
}

#wp-vendor_shipping_policy-wrap, #wp-vendor_refund_policy-wrap, #wp-vendor_cancellation_policy-wrap {
  width: calc(100% - 225px);
  display: inline-block;
  vertical-align: top;
  margin: 12px 0; }
  
.wcmp-loader{
    margin: 8px 0 0;
    display: inline-block;
    vertical-align: middle;
    -webkit-animation: roundAnimation 2s infinite; /* Safari 4.0 - 8.0 */
    animation: roundAnimation 2s infinite;
}
@-webkit-keyframes roundAnimation {
    0%   {-webkit-transform: rotate(-360deg);transform: rotate(-360deg);}
    100% {-webkit-transform: rotate(0deg);transform: rotate(0deg);}
}

@keyframes roundAnimation {
    0%   {-webkit-transform: rotate(-360deg);transform: rotate(-360deg);}
    100% {-webkit-transform: rotate(0deg);transform: rotate(0deg);}
}


/* social */

.social-sprite {
    background-image: url('../../images/social-sprite.png');
    background-repeat: no-repeat;
    display: inline-block;
    vertical-align: middle;
}

.social-sprite-facebook-icon {
    width: 30px;
    height: 30px;
    background-position: -1px -1px;
}

.social-sprite-googleplus-icon {
    width: 30px;
    height: 30px;
    background-position: -33px -1px;
}

.social-sprite-instagram-icon {
    width: 30px;
    height: 30px;
    background-position: -65px -1px;
}

.social-sprite-linkedin-icon {
    width: 30px;
    height: 30px;
    background-position: -97px -1px;
}

.social-sprite-twitter-icon {
    width: 30px;
    height: 30px;
    background-position: -129px -1px;
}

.social-sprite-youtube-icon {
    width: 30px;
    height: 30px;
    background-position: -161px -1px;
}


/* status sprite */

.status-sprite {
    background-image: url('../../images/vendor-status-sprite.png');
    background-repeat: no-repeat;
    display: inline-block;
    vertical-align: middle;
    margin-left: 8px;
    width: 28px;
    height: 28px;
}

.status-sprite-Approved-icon {
    background-position: -1px -1px;
}

.status-sprite-Pending-icon { 
    background-position: -29px -1px;
}

.status-sprite-Rejected-icon { 
    background-position: -57px -1px;
}

.status-sprite-Suspended-icon { 
    background-position: -85px -1px;
}


/*********************** Extension start ***********************/
.wcmp_page_wcmp-extensions #wpcontent{
    background: #fff;
}
.wcmp-addon-container .addonbox {
    width: 23%;
    float: right;
    top: 5px;
    background: #fff;
    margin-bottom: 30px;
    margin-left: 2.667%;
    min-width: auto;
    box-sizing: border-box;
    box-shadow: 0 0 5px rgba(0, 0, 0, 0.08);
    border: 1px solid #dddddd;
} 
.wcmp-addon-container .addonbox h2 {
    margin: 0;
    padding: 0 25px 0 0;
    min-height: auto;
    font-weight: normal;
    color: #4f4f4f;
    font-size: 17px;
    position: relative;
    height: 61px;
    display: table-cell;
    vertical-align: middle;
    width: 100%;
    line-height: 20px;
}
.wcmp-addon-container .addonbox h2:before{
    content: '';
    position: absolute;
    width: 7px;
    height: 39px;
    right: -1px;
    background: #ff9999;
    top: 0;
    bottom: 0;
    margin: auto;
}
.wcmp-addon-container .addonbox .inside .submitbox #publishing-action {  
    line-height: 23px;
}
.addon-content-holder{
    padding: 20px 25px;
    background: #fff;
}
.wcmp-addon-container div.addon-content-holder p { 
    font-size: 14px;
    line-height: 24px;
    margin: 0 0 20px;
    color: #4f4f4f;
    min-height: 96px;
    overflow: hidden;
}

.addon-content-holder a.button, .offline-addon-wrap a{ 
    text-align: center;
    width: 116px;
    max-width: 100%;
    height: auto;
    border: 1px solid #0494b4;
    background-color: #08a7cb;
    color: #fff;
    font-size: 15px;
    display: inline-block;
    padding: 6px;
    text-decoration: none;
}
.addon-content-holder a.button:hover, .addon-content-holder a.button:focus, .offline-addon-wrap a:focus, .offline-addon-wrap a:hover{ 
    background-color: #0494b4;
    border-color: #0494b4;
    color: #fff;
    box-shadow: none;
    outline: 0;
}

.addon-banner {
    width: 100%;
    background: #826dad;
    position: relative;
    overflow: hidden;
    box-sizing: border-box;
    display: -webkit-box;
    display: flex;
    -webkit-box-pack: justify;
            justify-content: space-between;
    -webkit-box-align: center;
            align-items: center;
    margin: 15px 0;
}

.addon-img-holder {
    height: 223px;
    overflow: hidden;
    padding: 0;
    text-align: center;
    background: #ededed url('../../images/extension-bkg.png') no-repeat center center;
    margin: 0 -1px;
}

.addon-img-holder img {
    max-width: 90%;
    height: auto;
    max-height: 160px;
    position: relative;
    top: 50%;
    transform: translateY(-50%);
    -moz-transform: translateY(-50%);
    -ms-transform: translateY(-50%);
    -webkit-transform: translateY(-50%);
}

.addon-banner-content {
    width: 500px;
    max-width: 100%;
    margin-left: 44px;
    padding: 0 35px;
}

.addon-banner-content h1,
.addon-banner-content p {
    color: #fff;
}

.addon-banner-content a {
    display: inline-block;
    background: #fff;
    color: #000;
    padding: 7px 20px;
    text-decoration: none;
}

.wcmp-addon-container .addonbox:nth-child(4n+1) {
    clear: both;
}

.wcmp-addon-container .addonbox:nth-child(4n+4) {
    margin: 0;
}
.offline-addon-wrap{
    background: #ececec;
    padding: 25px;
    margin-top: 25px;
    overflow: hidden;
}
.offline-addon-wrap .addon-content{
    float: right;
}
.offline-addon-wrap .addon-content h2, .offline-addon-wrap .addon-content p{
    margin: 0;
    padding: 0;
}
.offline-addon-wrap .addon-content p{
    margin-top: 5px;
}
.offline-addon-wrap a{
    float: left;
    margin-top: 5px;
}

@media screen and (max-width: 1100px) {
    .offline-addon-wrap{
        text-align: right;
    }
    .offline-addon-wrap .addon-content{
        width: 100%;
        margin-bottom: 10px;
    }
    .offline-addon-wrap a{
        float: right;
    }
}
@media screen and (max-width: 1800px) {
    .wcmp-addon-container .addonbox{
        width: 30%;
        margin-left: 5%;
    }
    .wcmp-addon-container .addonbox:nth-child(4n+4) {
        margin-left: 5%;
    }
    .wcmp-addon-container .addonbox:nth-child(3n+3) {
        margin-left: 0;
    }
    .wcmp-addon-container .addonbox:nth-child(4n+1) {
        clear: none;
    }
    .wcmp-addon-container .addonbox:nth-child(3n+1) {
        clear: both;
    }
}
@media screen and (max-width: 1400px) {
    .wcmp-addon-container .addonbox{
        width: 32%;
        margin-left: 2%;
    }
    .wcmp-addon-container .addonbox:nth-child(4n+4) {
        margin-left: 2%;
    }
    .wcmp-addon-container .addonbox:nth-child(3n+3) {
        margin-left: 0;
    }
    .wcmp-addon-container .addonbox:nth-child(4n+1) {
        clear: none;
    }
    .wcmp-addon-container .addonbox:nth-child(3n+1) {
        clear: both;
    }
}
@media all and (max-width: 1100px) and (min-width: 1024px) {
    .addon-banner-content {
        margin-left: 0;
    }
}
@media (max-width: 1023px) {
    .addon-banner-content {
        width: 100%;
        margin-left: 0;
        padding: 30px 35px;
        background: rgba(0, 0, 0, 0.5);
        position: relative;
    }
    .addon-banner img {
        position: absolute;
        top: 0;
    }
}

@media (max-width: 460px) {
    .addon-banner-content {
        padding: 10px 30px;
    }
    .addon-banner-content h1 {
        font-size: 16px;
        line-height: 22px;
        padding-bottom: 0;
    }
    .addon-banner-content p {
        margin: 10px 0;
    }
}

@media (max-width: 1000px) {
    .wcmp-addon-container .addonbox {
        width: 48%;
        margin-left: 4%;
    }
    .wcmp-addon-container .addonbox:nth-child(3n+1), .wcmp-addon-container .addonbox:nth-child(3n+1), .wcmp-addon-container .addonbox:nth-child(2n+2){
        clear: none;
    }
    .wcmp-addon-container .addonbox:nth-child(2n+1) {
        clear: both;
        margin-left: 4%;
    }
    .wcmp-addon-container .addonbox:nth-child(2n+2) {
        margin: 0;
    }
}

@media (max-width: 640px) {
    .wcmp-addon-container .addonbox {
        width: 100%;
        margin: 0 auto 30px !important;
        width: 370px;
        max-width: 100%;
        float: none;
    }
    .wcmp-addon-container .addonbox .hndle {
        min-height: 37px;
    }
    .wcmp-addon-container .addonbox h2{
        margin: 15px;
        font-size: 15px;
        height: 55px;
        padding: 0 15px 0 0;
    }
    .wcmp-addon-container .addonbox h2:before{
        width: 4px;
        height: 32px;
    }
    .addon-content-holder {
        background: #fff;
    }
    
    .wcmp-addon-container div.addon-content-holder p {
        min-height: auto;
        height: auto;
    }
}


/*********************** Extension End ***********************/


/*********************** Todo List Start ***********************/

.wcmp_page_wcmp-to-do .wcmp_vendors_settings #submit {
    display: none;
}

.wcmp_page_wcmp-to-do .wcmp_vendors_settings .form-table th {
    padding-bottom: 17px !important;
    padding-top: 17px !important;
}

.wcmp_page_wcmp-to-do .wcmp_vendors_settings .form-table {
    box-shadow: 0 3px 0 #dbdbdb;
    border-color: #d2d2d2;
    position: relative;
}

.wcmp_page_wcmp-to-do .wcmp_vendors_settings .form-table a {
    color: #e67853;
    text-decoration: none;
}

.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr:last-child {
    border: none;
}

.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr:first-child {
    border-bottom: solid 1px #d2d2d2;
    border-top: solid 1px #d2d2d2;
}

.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr td,
.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr th {
    padding-right: 20px;
}

.wcmp_page_wcmp-to-do .vendor_edit_button,
.wcmp_page_wcmp-to-do .activate_vendor,
.wcmp_page_wcmp-to-do .reject_vendor,
.wcmp_page_wcmp-to-do .vendor_dismiss_button,
.wcmp_page_wcmp-to-do .vendor_transaction_done_button {
    width: 35px;
    height: 35px;
    border-radius: 5px;
    border-top: solid 1px #d6c9c6;
    border-right: solid 1px #d6c9c6;
    border-left: solid 2px #d6c9c6;
    border-bottom: solid 3px #d6c9c6;
    text-indent: -99999px;
}

.wcmp_page_wcmp-to-do .vendor_edit_button:hover,
.wcmp_page_wcmp-to-do .activate_vendor:hover,
.wcmp_page_wcmp-to-do .reject_vendor:hover,
.wcmp_page_wcmp-to-do .vendor_dismiss_button:hover,
.wcmp_page_wcmp-to-do .vendor_transaction_done_button:hover {
    cursor: pointer;
    border-color: #beaba6;
}

.wcmp_page_wcmp-to-do .vendor_edit_button {
    background: url(../../images/icon-vendor-edit.png ) -2px -2px;
}

.wcmp_page_wcmp-to-do .activate_vendor {
    background: url(../../images/icon-vendor-activate.png) -2px -2px;
}

.wcmp_page_wcmp-to-do .reject_vendor {
    background: url(../../images/icon-vendor-reject.png) -2px -2px;
}

.wcmp_page_wcmp-to-do .vendor_dismiss_button {
    background: url(../../images/icon-vendor-dismiss.png) -2px -2px;
}

.wcmp_page_wcmp-to-do .vendor_transaction_done_button {
    background: url(../../images/icon-vendor-notify.png) -2px -2px;
}

/*********************** Todo List End ***********************/

/*********************** Vendor registration Start ***********************/

#menu-settings-column .button-controls {
    clear: none;
    padding: 0;
    width: 100%;
}

#menu-settings-column .button-controls a {
    width: 100%;
    text-align: center;
}

.menu-item-handle {
    background: #fff none repeat scroll 100% 0;
    padding: 10px 10px 10px 5px;
}

#post-body-content .menu-item-handle {
    width: 510px;
    max-width: 100%;
}
.wcmp_vendor_registration_settings_group #post-body-content{
    width:100%;
    min-width: auto;
}
#wcmp-vendor-form .postbox{
    width: 100%;
    min-width: 100%;
}

.menu-item-bar .menu-item-handle {
    line-height: 20px !important;
}

.wcmp-vendor-form-field-content {
    background: #fff none repeat scroll 100% 0;
    padding: 3px;
    width: 80%;
}

.wcmp-vendor-form-input-field {
    width: 100%;
}

.wcmp-vendor-form-input-field-container {
    padding: 5px;
}

.wcmp-remove-form-field {
    padding-right: 2px;
    color: red;
}

.wcmp-vendor-form-input-field-container ul li a {
    display: inline-block;
    line-height: 18px;
    text-decoration: none !important;
}

.field-selectbox-options {
    padding-top: 10px;
}

.field-selectbox-options li {
    background: #fcfcfc none repeat scroll 100% 0;
    border-bottom: 1px solid #ddd;
    padding: 2px 10px;
    cursor: move;
}

.wcmp-form-empty-container {
    border: 3px dashed #bbb;
    color: #ccc;
    font-size: 20px;
    margin: 20px 0;
    padding: 100px 10px;
    text-align: center;
    width: 90%;
}

.wcmp-form-field {
    border-bottom: 1px solid #f6f6f6;
    padding: 5px;
    width: 100%;
}

.wcmp-form-field>label {
    color: #666;
    display: inline-block;
    font-size: 13px;
    width: 20%;
}

.wcmp-form-field>span {
    color: #222;
    width: 79%;
    display: inline-block;
    vertical-align: top;
}
.wcmp-form-field h2{
    margin: 12px 0 23px;
}
.vendor-quick-view-notes h2{
    margin: 12px 0 23px;
}
.vendor-quick-view-notes .note-clm{
    max-width: 75%;
}
.form-control {
    width: 100%;
}
.pending-vendor-note {
    margin: 25px 0 0;
}
footer .pending-vendor-note {
    margin: 0 0 8px;
}
.note-clm-wrap{
    padding-right: 30px;
    position: relative;
}
.note-clm-wrap:after{
    content: '';
    width: 1px;
    height: 100%;
    background: #ccc;
    position: absolute;
    top: 0;
    right: 0;
}
.note-clm-wrap .note-clm:last-of-type{
    margin-bottom: 0;
}
.note-clm{
    display: block;
    padding: 14px 17px;
    max-width: 60%;
    margin: 0 0 20px 5%;
    border: 1px solid #ccc;
    position: relative;
    border-radius: 6px;
}
@media screen and (max-width:767px){
    .note-clm, .vendor-quick-view-notes .note-clm{
        max-width: 100%;
        margin-left: 0;
    }
    .wcmp-form-field>label{
        width: 100%;
    }
}
.note-clm:before {
    width: 13px;
    height: 13px;
    border: 2px solid #fff;
    border-radius: 100px;
    margin: 16px -4px 0 0;
    box-shadow: 0 1px 1px rgba(0,0,0,0.2), inset 0 1px 1px rgba(0,0,0,0.1);
    background: #fd5869;
    content: "";
    position: absolute;
    top: 0;
    right: -33px;
    z-index: 1;
    box-sizing: border-box;
}
.note-clm:after {
    width: 9px;
    height: 9px;
    border-width: 0 1px 1px 0;
    border-style: solid;
    border-color: #ccc;
    background: #fff;
    -webkit-transform: rotate(-45deg);
    -ms-transform: rotate(-45deg);
    transform: rotate(-45deg);
    content: "";
    position: absolute;
    top: 18px;
    right: -6px;
}
.note-description{
    margin: 0 0 21px;
}
.note-clm p.note-meta {
    margin: 0;
    display: inline-block;
    color: #969696;
}
.note-clm p.note-meta.note_owner{
    margin-right: 10px;
}
.wcmp-form-field .note-clm{
    max-width: 70%;
}
textarea.form-control {
    height: 80px;
}

#vendor_preview_tabs #vendor_profile_image_display {
    vertical-align: top;
}


/*********************** Vendor registration End ***********************/


/*********************** Report Start ***********************/

.box_data {
    padding: 10px;
}

.pad_left {
    padding-right: 10px;
}

.bars span {
    height: 15px !important;
}

.bar1,
.bar2,
.bar3 {
    width: 30px;
    height: 15px;
    float: right;
}

.bar1 {
    background-color: #ef6448;
    border-radius: 3px;
}

.bar2 {
    background-color: #1dbb5c;
    border-radius: 3px;
}
.bar3 {
    background-color: #ddd30c;
    border-radius: 3px;
}

.bar_indecator span {
    float: right;
    margin-right: 10px;
    margin-left: 45px;
}

.bar_indecator {
    float: right;
    margin-top: 16px;
    margin-right: 100px;
}

.sort_chart h4 {
    float: right;
}

.sorting_box {
    float: left;
    margin-top: 7px;
}

.bars span.alt {
    margin-top: 14px !important;
}

.bars span.main {
    margin-top: 2px !important;
}

.vendor_report_search,
.product_report_search {
    cursor: pointer;
}

.sorting_box input {
    display: none;
}

.low_to_high_btn_vendor,
.low_to_high_btn_product {
    background-color: #FAF8F9;
    width: 29px;
    height: 29px;
    border-radius: 5px;
    border-top: solid 1px #d6c9c6;
    border-right: solid 1px #d6c9c6;
    border-left: solid 2px #d6c9c6;
    border-bottom: solid 3px #d6c9c6;
}

.high_to_low_btn_vendor,
.high_to_low_btn_product {
    background-color: #ECEAED;
    width: 29px;
    height: 29px;
    border-radius: 5px;
    border-top: solid 1px #B2B0B1;
    border-right: solid 1px #B2B0B1;
    border-left: solid 1px #B2B0B1;
    border-bottom: solid 1px #B2B0B1;
}

.low_to_high_btn_vendor:hover,
.low_to_high_btn_product:hover,
.high_to_low_btn_vendor:hover,
.high_to_low_btn_product:hover {
    cursor: pointer;
}

.bars span {
    background-color: #ef6448 !important;
}

.bars span.alt {
    background-color: #1dbb5c !important;
}
.bars span.vendor_bar {
    background-color: #ddd30c !important;
}

i.dashicons-arrow-up-alt {
    color: #AF938F;
    font-size: 16px;
}

i.dashicons-arrow-down-alt {
    color: #AF9692;
    font-size: 16px;
    font-weight: 100;
}


/*********************** Report End ***********************/

/*********************** WCMp Settings Tools ***********************/
table.wcmp_tools_table.widefat { border: none; }
table.wcmp_tools_table.widefat td, table.wcmp_tools_table.widefat th { padding: 2em; }
table.wcmp_tools_table td strong.name, table.wcmp_tools_table th strong.name {
    display: block;
    margin-bottom: .5em;
}
table.wcmp_tools_table td.run-tool, table.wcmp_tools_table th.run-tool {
    text-align: left;
}
/***********************  Shop order Suborder Icon ***********************/
.widefat .column-wcmp_suborder mark{
    display: block;
    text-indent: -9999px;
    position: relative;
    height: 1em;
    width: 1em;
    background: 100% 0;
    font-size: 1.4em;
    margin: 0 auto
}
.widefat .column-wcmp_suborder mark:after{
    font-family: WooCommerce;
    speak: none;
    font-weight: 400;
    font-variant: normal;
    text-transform: none;
    line-height: 1;
    -webkit-font-smoothing: antialiased;
    margin: 0;
    text-indent: 0;
    position: absolute;
    top: 0;
    right: 0;
    width: 100%;
    height: 100%;
    text-align: center
}

.widefat .column-wcmp_suborder mark {
    float: right;
    margin-left: 8px;
    font-size: 1.1em;
    margin-top: 2px;
}

/* Suborder Icon */

.widefat .column-wcmp_suborder mark.pending:after{
    content: "\e012";
    color: #ffba00
}

.widefat .column-wcmp_suborder mark.trash:after{
    content: "\e602";
    color: #a00
}

.widefat .column-wcmp_suborder mark.completed:after{
    content: "\e015";
    color: #2ea2cc
}

.widefat .column-wcmp_suborder mark.on-hold:after{
    content: "\e033";
    color: #999
}

.widefat .column-wcmp_suborder mark.failed:after{
    content: "\e016";
    color: #d0c21f
}

.widefat .column-wcmp_suborder mark.cancelled:after{
    content: "\e013";
    color: #a00
}

.widefat .column-wcmp_suborder mark.processing:after{
    content: "\e011";
    color: #73a724
}

.widefat .column-wcmp_suborder mark.refunded:after {
    content: "\e014";
    color: #999
}

#wcmp-commission-data button.handlediv, #wcmp-commission-data .hndle{ display: none;}
/*#wcmp-commission-data{ padding: 23px 24px 12px; }*/
#wcmp-commission-data .inside{ padding: 0; margin: 0;}
#wcmp-commission-data h2 {
    margin: 0;
    font-family: HelveticaNeue-Light,'Helvetica Neue Light','Helvetica Neue',sans-serif;
    font-size: 21px;
    font-weight: 400;
    line-height: 1.2;
    text-shadow: -1px 1px 1px #fff;
    padding: 0;
}

.wcmp-commission-amount a.edit_commission_amount {
    width: 14px;
    height: 0;
    padding: 14px 0 0;
    margin: 0 6px 0 0;
    overflow: hidden;
    position: relative;
    color: #999;
    border: 0;
    float: left;
}

.wcmp-commission-amount a.edit_commission_amount:after {
    font-family: Dashicons;
    content: '\f464';
    position: absolute;
    top: 0;
    right: 0;
    text-align: center;
    vertical-align: top;
    line-height: 14px;
    font-size: 14px;
    font-weight: 400;
},img{max-width:100%}.full-width{width:100%}#dc_admin_footer{bottom:24px;text-align:center;position:absolute;width:100%;right:0;left:0}#dc_admin_footer a{text-decoration:none}#dc_admin_footer img{border:none;margin-left:5px;vertical-align:bottom}.required{color:red}.img_tip{background:url(../../images/question_mark.png) no-repeat scroll 100% 0 rgba(0,0,0,0);display:inline-block;height:16px;vertical-align:-1px;width:16px;cursor:pointer}.multi_input_block{border:1px solid #dfdfdf;border-radius:10px;padding:10px;margin-bottom:10px}.multi_input_block_manupulate{float:left;margin:2px!important}#choose_vendor_ajax_chosen{width:90%!important}#choose_vendor_ajax_chosen .chosen-results{padding:3px;clear:both}#vendor-label{float:none;margin:0;padding:0;width:52px}label[for=block_vendor_desc],label[for=catalog_colorpicker],label[for=catalog_hover_colorpicker],label[for=commission_include_coupon],label[for=commission_type],label[for=default_commission],label[for=give_shipping],label[for=give_tax],label[for=sold_by_cart_and_checkout],label[for=sold_by_catalog],label[for=sold_by_text]{text-decoration:line-through}#block_vendor_desc,#catalog_colorpicker,#catalog_hover_colorpicker,#commission_include_coupon,#commission_type,#default_commission,#give_shipping,#give_tax,#sold_by_cart_and_checkout,#sold_by_catalog,#sold_by_text,#sold_by_texttt{display:none}.wcmp_vendors_settings .form-table tbody tr td{padding:0;padding-top:10px;padding-bottom:10px}.wcmp_vendors_settings .form-table td p{margin-bottom:10px!important}.wcmp_vendors_settings #to_do_list tbody tr td,.wcmp_vendors_settings #to_do_list tbody tr th{padding-right:10px}.wcmp_vendors_settings #to_do_list tbody tr th{background:#d8d8d8}.wcmp_vendors_settings #to_do_list tbody tr{border-bottom:2px solid #f7f5f5}.wcmp_vendors_settings #to_do_list{border-radius:5px}span.vendor_bulk div.chosen-container{width:50%!important}span.variable_commission_cls{font-size:15px}.wcmpsubtabadmin{margin:8px 0 0!important}a.wcmp_sub_sction{border-left:1px solid #999!important;display:inline-block;line-height:14px;padding:0 8px}.wcmpsubtabadmin li:last-child a.wcmp_sub_sction{border-left:0 none!important}.inline-edit-col span.inline-edit-categories-label:nth-of-type(2),.inline-edit-col ul.product_shipping_class-checklist{display:none!important}#wcmp_auto_suggest_product_title{box-shadow:0 0 3px 1px rgba(0,0,0,.3);margin-top:-16px;margin-bottom:16px}#wcmp_auto_suggest_product_title>ul{border:1px solid #aaa;list-style:outside none none;max-height:140px;overflow-y:auto;padding:10px}#wcmp_auto_suggest_product_title li a{border-bottom:1px solid #ddd;color:#727272;display:inline-block;text-decoration:none;width:100%}.wcmp_vendors_settings select{width:25em}.wcmp_vendors_settings fieldset{position:relative;padding-right:25px}.wcmp_vendors_settings fieldset .hint-container{position:absolute;right:0;margin:auto!important;top:5px}@media screen and (max-width:782px){.wcmp_vendors_settings fieldset .hint-container{position:relative;top:-5px}.wcmp_vendors_settings fieldset{padding-right:0}}.wcmp_template_list{margin:0 0 20px 0;position:relative}.wcmp_template_list .dashicons{position:absolute;background:#151715;color:#fff;line-height:34px;top:0;width:35px;left:0;height:35px}.wcmp_template_list input[type=radio]:checked+.dashicons{background:#0073aa}.wcmp_template_list input[type=radio]:checked+.dashicons:before{content:'\f160'}.wcmp_subtab_container{border:solid 1px #ccc;box-sizing:border-box;overflow:hidden}.vendor-preview-form,.wcmp_subtab_content{background:#fff;float:right;width:100%;padding:25px;box-sizing:border-box}.wcmp-ui-tabs h2,.wcmp_subtab_content h2{border-bottom:1px solid #ccc;padding-top:30px}.wcmp-ui-tabs h2:first-of-type,.wcmp_subtab_content h2:first-of-type{padding-top:0;margin-top:0}.subsubsub.wcmpsubtabadmin li a,.wcmp-ui-tabs .ui-tabs-nav li a{padding:10px;display:block;font-size:13px;font-weight:600;border-color:#ccc!important;border-width:0 0 1px 1px!important;border-style:solid;border-bottom:0;color:#7f8591;text-decoration:none}.subsubsub.wcmpsubtabadmin li a .dashicons,.wcmp-ui-tabs li .dashicons{vertical-align:middle;margin-left:7px}.subsubsub.wcmpsubtabadmin li a.current,.subsubsub.wcmpsubtabadmin li a:hover,.wcmp-ui-tabs .ui-tabs-nav li a:hover,.wcmp-ui-tabs .ui-tabs-nav li.ui-tabs-active a{background:#fff;color:#333}.subsubsub.wcmpsubtabadmin li a.current .dashicons,.subsubsub.wcmpsubtabadmin li a:hover .dashicons,.wcmp-ui-tabs li a:hover .dashicons,.wcmp-ui-tabs li.ui-tabs-active .dashicons{color:#f86a7c}.wcmp_subtab_container .wcmpsubtabadmin li a{white-space:normal}#vendor_preview_tabs{padding:0}#vendor_preview_tabs .ui-tabs-nav{padding:0;border:0}#vendor_preview_tabs .ui-tabs-nav li{float:none!important;white-space:initial;clear:both;border:0}#vendor_preview_tabs .ui-tabs-nav li a{float:none!important}#vendor_preview_tabs .ui-widget-header{border:1px solid #e78f08;background:#f1f1f1}@media screen and (min-width:768px){.subsubsub.wcmpsubtabadmin li a,.wcmp-ui-tabs .ui-tabs-nav li a{padding:20px 15px!important}}@media screen and (min-width:768px){.wcmp-ui-tabs ul.ui-tabs-nav,.wcmp_subtab_container .subsubsub.wcmpsubtabadmin{margin:0 0 0 -1px!important;width:200px;position:relative;float:right}.subsubsub.wcmpsubtabadmin li,.ui-tabs-vertical .ui-tabs-nav li{display:block;padding:0;margin:0}.subsubsub.wcmpsubtabadmin li a,.wcmp-ui-tabs .ui-tabs-nav li a{padding:20px 15px;display:block;font-size:14px;border-color:#ccc!important;border-width:0 0 1px 1px!important;border-style:solid}.vendor-preview-form,.wcmp_subtab_content{width:calc(100% - 200px);border-right:1px solid #ccc}#vendor_preview_tabs{display:-webkit-box;display:flex}.wcmpsubtabadmin li:last-child a.wcmp_sub_sction{border-left:1px solid #ccc!important}}.subsubsub.wcmpsubtabadmin li a:focus{box-shadow:none}h4.wcmp-tab-description{padding:12px 20px;margin:25px 0;border-style:solid;border-width:0 4px 0 0;border-color:#ffb900;background-color:#fff8e5}h4.wcmp-tab-description.success{border-color:#ffb900;background-color:#fff8e5}.wcmp_tab_content{background:#fff;padding:0 20px;border:solid 1px #ccc}.wcmp_vendors_settings table{margin:0}.nav-tab-active,.nav-tab-active:active,.nav-tab-active:focus,.nav-tab-active:hover{background:#fff}.nav-tab-wrapper a.nav-tab:focus{box-shadow:none}.wcmp_vendors_settings #poststuff{min-width:auto}.wcmp_subtab_container .form-table tr:first-of-type td,.wcmp_subtab_container .form-table tr:first-of-type th{padding-top:0}.wcmp-ui-tabs h2,.wcmp_vendors_settings h2{color:#0073aa;padding-bottom:10px}.wcmp_vendors_settings h2+.wcmp-section-info{margin:-11px 0 30px 0}.wcmp_page_wcmp-setting-admin .nav-tab-wrapper a.nav-tab:first-of-type{margin-right:0}.wcmp-settings-wrap h2.nav-tab-wrapper{background:#8abee5;margin:40px 0 25px;padding:0;border:0}.wcmp-settings-wrap h2.nav-tab-wrapper a{margin:0;border:0;background:100% 0;padding:13px 35px;color:#fff;-webkit-transition:color .3s;transition:color .3s;position:relative}.wcmp-settings-wrap h2.nav-tab-wrapper a:after{content:'';position:absolute;right:0;left:0;bottom:10px;width:0;height:2px;background:#fff;margin:auto;-webkit-transition:all .3s;transition:all .3s}.wcmp-settings-wrap h2.nav-tab-wrapper a:hover:after{width:25px}.wcmp-settings-wrap h2.nav-tab-wrapper a:hover{color:#fff}.wcmp-settings-wrap h2.nav-tab-wrapper a.nav-tab-active{background:#fff;color:#8abee5}.wcmp-settings-wrap h2.nav-tab-wrapper a dashicons{color:#aff1b6}.wcmp-settings-wrap h2.nav-tab-wrapper a.nav-tab-active dashicons{color:#24b47e}.wcmp-primary-btn.button-primary,.wp-core-ui .wrap.wcmp-settings-wrap .button-primary,.wp-core-ui .wrap.wcmp-settings-wrap .button.button-primary{background:#fe6778;border-color:#fe6778;box-shadow:0 1px 0 #d85261;text-shadow:none;width:auto;height:auto;padding:4px 15px;font-size:14px}.wcmp-primary-btn.button-primary:focus,.wcmp-primary-btn.button-primary:hover,.wp-core-ui .wrap.wcmp-settings-wrap .button.button-primary:hover{background:#d85261;border-color:#d85261;box-shadow:0 1px 0 #d85261}.wcmp-action-button.pull-right{background:#696364;border-color:#696364;box-shadow:0 1px 0 #696364;text-shadow:none;width:auto;height:auto;padding:4px 15px;font-size:14px}.wcmp-action-button.pull-right:focus,.wcmp-action-button.pull-right:hover{background:#4c4848;border-color:#4c4848}.wcmp-settings-wrap .form-table th{padding:10px 0 0 10px}.wcmp-settings-wrap .wcmp_payment_help{left:23px}.vendor-preview-form #wc-backbone-modal-dialog{margin-top:25px}@media screen and (max-width:1006px){.vendor-preview-form #display_name,.vendor-preview-form .regular-text,body #wp-vendor_description-wrap,body .wcmp-ui-tabs form fieldset textarea{width:100%}body .wcmp-ui-tabs form fieldset p.description{margin:0}.dc-wp-fields-uploader{display:block}}@media screen and (max-width:1170px){.wcmp-settings-wrap .wcmp_payment_help,form.wcmp_vendors_settings .frontend_manager_promo{left:auto;position:relative;padding-right:0;padding-top:0}}#wcmp_dashboard_site_logo,#wcmp_dashboard_site_logo_button,#wcmp_dashboard_site_logo_display{vertical-align:middle!important}#vendor_color_scheme_picker .color-option{width:42%;padding:0;margin:-12px 0 5px 8%}#vendor_color_scheme_picker .color-option .color-palette td{padding-bottom:0}@media screen and (max-width:480px){#vendor_color_scheme_picker .color-option{width:100%;margin:0 0 5px}}#vendor_color_scheme_picker .color-option>label{padding:10px;margin:0!important;display:block}#vendor_color_scheme_picker .color-option:last-of-type{margin:0}#woocommerce-coupon-data ul.wc-tabs li.vendor_icons a::before,#woocommerce-product-data ul.wc-tabs li.vendor_icons a::before,.woocommerce ul.wc-tabs li.vendor_icons a::before{font-family:Dashicons;content:"\f338"}#woocommerce-coupon-data ul.wc-tabs li.policy_icons a::before,#woocommerce-product-data ul.wc-tabs li.policy_icons a::before,.woocommerce ul.wc-tabs li.policy_icons a::before{font-family:Dashicons;content:"\f497"}.wp-list-table.vendors th#username{width:250px}.wp-list-table.vendors td{padding:8px 10px}.wp-list-table.bordered{margin-top:5px}.wp-list-table.bordered,.wp-list-table.bordered td,.wp-list-table.bordered th{border:1px solid #e6e6e6;border-collapse:collapse;text-align:center}.pending-vendor-clm{display:inline-block;width:calc(100% - 80px)}.vendor-preview{float:left;width:16px;padding:20px 4px 4px 4px;height:0;overflow:hidden;position:relative;border:2px solid transparent;margin:6px 0 0 6px}@media screen and (max-width:782px){.vendor-preview{margin:0 0 0 37px}}.vendor-preview::before{font-family:WooCommerce;speak:none;font-weight:400;text-transform:none;text-indent:0;position:absolute;right:0;width:100%;height:100%;text-align:center;content:"";line-height:16px;font-size:14px;vertical-align:middle;top:4px;font-variant:normal normal;margin:0}.vendor-status{display:inline-block;font-size:17px;padding:8px 10px;border-radius:3px}.vendor-status.pending-vendor{background:#f8dda7;color:#94660c}.vendor-status.approved-vendor{background:#c8d7e1;color:#2e4453}.vendor-status.rejected-vendor{background:#e86767;color:#420c0c}.vendor-status.suspended-vendor{background:#6d6d6d;color:#191919}body .wcmp-vendor-modal-content article p{margin:0 0 10px}.shop-link{float:left}.wcmp-vendor-modal *{box-sizing:border-box}.wcmp-vendor-modal .wcmp-vendor-modal-content{position:fixed;background:#fff;z-index:100000;right:50%;top:50%;-webkit-transform:translate(50%,-50%);-ms-transform:translate(50%,-50%);transform:translate(50%,-50%);width:590px;max-width:100%;max-height:90%;overflow:auto}.wcmp-vendor-modal-content a{text-decoration:none}.wcmp-vendor-modal .wcmp-vendor-modal-content article{overflow:auto}.wcmp-vendor-modal.wcmp-vendor-modal-shipping-method-settings .wcmp-vendor-modal-content{width:75%;min-width:500px}.wcmp-vendor-modal .select2-container{width:100%!important}@media screen and (max-width:782px){.wcmp-vendor-modal .wcmp-vendor-modal-content{width:100%;min-width:100%}}.wcmp-vendor-modal-backdrop{position:fixed;top:0;right:0;left:0;bottom:0;min-height:360px;background:#000;opacity:.7;z-index:99900}.wcmp-vendor-modal-main{padding-bottom:55px;position:relative}.wcmp-vendor-modal-main article,.wcmp-vendor-modal-main header{display:block;position:relative}.wcmp-vendor-modal-main .wcmp-vendor-modal-header{height:auto;background:#fcfcfc;padding:20px;border-bottom:1px solid #ddd;position:relative}.wcmp-vendor-modal-main .wcmp-vendor-modal-header+article{padding:30px 0 10px}.wcmp-vendor-modal-main .wcmp-vendor-modal-header+article.pending-vendor-article{padding:30px 0 110px}.wcmp-vendor-modal-main .wcmp-vendor-modal-header h1{margin:0;font-size:21px;font-weight:700;line-height:20px;display:inline-block}.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link{cursor:pointer;color:#777;height:100%;width:64px;padding:0;position:absolute;top:0;left:0;text-align:center;border:0;border-right:1px solid #ddd;background-color:transparent;-webkit-transition:color .1s ease-in-out,background .1s ease-in-out;transition:color .1s ease-in-out,background .1s ease-in-out}.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link::before{font:normal 40px/50px dashicons!important;color:#666;display:block;content:'\f335';font-weight:300}.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link:focus,.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link:hover{background:#ddd;border-color:#ccc;color:#000}.wcmp-vendor-modal-main .wcmp-vendor-modal-header .modal-close-link:focus{outline:0}.wcmp-vendor-modal-header h1 span{background:#028ed9;padding:2px 10px;font-size:16px;border-radius:3px;margin-right:5px;color:#fff}.wcmp-vendor-modal-header h1 span i{font-size:14px;vertical-align:middle;margin-right:2px}.wcmp-vendor-modal-main article{padding:1.5em;max-height:54vh}.wcmp-vendor-modal-main article p{margin:1.5em 0}.wcmp-vendor-modal-main article p:first-child{margin-top:0}.wcmp-vendor-modal-main article p:last-child{margin-bottom:0}.wcmp-vendor-modal-main article .pagination{padding:10px 0 0;text-align:center}.wcmp-vendor-modal-main footer{position:absolute;right:0;left:0;bottom:0;z-index:100;padding:15px 25px;background:#fcfcfc;border-top:1px solid #dfdfdf;box-shadow:0 -4px 4px -4px rgba(0,0,0,.1)}.wcmp-vendor-modal-main footer .inner{text-align:left;line-height:23px}.wcmp-vendor-modal-main footer .inner .button{margin-bottom:0}.wcmp-vendor-modal-main footer .inner .button.wcmp-action-button{box-shadow:none;margin-left:5px;text-shadow:none}.wcmp-vendor-modal-main footer .inner .button.pending-vendor{border-color:#eed093;background:#f8dda7;color:#94660c}.wcmp-vendor-modal-main footer .inner .button.activate-vendor,.wcmp-vendor-modal-main footer .inner .button.approve-vendor{border-color:#b8ccda;background:#c8d7e1;color:#2e4453}.wcmp-vendor-modal-main footer .inner .button.reject-vendor{border-color:#eb6363;background:#e86767;color:#420c0c}.wcmp-vendor-modal-main footer .inner .button.suspend-vendor{border-color:#5f5d5d;background:#6d6d6d;color:#191919}.wcmp-vendor-modal h2{margin:0}.wcmp-vendor-modal form{margin-top:20px}.wcmp-vendor-modal form textarea{width:100%;height:80px}.wcmp-vendor-modal .vendor-status-header{margin:1px 0 0 70px;float:left;font-size:16px}@media screen and (max-width:480px){.wcmp-vendor-modal .vendor-status-header span{display:none}.wcmp-vendor-modal .vendor-status-header{margin:1px 0 0 45px}}.wcmp-vendor-modal-main article .vendor-info-row h3{margin:0 0 10px;color:#484848}@media screen and (max-width:768px){.wcmp-vendor-modal-main article{max-height:initial}.wcmp-vendor-modal-main .wcmp-vendor-modal-header+article.pending-vendor-article{padding:10px 0}.wcmp-vendor-modal-main footer{position:initial}.wcmp-vendor-modal-main{padding-bottom:0}}.wcmp-ui-tabs{border:1px solid #ccc;border-radius:0}.wcmp-ui-tabs ul li a:focus,.wcmp-ui-tabs ul li:focus{outline:0;box-shadow:none}.wcmp-ui-tabs .ui-widget-content{border-width:1px 0 1px 1px}.wcmp-ui-tabs .ui-widget-content,.wcmp-ui-tabs ul,.wcmp-ui-tabs ul li,.wcmp-ui-tabs ul li a{border-color:#ccc!important}.wcmp-ui-tabs li.ui-state-default{background:#f1f1f1}.ui-tabs .ui-tabs-nav li.ui-tabs-active{margin-bottom:0;padding:0}.ui-tabs-vertical{overflow:hidden}.ui-tabs-vertical *{box-sizing:border-box}.ui-tabs-vertical .ui-tabs-panel{float:left;width:100%;border-radius:0;border:0;min-height:108px}.wcmp-ui-tabs form fieldset p{display:inline-block;width:200px;vertical-align:middle}.wcmp-ui-tabs form fieldset p.description{display:block;margin:-7px 200px 5px 0}.wcmp-ui-tabs table{width:100%}.wcmp-ui-tabs form fieldset textarea{width:25em}.wcmp-ui-tabs table td,.wcmp-ui-tabs table th{padding-bottom:15px;vertical-align:top}.wcmp-ui-tabs table th{width:200px;text-align:right}.wcmp-ui-tabs input.form-control{width:25em}.vendor-review-btn.button.button-primary{margin:20px 0 0;background:#fe6778;border-color:#fe6778;box-shadow:0 1px 0 #d85261;text-shadow:none;width:auto;height:auto;padding:4px 15px;font-size:14px}.vendor-top-info-holder .pull-left{width:150px}.pull-left{float:right}.pull-right{float:left}.user-social-link{padding:0;margin:0;overflow:hidden}.user-social-link li{float:right;margin:7px 0 0 10px}.user-social-link li:nth-child(4n+4){margin-left:0}.vendor-top-info-holder .pull-right{width:calc(100% - 185px)}@media screen and (max-width:768px){.vendor-top-info-holder .pull-right{width:100%}.vendor-img-holder img{max-width:100px;max-height:100px;margin-bottom:15px}}.vendor-info-row{display:inline-block;width:100%!important;border-bottom:1px solid #ddd;padding-bottom:15px;margin-bottom:15px}.vendor-info-row h2{margin-bottom:9px}.vendor-info-row .dashicons-arrow-right-alt{font-size:14px;margin:1px 0 0}article .vendor-info-row{border:0;padding:0 30px}article .vendor-info-row.payment-info-row{background:#e6e6e6;padding:15px 30px;position:relative;width:100%;margin:8px 0 20px}.payment-info-row:after,.payment-info-row:before{content:'';position:absolute;width:100%;height:1px;top:-3px;right:0;background:#e6e6e6}.payment-info-row:before{bottom:-3px;top:auto}.vendor-personal-info-table td{padding:4px 0 2px 20px;font-size:15px;color:#484848}body .wcmp-vendor-modal-content article .payment-info-row p{float:right;font-size:16px;margin:0}body .wcmp-vendor-modal-content article .payment-info-row p:nth-child(2){float:left;margin:0 12px 0 0}body .wcmp-vendor-modal-content article .payment-info-row p mark{margin-right:7px;background:#fff;padding:3px 10px;font-size:13px;color:#028ed9}.payment-info-row img{vertical-align:middle;margin-right:8px;max-width:100px;max-height:50px}.wcmp-vendor-modal-main footer .button{height:40px}.wcmp-vendor-modal-main footer .button{height:35px!important;line-height:35px!important;font-size:15px;padding:0 20px!important}@media screen and (max-width:480px){.wcmp-vendor-modal-main footer .button{height:32px!important;line-height:32px!important;padding:0 10px!important;font-size:13px!important}.wcmp-vendor-modal-main .wcmp-vendor-modal-header h1{font-size:18px}}body .wcmp-vendor-modal-content article p span{vertical-align:middle}body .wcmp-vendor-modal-content article p span.woocommerce-Price-amount{vertical-align:baseline}body .wcmp-vendor-modal-content article p span.dashicons{vertical-align:sub}body .wcmp-vendor-modal-content article p span.woocommerce-Price-currencySymbol{vertical-align:baseline}.vendor-profile p{width:100%;margin:0;padding:10px;background:#fff2c7;text-align:center;font-size:18px}.vendor-profile p.vendor-profile-complete{background:#dcf7e0}#wp-vendor_description-wrap{width:calc(100% - 225px);display:inline-block;vertical-align:top;margin:12px 0}.wcmp-loader{margin:8px 0 0;display:inline-block;vertical-align:middle;-webkit-animation:roundAnimation 2s infinite;animation:roundAnimation 2s infinite}@-webkit-keyframes roundAnimation{0%{-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}100%{-webkit-transform:rotate(0);transform:rotate(0)}}@keyframes roundAnimation{0%{-webkit-transform:rotate(-360deg);transform:rotate(-360deg)}100%{-webkit-transform:rotate(0);transform:rotate(0)}}.social-sprite{background-image:url(../../images/social-sprite.png);background-repeat:no-repeat;display:inline-block;vertical-align:middle}.social-sprite-facebook-icon{width:30px;height:30px;background-position:-1px -1px}.social-sprite-googleplus-icon{width:30px;height:30px;background-position:-33px -1px}.social-sprite-instagram-icon{width:30px;height:30px;background-position:-65px -1px}.social-sprite-linkedin-icon{width:30px;height:30px;background-position:-97px -1px}.social-sprite-twitter-icon{width:30px;height:30px;background-position:-129px -1px}.social-sprite-youtube-icon{width:30px;height:30px;background-position:-161px -1px}.status-sprite{background-image:url(../../images/vendor-status-sprite.png);background-repeat:no-repeat;display:inline-block;vertical-align:middle;margin-left:8px;width:28px;height:28px}.status-sprite-Approved-icon{background-position:-1px -1px}.status-sprite-Pending-icon{background-position:-29px -1px}.status-sprite-Rejected-icon{background-position:-57px -1px}.status-sprite-Suspended-icon{background-position:-85px -1px}.wcmp_page_wcmp-extensions #wpcontent{background:#fff}.wcmp-addon-container .addonbox{width:23%;float:right;top:5px;background:#fff;margin-bottom:30px;margin-left:2.667%;min-width:auto;box-sizing:border-box;box-shadow:0 0 5px rgba(0,0,0,.08);border:1px solid #ddd}.wcmp-addon-container .addonbox h2{margin:0;padding:0 25px 0 0;min-height:auto;font-weight:400;color:#4f4f4f;font-size:17px;position:relative;height:61px;display:table-cell;vertical-align:middle;width:100%;line-height:20px}.wcmp-addon-container .addonbox h2:before{content:'';position:absolute;width:7px;height:39px;right:-1px;background:#f99;top:0;bottom:0;margin:auto}.wcmp-addon-container .addonbox .inside .submitbox #publishing-action{line-height:23px}.addon-content-holder{padding:20px 25px;background:#fff}.wcmp-addon-container div.addon-content-holder p{font-size:14px;line-height:24px;margin:0 0 20px;color:#4f4f4f;min-height:96px;overflow:hidden}.addon-content-holder a.button,.offline-addon-wrap a{text-align:center;width:116px;max-width:100%;height:auto;border:1px solid #0494b4;background-color:#08a7cb;color:#fff;font-size:15px;display:inline-block;padding:6px;text-decoration:none}.addon-content-holder a.button:focus,.addon-content-holder a.button:hover,.offline-addon-wrap a:focus,.offline-addon-wrap a:hover{background-color:#0494b4;border-color:#0494b4;color:#fff;box-shadow:none;outline:0}.addon-banner{width:100%;background:#826dad;position:relative;overflow:hidden;box-sizing:border-box;display:-webkit-box;display:flex;-webkit-box-pack:justify;justify-content:space-between;-webkit-box-align:center;align-items:center;margin:15px 0}.addon-img-holder{height:223px;overflow:hidden;padding:0;text-align:center;background:#ededed url(../../images/extension-bkg.png) no-repeat center center;margin:0 -1px}.addon-img-holder img{max-width:90%;height:auto;max-height:160px;position:relative;top:50%;transform:translateY(-50%);-moz-transform:translateY(-50%);-ms-transform:translateY(-50%);-webkit-transform:translateY(-50%)}.addon-banner-content{width:500px;max-width:100%;margin-left:44px;padding:0 35px}.addon-banner-content h1,.addon-banner-content p{color:#fff}.addon-banner-content a{display:inline-block;background:#fff;color:#000;padding:7px 20px;text-decoration:none}.wcmp-addon-container .addonbox:nth-child(4n+1){clear:both}.wcmp-addon-container .addonbox:nth-child(4n+4){margin:0}.offline-addon-wrap{background:#ececec;padding:25px;margin-top:25px;overflow:hidden}.offline-addon-wrap .addon-content{float:right}.offline-addon-wrap .addon-content h2,.offline-addon-wrap .addon-content p{margin:0;padding:0}.offline-addon-wrap .addon-content p{margin-top:5px}.offline-addon-wrap a{float:left;margin-top:5px}@media screen and (max-width:1100px){.offline-addon-wrap{text-align:right}.offline-addon-wrap .addon-content{width:100%;margin-bottom:10px}.offline-addon-wrap a{float:right}}@media screen and (max-width:1800px){.wcmp-addon-container .addonbox{width:30%;margin-left:5%}.wcmp-addon-container .addonbox:nth-child(4n+4){margin-left:5%}.wcmp-addon-container .addonbox:nth-child(3n+3){margin-left:0}.wcmp-addon-container .addonbox:nth-child(4n+1){clear:none}.wcmp-addon-container .addonbox:nth-child(3n+1){clear:both}}@media screen and (max-width:1400px){.wcmp-addon-container .addonbox{width:32%;margin-left:2%}.wcmp-addon-container .addonbox:nth-child(4n+4){margin-left:2%}.wcmp-addon-container .addonbox:nth-child(3n+3){margin-left:0}.wcmp-addon-container .addonbox:nth-child(4n+1){clear:none}.wcmp-addon-container .addonbox:nth-child(3n+1){clear:both}}@media all and (max-width:1100px) and (min-width:1024px){.addon-banner-content{margin-left:0}}@media (max-width:1023px){.addon-banner-content{width:100%;margin-left:0;padding:30px 35px;background:rgba(0,0,0,.5);position:relative}.addon-banner img{position:absolute;top:0}}@media (max-width:460px){.addon-banner-content{padding:10px 30px}.addon-banner-content h1{font-size:16px;line-height:22px;padding-bottom:0}.addon-banner-content p{margin:10px 0}}@media (max-width:1000px){.wcmp-addon-container .addonbox{width:48%;margin-left:4%}.wcmp-addon-container .addonbox:nth-child(2n+2),.wcmp-addon-container .addonbox:nth-child(3n+1){clear:none}.wcmp-addon-container .addonbox:nth-child(2n+1){clear:both;margin-left:4%}.wcmp-addon-container .addonbox:nth-child(2n+2){margin:0}}@media (max-width:640px){.wcmp-addon-container .addonbox{width:100%;margin:0 auto 30px!important;width:370px;max-width:100%;float:none}.wcmp-addon-container .addonbox .hndle{min-height:37px}.wcmp-addon-container .addonbox h2{margin:15px;font-size:15px;height:55px;padding:0 15px 0 0}.wcmp-addon-container .addonbox h2:before{width:4px;height:32px}.addon-content-holder{background:#fff}.wcmp-addon-container div.addon-content-holder p{min-height:auto;height:auto}}.wcmp_page_wcmp-to-do .wcmp_vendors_settings #submit{display:none}.wcmp_page_wcmp-to-do .wcmp_vendors_settings .form-table th{padding-bottom:17px!important;padding-top:17px!important}.wcmp_page_wcmp-to-do .wcmp_vendors_settings .form-table{box-shadow:0 3px 0 #dbdbdb;border-color:#d2d2d2;position:relative}.wcmp_page_wcmp-to-do .wcmp_vendors_settings .form-table a{color:#e67853;text-decoration:none}.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr:last-child{border:none}.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr:first-child{border-bottom:solid 1px #d2d2d2;border-top:solid 1px #d2d2d2}.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr td,.wcmp_page_wcmp-to-do .wcmp_vendors_settings #to_do_list tbody tr th{padding-right:20px}.wcmp_page_wcmp-to-do .activate_vendor,.wcmp_page_wcmp-to-do .reject_vendor,.wcmp_page_wcmp-to-do .vendor_dismiss_button,.wcmp_page_wcmp-to-do .vendor_edit_button,.wcmp_page_wcmp-to-do .vendor_transaction_done_button{width:35px;height:35px;border-radius:5px;border-top:solid 1px #d6c9c6;border-right:solid 1px #d6c9c6;border-left:solid 2px #d6c9c6;border-bottom:solid 3px #d6c9c6;text-indent:-99999px}.wcmp_page_wcmp-to-do .activate_vendor:hover,.wcmp_page_wcmp-to-do .reject_vendor:hover,.wcmp_page_wcmp-to-do .vendor_dismiss_button:hover,.wcmp_page_wcmp-to-do .vendor_edit_button:hover,.wcmp_page_wcmp-to-do .vendor_transaction_done_button:hover{cursor:pointer;border-color:#beaba6}.wcmp_page_wcmp-to-do .vendor_edit_button{background:url(../../images/icon-vendor-edit.png ) -2px -2px}.wcmp_page_wcmp-to-do .activate_vendor{background:url(../../images/icon-vendor-activate.png) -2px -2px}.wcmp_page_wcmp-to-do .reject_vendor{background:url(../../images/icon-vendor-reject.png) -2px -2px}.wcmp_page_wcmp-to-do .vendor_dismiss_button{background:url(../../images/icon-vendor-dismiss.png) -2px -2px}.wcmp_page_wcmp-to-do .vendor_transaction_done_button{background:url(../../images/icon-vendor-notify.png) -2px -2px}#menu-settings-column .button-controls{clear:none;padding:0;width:100%}#menu-settings-column .button-controls a{width:100%;text-align:center}.menu-item-handle{background:#fff none repeat scroll 100% 0;padding:10px 10px 10px 5px}#post-body-content .menu-item-handle{width:510px;max-width:100%}.wcmp_vendor_registration_settings_group #post-body-content{width:100%;min-width:auto}#wcmp-vendor-form .postbox{width:100%;min-width:100%}.menu-item-bar .menu-item-handle{line-height:20px!important}.wcmp-vendor-form-field-content{background:#fff none repeat scroll 100% 0;padding:3px;width:80%}.wcmp-vendor-form-input-field{width:100%}.wcmp-vendor-form-input-field-container{padding:5px}.wcmp-remove-form-field{padding-right:2px;color:red}.wcmp-vendor-form-input-field-container ul li a{display:inline-block;line-height:18px;text-decoration:none!important}.field-selectbox-options{padding-top:10px}.field-selectbox-options li{background:#fcfcfc none repeat scroll 100% 0;border-bottom:1px solid #ddd;padding:2px 10px;cursor:move}.wcmp-form-empty-container{border:3px dashed #bbb;color:#ccc;font-size:20px;margin:20px 0;padding:100px 10px;text-align:center;width:90%}.wcmp-form-field{border-bottom:1px solid #f6f6f6;padding:5px;width:100%}.wcmp-form-field>label{color:#666;display:inline-block;font-size:13px;width:20%}.wcmp-form-field>span{color:#222;width:79%;display:inline-block;vertical-align:top}.wcmp-form-field h2{margin:12px 0 23px}.vendor-quick-view-notes h2{margin:12px 0 23px}.vendor-quick-view-notes .note-clm{max-width:75%}.form-control{width:100%}.pending-vendor-note{margin:25px 0 0}footer .pending-vendor-note{margin:0 0 8px}.note-clm-wrap{padding-right:30px;position:relative}.note-clm-wrap:after{content:'';width:1px;height:100%;background:#ccc;position:absolute;top:0;right:0}.note-clm-wrap .note-clm:last-of-type{margin-bottom:0}.note-clm{display:block;padding:14px 17px;max-width:60%;margin:0 0 20px 5%;border:1px solid #ccc;position:relative;border-radius:6px}@media screen and (max-width:767px){.note-clm,.vendor-quick-view-notes .note-clm{max-width:100%;margin-left:0}.wcmp-form-field>label{width:100%}}.note-clm:before{width:13px;height:13px;border:2px solid #fff;border-radius:100px;margin:16px -4px 0 0;box-shadow:0 1px 1px rgba(0,0,0,.2),inset 0 1px 1px rgba(0,0,0,.1);background:#fd5869;content:"";position:absolute;top:0;right:-33px;z-index:1;box-sizing:border-box}.note-clm:after{width:9px;height:9px;border-width:0 1px 1px 0;border-style:solid;border-color:#ccc;background:#fff;-webkit-transform:rotate(-45deg);-ms-transform:rotate(-45deg);transform:rotate(-45deg);content:"";position:absolute;top:18px;right:-6px}.note-description{margin:0 0 21px}.note-clm p.note-meta{margin:0;display:inline-block;color:#969696}.note-clm p.note-meta.note_owner{margin-right:10px}.wcmp-form-field .note-clm{max-width:70%}textarea.form-control{height:80px}#vendor_preview_tabs #vendor_profile_image_display{vertical-align:top}.box_data{padding:10px}.pad_left{padding-right:10px}.bars span{height:15px!important}.bar1,.bar2,.bar3{width:30px;height:15px;float:right}.bar1{background-color:#ef6448;border-radius:3px}.bar2{background-color:#1dbb5c;border-radius:3px}.bar3{background-color:#ddd30c;border-radius:3px}.bar_indecator span{float:right;margin-right:10px;margin-left:45px}.bar_indecator{float:right;margin-top:16px;margin-right:100px}.sort_chart h4{float:right}.sorting_box{float:left;margin-top:7px}.bars span.alt{margin-top:14px!important}.bars span.main{margin-top:2px!important}.product_report_search,.vendor_report_search{cursor:pointer}.sorting_box input{display:none}.low_to_high_btn_product,.low_to_high_btn_vendor{background-color:#faf8f9;width:29px;height:29px;border-radius:5px;border-top:solid 1px #d6c9c6;border-right:solid 1px #d6c9c6;border-left:solid 2px #d6c9c6;border-bottom:solid 3px #d6c9c6}.high_to_low_btn_product,.high_to_low_btn_vendor{background-color:#eceaed;width:29px;height:29px;border-radius:5px;border-top:solid 1px #b2b0b1;border-right:solid 1px #b2b0b1;border-left:solid 1px #b2b0b1;border-bottom:solid 1px #b2b0b1}.high_to_low_btn_product:hover,.high_to_low_btn_vendor:hover,.low_to_high_btn_product:hover,.low_to_high_btn_vendor:hover{cursor:pointer}.bars span{background-color:#ef6448!important}.bars span.alt{background-color:#1dbb5c!important}.bars span.vendor_bar{background-color:#ddd30c!important}i.dashicons-arrow-up-alt{color:#af938f;font-size:16px}i.dashicons-arrow-down-alt{color:#af9692;font-size:16px;font-weight:100}table.wcmp_tools_table.widefat{border:none}table.wcmp_tools_table.widefat td,table.wcmp_tools_table.widefat th{padding:2em}table.wcmp_tools_table td strong.name,table.wcmp_tools_table th strong.name{display:block;margin-bottom:.5em}table.wcmp_tools_table td.run-tool,table.wcmp_tools_table th.run-tool{text-align:left}.widefat .column-wcmp_suborder mark{display:block;text-indent:-9999px;position:relative;height:1em;width:1em;background:100% 0;font-size:1.4em;margin:0 auto}.widefat .column-wcmp_suborder mark:after{font-family:WooCommerce;speak:none;font-weight:400;font-variant:normal;text-transform:none;line-height:1;-webkit-font-smoothing:antialiased;margin:0;text-indent:0;position:absolute;top:0;right:0;width:100%;height:100%;text-align:center}.widefat .column-wcmp_suborder mark{float:right;margin-left:8px;font-size:1.1em;margin-top:2px}.widefat .column-wcmp_suborder mark.pending:after{content:"\e012";color:#ffba00}.widefat .column-wcmp_suborder mark.trash:after{content:"\e602";color:#a00}.widefat .column-wcmp_suborder mark.completed:after{content:"\e015";color:#2ea2cc}.widefat .column-wcmp_suborder mark.on-hold:after{content:"\e033";color:#999}.widefat .column-wcmp_suborder mark.failed:after{content:"\e016";color:#d0c21f}.widefat .column-wcmp_suborder mark.cancelled:after{content:"\e013";color:#a00}.widefat .column-wcmp_suborder mark.processing:after{content:"\e011";color:#73a724}.widefat .column-wcmp_suborder mark.refunded:after{content:"\e014";color:#999}#wcmp-commission-data .hndle,#wcmp-commission-data button.handlediv{display:none}#wcmp-commission-data .inside{padding:0;margin:0}#wcmp-commission-data h2{margin:0;font-family:HelveticaNeue-Light,'Helvetica Neue Light','Helvetica Neue',sans-serif;font-size:21px;font-weight:400;line-height:1.2;text-shadow:-1px 1px 1px #fff;padding:0}.wcmp-commission-amount a.edit_commission_amount{width:14px;height:0;padding:14px 0 0;margin:0 6px 0 0;overflow:hidden;position:relative;color:#999;border:0;float:left}.wcmp-commission-amount a.edit_commission_amount:after{font-family:Dashicons;content:'\f464';position:absolute;top:0;right:0;text-align:center;vertical-align:top;line-height:14px;font-size:14px;font-weight:400}