<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Automattic\\WooCommerce\\Vendor\\' => array($baseDir . '/lib/packages'),
);
