/*@media screen and (min-width: 1099px) {
    div.row.rowhead div.rowsub, div.row.rowbody div.rowsub{
        width:32% !important;
    }
}*/
/* policies */
/* Q and A  */
div.rowhead {
  background-color: #CCC;
  margin: 0 !important;
  width: 100%;
  box-sizing: border-box; }

div.row.rowhead div.rowsub {
  padding: 2%;
  float: right;
  padding-top: 15px;
  padding-bottom: 15px; }

div.row.rowbody div.rowsub {
  padding: 1%;
  float: right;
  padding-top: 5px;
  padding-bottom: 5px; }

#wcmp_multiple_product_sorting {
  margin: 0; }

div.rowbody {
  width: 100%;
  border: 1px solid #eee;
  padding: 5px;
  margin: 0 !important;
  box-sizing: border-box; }
  div.rowbody:nth-child(odd) {
    background-color: #ddd; }
  div.rowbody:nth-child(even) {
    background-color: #eee; }

.buttongap {
  margin-bottom: 5px !important;
  margin-top: 5px !important;
  clear: both; }

.centerclass {
  text-align: center; }

.wcmp_regular_price {
  text-decoration: line-through;
  color: #8A0808;
  font-weight: normal; }

.woocommerce-Tabs-panel del .woocommerce-Price-amount {
  text-decoration: line-through;
  color: #8A0808;
  font-weight: normal; }

.woocommerce-Tabs-panel ins .woocommerce-Price-amount {
  color: #0B3B0B;
  font-weight: bold;
  display: block; }

.woocommerce-tabs ins {
  text-decoration: none; }

.woocommerce-tabs .tab-panels {
  position: relative; }

.wcmp_sale_price {
  color: #0B3B0B;
  font-weight: bold;
  display: block; }

.wcmp_btn_view_details {
  background: linear-gradient(to bottom, #f78828 0, #dd771f 100%) repeat scroll 100% 0 rgba(0, 0, 0, 0);
  border: 1px solid #da7532;
  border-radius: 4px;
  color: #f9f9f9;
  cursor: pointer;
  outline: 0 none;
  text-shadow: 0 0 1px rgba(0, 0, 0, 0.33);
  text-align: center;
  padding: 5px;
  margin: 5px; }
  .wcmp_btn_view_details:hover {
    background: linear-gradient(to bottom, #f78828 0, #ba5400 100%) repeat scroll 100% 0 rgba(0, 0, 0, 0);
    color: #f9f9f9; }

.wcmp_btn_add_to_cart {
  -moz-border-bottom-colors: none;
  -moz-border-right-colors: none;
  -moz-border-left-colors: none;
  -moz-border-top-colors: none;
  background-color: #7fbf4d;
  background-image: -moz-linear-gradient(center top, #7fbf4d, #63a62f);
  border-color: #63a62f #63a62f #5b992b;
  border-image: none;
  border-style: solid;
  border-width: 1px;
  box-shadow: 0 1px 0 0 #96ca6d inset;
  color: #fff;
  padding: 7px 0 6px;
  text-shadow: 0 -1px 0 #4c9021;
  text-transform: uppercase;
  text-align: center;
  padding: 5px;
  border-radius: 4px;
  margin: 5px;
  cursor: pointer; }
  .wcmp_btn_add_to_cart:hover {
    color: #fff;
    background-color: #5fbf4d;
    background-image: -moz-linear-gradient(center top, #5fbf4d, #63a62f); }

a.wcmp_seller_name {
  text-decoration: none;
  list-style: none;
  box-shadow: 0; }

span.wcmp_superb_rating {
  color: #fff;
  background-color: #0B6121;
  padding: 5px;
  border-radius: 3px; }

span.wcmp_excellent_rating {
  color: #fff;
  background-color: #088A29;
  padding: 5px;
  border-radius: 3px; }

span.wcmp_good_rating {
  color: #fff;
  background-color: #04B404;
  padding: 5px;
  border-radius: 3px; }

span.wcmp_above_averege_rating {
  color: #fff;
  background-color: #FF8000;
  padding: 5px;
  border-radius: 3px; }

span.wcmp_averege_rating {
  color: #fff;
  background-color: #8A2908;
  padding: 5px;
  border-radius: 3px; }

span.wcmp_bad_rating {
  color: #fff;
  background-color: #B40431;
  padding: 5px;
  border-radius: 3px; }

span.wcmp_norating {
  background-color: #088A85;
  color: #fff;
  padding: 5px;
  border-radius: 3px; }

#tab-singleproductmultivendor {
  position: relative; }
  #tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(1) {
    width: 30% !important; }
  #tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(2) {
    width: 20% !important; }
  #tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(3) {
    width: 50% !important; }
  #tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(1) {
    width: 30% !important; }
  #tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(2) {
    width: 20% !important; }
  #tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(3) {
    width: 50% !important;
    text-align: left; }
  #tab-singleproductmultivendor .container {
    max-width: 100%; }

.singleproductmultivendor_tab .container {
  max-width: 100%; }

.singleproductmultivendor_tab .wc-accordion-content {
  position: relative; }

.woocommerce-Tabs-panel--singleproductmultivendor {
  position: relative; }
  .woocommerce-Tabs-panel--singleproductmultivendor .container {
    max-width: 100%; }

#content_tab_singleproductmultivendor {
  position: relative; }

.ajax_loader_class_msg {
  width: 100%;
  position: absolute;
  z-index: 8888;
  background-color: #ddd;
  opacity: 0.5;
  height: auto;
  text-align: center;
  display: none;
  height: 100%;
  top: 0;
  right: 0; }
  .ajax_loader_class_msg img {
    margin: auto;
    position: absolute;
    top: 0;
    right: 0;
    left: 0;
    bottom: 0; }

.goto_more_offer_tab.button {
  margin: 15px 0 10px; }

.wcmp-product-policies h2.wcmp_policies_heading {
  margin: 0 0 7px 0 !important; }

.wcmp-product-policies .wcmp_policies_description {
  margin-bottom: 25px; }

h2.custqna-title {
  margin: 0; }

form.customerqna-form {
  margin: 12px 0 20px 0; }

.qna-item-wrap {
  margin: 0;
  width: 100%;
  border: 1px solid #e2e2e2;
  border-bottom: 0; }

.qna-result-wrap .qna-item-wrap:last-of-type {
  border-bottom: 1px solid #e2e2e2; }

#qna-ask-input {
  padding: 0;
  border-radius: 0;
  background-color: #fff; }
  #qna-ask-input input[type="text"] {
    border: 0;
    box-shadow: none;
    width: 100%;
    background: #fff;
    padding: 10px 20px;
    margin: 0;
    font-size: 17px;
    height: 54px;
    border: 1px solid #e2e2e2; }
  #qna-ask-input #ask-wrap {
    display: none;
    padding: 0;
    margin: 40px 5px 0;
    overflow: hidden;
    text-align: center; }
  #qna-ask-input button {
    margin: 0 20px 0 0;
    box-shadow: none;
    border: 1px solid #c1c1c1;
    color: #4a4a4a;
    text-transform: uppercase;
    padding: 2px 40px;
    background: #ffffff;
    background: -moz-linear-gradient(top, #ffffff 0%, #ebebeb 100%);
    background: -webkit-linear-gradient(top, #ffffff 0%, #ebebeb 100%);
    background: linear-gradient(to bottom, #ffffff 0%, #ebebeb 100%);
    filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#ffffff', endColorstr='#ebebeb',GradientType=0 );
    vertical-align: middle; }
    #qna-ask-input button:hover {
      background: #ebebeb;
      background: -moz-linear-gradient(top, #ebebeb 0%, #ffffff 100%);
      background: -webkit-linear-gradient(top, #ebebeb 0%, #ffffff 100%);
      background: linear-gradient(to bottom, #ebebeb 0%, #ffffff 100%);
      filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#ebebeb', endColorstr='#ffffff',GradientType=0 ); }

.qna-block {
  padding: 1rem;
  background-color: #fff;
  width: 100%;
  border-radius: 0;
  display: table; }
  .qna-block textarea {
    width: 100%;
    resize: none; }
  .qna-block > p {
    font-size: 15px;
    margin-bottom: 5px; }
  .qna-block > p.qna-answer {
    margin-bottom: 1.25rem; }

.qna-vote {
  display: table-cell;
  width: 56px;
  padding-left: 20px; }
  .qna-vote .vote {
    background: #555555;
    text-align: center; }
  .qna-vote a {
    display: block;
    padding: 6px 2px;
    line-height: 14px;
    border: 0; }

.vote-sprite {
  background-image: url("../../images/qna-vote.png");
  background-repeat: no-repeat;
  display: block;
  margin: 0 auto; }

.vote-sprite-like {
  width: 16px;
  height: 16px;
  background-position: -1px -1px; }

.vote-sprite-dislike {
  width: 16px;
  height: 16px;
  background-position: -1px -19px; }

.qtn-content {
  display: table-cell;
  padding: 0;
  vertical-align: top; }

.vote-count {
  font-size: 16px;
  color: #fff;
  padding: 0 3px; }

.bottom-qna {
  overflow: hidden;
  margin-top: 0;
  color: #acb4c2;
  font-size: 0.875rem; }
  .bottom-qna .qna-info li {
    display: inline;
    margin: 0; }

.qna-info {
  float: right;
  margin: 0;
  width: 100%; }
  .qna-info li {
    padding-left: 0.75rem;
    padding-right: 0.75rem;
    font-size: 15px;
    color: #898989;
    position: relative; }
    .qna-info li:first-child {
      padding-right: 0; }
    .qna-info li:last-child {
      border-left: 0; }
      .qna-info li:last-child:after {
        display: none; }
    .qna-info li:after {
      content: '';
      position: absolute;
      width: 1px;
      height: calc(100% - 5px);
      left: 0;
      background: #898989;
      top: 4px; }

.qtn-row {
  margin-bottom: 10px; }
  .qtn-row p {
    margin: 0;
    display: inline-block;
    color: #343434;
    font-size: 16px;
    vertical-align: top;
    line-height: 21px; }
    .qtn-row p span {
      margin-left: 2px; }
  .qtn-row p.qtn-title-text {
    margin-left: 1px; }

.qna-question {
  font-weight: 600; }

#qna-result-msg {
  background: #e4e4e4;
  padding: 6px 10px;
  font-size: 15px; }
  #qna-result-msg:empty {
    background: none;
    padding: 0; }

.no-answer-lbl {
  margin: 0;
  display: inline-block;
  font-weight: 400;
  vertical-align: middle;
  font-size: 20px;
  line-height: 20px;
  padding: 0; }

@media screen and (max-width: 1100px) {
  div.rowhead div.rowsub {
    width: 98%;
    padding: 1%;
    float: right;
    padding-top: 15px;
    padding-bottom: 15px;
    text-align: center; }
  div.rowbody div.rowsub {
    width: 98%;
    padding: 1%;
    float: right;
    padding-top: 5px;
    padding-bottom: 5px;
    text-align: center; }
  .centerclass {
    text-align: right; } }

@media (max-width: 767px) {
  #tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(1) {
    width: 100%; }
  #tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(2) {
    width: 100%; }
  #tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(3) {
    width: 100%; }
  #tab-singleproductmultivendor .row.rowbody .star-rating {
    right: calc(50% - 50px); }
  #tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(1) {
    width: 100%;
    display: none; }
  #tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(2) {
    width: 100%;
    display: none; }
  #tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(3) {
    width: 100%;
    text-align: center; } }

@media screen and (max-width: 400px) {
  div.rowhead div.rowsub {
    width: 98%;
    padding: 1%;
    float: right;
    padding-top: 15px;
    padding-bottom: 15px;
    text-align: center; }
  div.rowbody div.rowsub {
    width: 98%;
    padding: 1%;
    float: right;
    padding-top: 5px;
    padding-bottom: 5px;
    text-align: center; } }

@media screen and (max-width: 480px) {
  #qna-ask-input button {
    float: none;
    display: table;
    margin: 10px auto 0; }
  #qna-ask-input #ask-wrap {
    text-align: center; } },div.rowhead{background-color:#ccc;margin:0!important;width:100%;box-sizing:border-box}div.row.rowhead div.rowsub{padding:2%;float:right;padding-top:15px;padding-bottom:15px}div.row.rowbody div.rowsub{padding:1%;float:right;padding-top:5px;padding-bottom:5px}#wcmp_multiple_product_sorting{margin:0}div.rowbody{width:100%;border:1px solid #eee;padding:5px;margin:0!important;box-sizing:border-box}div.rowbody:nth-child(odd){background-color:#ddd}div.rowbody:nth-child(even){background-color:#eee}.buttongap{margin-bottom:5px!important;margin-top:5px!important;clear:both}.centerclass{text-align:center}.wcmp_regular_price,.woocommerce-Tabs-panel del .woocommerce-Price-amount{text-decoration:line-through;color:#8a0808;font-weight:400}.woocommerce-tabs ins{text-decoration:none}.wcmp_sale_price,.woocommerce-Tabs-panel ins .woocommerce-Price-amount{color:#0b3b0b;font-weight:700;display:block}.wcmp_btn_view_details{background:-webkit-gradient(linear,right top,right bottom,color-stop(0,#f78828),to(#dd771f)) repeat scroll 100% 0 rgba(0,0,0,0);background:-webkit-linear-gradient(top,#f78828 0,#dd771f 100%) repeat scroll 100% 0 rgba(0,0,0,0);background:-webkit-gradient(linear,right top, right bottom,color-stop(0, #f78828),to(#dd771f)) repeat scroll 100% 0 rgba(0,0,0,0);background:linear-gradient(to bottom,#f78828 0,#dd771f 100%) repeat scroll 100% 0 rgba(0,0,0,0);border:1px solid #da7532;border-radius:4px;color:#f9f9f9;cursor:pointer;outline:0 none;text-shadow:0 0 1px rgba(0,0,0,.33);text-align:center;padding:5px;margin:5px}.wcmp_btn_view_details:hover{background:-webkit-gradient(linear,right top,right bottom,color-stop(0,#f78828),to(#ba5400)) repeat scroll 100% 0 rgba(0,0,0,0);background:-webkit-linear-gradient(top,#f78828 0,#ba5400 100%) repeat scroll 100% 0 rgba(0,0,0,0);background:-webkit-gradient(linear,right top, right bottom,color-stop(0, #f78828),to(#ba5400)) repeat scroll 100% 0 rgba(0,0,0,0);background:linear-gradient(to bottom,#f78828 0,#ba5400 100%) repeat scroll 100% 0 rgba(0,0,0,0);color:#f9f9f9}.wcmp_btn_add_to_cart{-moz-border-bottom-colors:none;-moz-border-right-colors:none;-moz-border-left-colors:none;-moz-border-top-colors:none;background-color:#7fbf4d;background-image:-moz-linear-gradient(center top ,#7fbf4d,#63a62f);border-color:#63a62f #63a62f #5b992b;-webkit-border-image:none;-o-border-image:none;border-image:none;border-style:solid;border-width:1px;box-shadow:0 1px 0 0 #96ca6d inset;color:#fff;padding:7px 0 6px;text-shadow:0 -1px 0 #4c9021;text-transform:uppercase;text-align:center;padding:5px;border-radius:4px;margin:5px;cursor:pointer}.wcmp_btn_add_to_cart:hover{color:#fff;background-color:#5fbf4d;background-image:-moz-linear-gradient(center top ,#5fbf4d,#63a62f)}a.wcmp_seller_name{text-decoration:none;list-style:none;box-shadow:0}span.wcmp_superb_rating{color:#fff;background-color:#0b6121;padding:5px;border-radius:3px}span.wcmp_excellent_rating{color:#fff;background-color:#088a29;padding:5px;border-radius:3px}span.wcmp_good_rating{color:#fff;background-color:#04b404;padding:5px;border-radius:3px}span.wcmp_above_averege_rating{color:#fff;background-color:#ff8000;padding:5px;border-radius:3px}span.wcmp_averege_rating{color:#fff;background-color:#8a2908;padding:5px;border-radius:3px}span.wcmp_bad_rating{color:#fff;background-color:#b40431;padding:5px;border-radius:3px}span.wcmp_norating{background-color:#088a85;color:#fff;padding:5px;border-radius:3px}#tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(1),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(1){width:30%!important}#tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(2),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(2){width:20%!important}#tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(3),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(3){width:50%!important}#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(3){text-align:left}@media screen and (max-width:1100px){div.rowhead div.rowsub{width:98%;padding:1%;float:right;padding-top:15px;padding-bottom:15px;text-align:center}div.rowbody div.rowsub{width:98%;padding:1%;float:right;padding-top:5px;padding-bottom:5px;text-align:center}.centerclass{text-align:right}}#tab-singleproductmultivendor .container,.singleproductmultivendor_tab .container,.woocommerce-Tabs-panel--singleproductmultivendor .container{max-width:100%}#content_tab_singleproductmultivendor,#tab-singleproductmultivendor,.singleproductmultivendor_tab .wc-accordion-content,.woocommerce-Tabs-panel--singleproductmultivendor,.woocommerce-tabs .tab-panels{position:relative}.ajax_loader_class_msg{width:100%;position:absolute;z-index:8888;background-color:#ddd;opacity:.5;height:auto;text-align:center;display:none;height:100%;top:0;right:0}.ajax_loader_class_msg img{margin:auto;position:absolute;top:0;right:0;left:0;bottom:0}.goto_more_offer_tab.button{margin:15px 0 10px}@media (max-width:767px){#tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(1),#tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(2),#tab-singleproductmultivendor .row.rowbody .rowsub:nth-of-type(3),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(1),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(2),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(3){width:100%}#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(3){text-align:center}#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(1),#tab-singleproductmultivendor .row.rowhead .rowsub:nth-of-type(2){display:none}#tab-singleproductmultivendor .row.rowbody .star-rating{right:calc(50% - 50px)}}@media screen and (max-width:400px){div.rowhead div.rowsub{width:98%;padding:1%;float:right;padding-top:15px;padding-bottom:15px;text-align:center}div.rowbody div.rowsub{width:98%;padding:1%;float:right;padding-top:5px;padding-bottom:5px;text-align:center}}.wcmp-product-policies h2.wcmp_policies_heading{margin:0 0 7px 0!important}.wcmp-product-policies .wcmp_policies_description{margin-bottom:25px}h2.custqna-title{margin:0}form.customerqna-form{margin:12px 0 20px 0}.qna-item-wrap{margin:0;width:100%;border:1px solid #e2e2e2;border-bottom:0}.qna-result-wrap .qna-item-wrap:last-of-type{border-bottom:1px solid #e2e2e2}#qna-ask-input{padding:0;border-radius:0;background-color:#fff}#qna-ask-input input[type=text]{border:0;box-shadow:none;width:100%;background:#fff;padding:10px 20px;margin:0;font-size:17px;height:54px;border:1px solid #e2e2e2}#qna-ask-input #ask-wrap{display:none;padding:0;margin:40px 5px 0;overflow:hidden;text-align:center}#qna-ask-input button{margin:0 20px 0 0;box-shadow:none;border:1px solid #c1c1c1;color:#4a4a4a;text-transform:uppercase;padding:2px 40px;background:#fff;background:-webkit-linear-gradient(top,#fff 0,#ebebeb 100%);background:-webkit-gradient(linear,right top,right bottom,from(#fff),to(#ebebeb));background:-webkit-gradient(linear,right top, right bottom,color-stop(0, #fff),to(#ebebeb));background:linear-gradient(to bottom,#fff 0,#ebebeb 100%);vertical-align:middle}#qna-ask-input button:hover{background:#ebebeb;background:-webkit-linear-gradient(top,#ebebeb 0,#fff 100%);background:-webkit-gradient(linear,right top,right bottom,from(#ebebeb),to(#fff));background:-webkit-gradient(linear,right top, right bottom,color-stop(0, #ebebeb),to(#fff));background:linear-gradient(to bottom,#ebebeb 0,#fff 100%)}@media screen and (max-width:480px){#qna-ask-input button{float:none;display:table;margin:10px auto 0}#qna-ask-input #ask-wrap{text-align:center}}.qna-block{padding:1rem;background-color:#fff;width:100%;border-radius:0;display:table}.qna-vote{display:table-cell;width:56px;padding-left:20px}.qna-vote .vote{background:#555;text-align:center}.qna-vote a{display:block;padding:6px 2px;line-height:14px;border:0}.vote-sprite{background-image:url(../../images/qna-vote.png);background-repeat:no-repeat;display:block;margin:0 auto}.vote-sprite-like{width:16px;height:16px;background-position:-1px -1px}.vote-sprite-dislike{width:16px;height:16px;background-position:-1px -19px}.qtn-content{display:table-cell;padding:0;vertical-align:top}.qna-block textarea{width:100%;resize:none}.vote-count{font-size:16px;color:#fff;padding:0 3px}.qna-block>p{font-size:15px;margin-bottom:5px}.qna-block>p.qna-answer{margin-bottom:1.25rem}.bottom-qna{overflow:hidden;margin-top:0;color:#acb4c2;font-size:.875rem}.qna-info{float:right;margin:0;width:100%}.bottom-qna .qna-info li{display:inline;margin:0}.qna-info li:first-child{padding-right:0}.qna-info li:last-child{border-left:0}.qna-info li{padding-left:.75rem;padding-right:.75rem;font-size:15px;color:#898989;position:relative}.qna-info li:after{content:'';position:absolute;width:1px;height:calc(100% - 5px);left:0;background:#898989;top:4px}.qna-info li:last-child:after{display:none}.qtn-row{margin-bottom:10px}.qtn-row p{margin:0;display:inline-block;color:#343434;font-size:16px;vertical-align:top;line-height:21px}.qtn-row p.qtn-title-text{margin-left:1px}.qtn-row p span{margin-left:2px}.qna-question{font-weight:600}#qna-result-msg{background:#e4e4e4;padding:6px 10px;font-size:15px}#qna-result-msg:empty{background:100% 0;padding:0}.no-answer-lbl{margin:0;display:inline-block;font-weight:400;vertical-align:middle;font-size:20px;line-height:20px;padding:0}