/**
 * Internal dependencies
 */
import Button from '../';

export default {
	title: 'WooCommerce Blocks/@base-components/Button',
	component: Button,
};

export const Default = () => <Button>Buy now</Button>;
