:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}.components-visually-hidden{border:0;clip:rect(1px,1px,1px,1px);-webkit-clip-path:inset(50%);clip-path:inset(50%);height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute;width:1px;word-wrap:normal!important}.components-visually-hidden:focus{background-color:#ddd;clip:auto!important;-webkit-clip-path:none;clip-path:none;color:#444;display:block;font-size:1em;height:auto;right:5px;line-height:normal;padding:15px 23px 14px;text-decoration:none;top:5px;width:auto;z-index:100000}#woocommerce-admin-print-label{min-height:100px}#woocommerce-admin-print-label .handlediv,#woocommerce-admin-print-label .hndle,#woocommerce-admin-print-label .postbox-header{display:none}#woocommerce-admin-print-label .wc-admin-shipping-banner-container{display:flex;align-items:center;margin-top:18px;margin-bottom:6px}#woocommerce-admin-print-label .wc-admin-shipping-banner-blob,#woocommerce-admin-print-label .wc-admin-shipping-banner-illustration{margin-left:8px}#woocommerce-admin-print-label h3{margin:0;font-size:1.15em}#woocommerce-admin-print-label p{margin:0;font-size:15px;line-height:19px}#woocommerce-admin-print-label .components-button.is-primary{margin:10px auto 10px 61px;padding:0 30px;font-size:14px;font-weight:500;line-height:25px;text-decoration:none;text-shadow:none;overflow:visible}#woocommerce-admin-print-label .notice-dismiss{top:14px;left:18px}#woocommerce-admin-print-label .wc-admin-shipping-banner-install-error{color:#d63638}#woocommerce-admin-print-label .wc-admin-shipping-banner-install-error>.warning-icon{margin-left:3px;vertical-align:middle;margin-top:-2px}#woocommerce-admin-print-label .wc-admin-shipping-banner-install-error>.warning-icon path{fill:#d63638}#woocommerce-admin-print-label .components-external-link__icon{display:none}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal{width:500px;max-width:100%}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .components-modal__header{border-bottom:0;margin:4px 0;padding:0;height:50px}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .components-modal__header .components-button.has-icon{right:15px}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .components-modal__header .components-button.has-icon svg{width:30px;height:30px}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .components-modal__header-heading{font-style:normal;font-weight:500;font-size:20px;line-height:32px}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .wc-admin-shipping-banner__dismiss-modal-help-text{font-size:16px;line-height:24px;margin:0 0 1em 60px}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .wc-admin-shipping-banner__dismiss-modal-actions{text-align:left}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .wc-admin-shipping-banner__dismiss-modal-actions .components-button.is-primary{height:30px;padding-right:15px;padding-left:15px;text-align:center;font-size:14px;line-height:15px;font-weight:500;align-items:center;margin-right:10px}.components-modal__frame.wc-admin-shipping-banner__dismiss-modal .wc-admin-shipping-banner__dismiss-modal-actions button.is-primary{align-self:flex-end}@media(max-width:1080px){#woocommerce-admin-print-label{text-align:center}#woocommerce-admin-print-label .wc-admin-shipping-banner-container{flex-wrap:wrap;justify-content:center}#woocommerce-admin-print-label .components-button.is-primary{margin:10px 0 0}#woocommerce-admin-print-label p{margin:5px 15px;max-width:none}#woocommerce-admin-print-label h3{margin-top:10px}}.notice.wcs-nux__notice,.wc-admin-shipping-banner__dismiss-modal .components-button span{display:none}