<?php
if(!defined('ABSPATH')) exit();
?>
	<rs-module-wrap id="rev_slider_735_1_wrapper" data-alias="rvsldr-addons" data-source="gallery" style="background:#2a2c2f;padding:0px;margin:0px auto;margin-top:0px;margin-bottom:0px;">
							<rs-module id="rev_slider_735_1" class="feature_slider" style="display:none;" data-version="6.0.0">
								<rs-slides>
									<rs-slide data-key="rs-2449" data-title="Slide" data-anim="ei:d;eo:d;s:300;r:0;t:fade;sl:d;">
										<img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png"  data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/transparent.png" data-bg="p:center center;c:#2a2c2f;" class="rev-slidebg" data-no-retina>
						<!--
										--><rs-layer
											id="slide-2449-layer-1" 
											data-type="image"
											data-rsp_ch="on"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:100;sp:2000;sR:100;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6900;"
											style="z-index:5;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon1.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-16" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:100px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:150;sp:2000;sR:150;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6850;"
											style="z-index:6;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon2.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-17" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:200px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:200;sp:2000;sR:200;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6800;"
											style="z-index:7;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon3.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-18" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:300px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:250;sp:2000;sR:250;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6750;"
											style="z-index:8;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon4.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-19" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="y:67px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:300;sp:2000;sR:300;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6700;"
											style="z-index:9;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon7.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-20" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:100px;y:67px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:350;sp:2000;sR:350;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6650;"
											style="z-index:10;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon8.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-21" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:200px;y:67px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:400;sp:2000;sR:400;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6600;"
											style="z-index:11;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon5.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-22" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:300px;y:67px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:450;sp:2000;sR:450;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6550;"
											style="z-index:12;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon6.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-23" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="y:133px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:500;sp:2000;sR:500;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6500;"
											style="z-index:13;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon9.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-24" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:100px;y:133px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:550;sp:2000;sR:550;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6450;"
											style="z-index:14;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon10.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-25" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:200px;y:133px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:600;sp:2000;sR:600;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6400;"
											style="z-index:15;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon11.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-26" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:300px;y:133px;"
											data-text="l:22;a:inherit;"
											data-dim="w:100px;h:67px;"
											data-border="bow:0px;"
											data-frame_0="x:center;y:middle;sX:0;sY:0;"
											data-frame_1="e:power4.out;st:650;sp:2000;sR:650;"
											data-frame_999="x:center;y:middle;sX:0;sY:0;o:0;st:a;sp:1000;sR:6350;"
											style="z-index:16;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="100px" data-hh="67px" width="200" height="133" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addon12.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-27" 
											class="tp-shape tp-shapewrapper"
											data-type="shape"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="fw:700;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-border="bos:solid;boc:rgba(0,0,0,0.5);bow:0px;"
											data-frame_1="sR:10;"
											data-frame_999="o:0;st:w;sR:8690;"
											style="z-index:17;background:linear-gradient(180deg, rgba(0,0,0,0) 0%, rgba(0,0,0,0.00) 0%, rgba(0,0,0,0.65) 100%);"
										> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-28" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="l:22;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-border="bow:0px;"
											data-frame_0="y:bottom;sX:2;sY:2;"
											data-frame_1="st:a;sp:1000;sR:100;"
											data-frame_999="o:0;st:a;sp:700;sR:7900;"
											style="z-index:18;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="full-proportional" data-hh="full-proportional" width="640" height="370" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/addons_bg-2.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-2" 
											data-type="text"
											data-color="#ffffff"
											data-rsp_ch="on"
											data-xy="x:c;y:c;yo:-20px;"
											data-text="s:25;l:30;fw:700;a:center;"
											data-border="bow:0px;"
											data-frame_0="y:-50;"
											data-frame_1="e:power4.out;st:160;sp:1000;sR:160;"
											data-frame_999="y:50;o:0;e:power4.out;st:a;sp:600;sR:7840;"
											style="z-index:19;font-family:Roboto;"
										>25+<br/>Plugin Add-Ons 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-12" 
											data-type="text"
											data-color="#ffffff"
											data-rsp_ch="on"
											data-xy="x:c;y:c;yo:-20px;"
											data-text="s:25;l:30;fw:700;a:center;"
											data-border="bow:0px;"
											data-frame_0="y:-50;"
											data-frame_1="e:power4.out;st:a;sp:1000;sR:100;"
											data-frame_999="y:50px;o:0;e:power4.out;st:a;sp:600;sR:7900;"
											style="z-index:20;font-family:Roboto;"
										>Special Effects &<br/>
						Added Functionality 
										</rs-layer><!--

										--><rs-layer
											id="slide-2449-layer-4" 
											class="rev-btn"
											data-type="button"
											data-color="rgba(255,255,255,1)"
											data-xy="x:c;y:c;yo:40px;"
											data-text="s:15;l:34;fw:500;a:inherit;"
											data-rsp_bd="off"
											data-padding="r:35;l:35;"
											data-border="bos:solid;boc:rgba(0,0,0,1);bow:0px;bor:3px,3px,3px,3px;"
											data-frame_0="y:50;"
											data-frame_1="e:power4.out;st:500;sp:1000;sR:500;"
											data-frame_999="o:0;e:power4.inOut;st:w;sp:1000;sR:7500;"
											data-frame_hover="c:rgba(255,255,255,1);bgc:#0a447a;boc:rgba(0,0,0,1);bor:3px,3px,3px,3px;bos:solid;oX:50;oY:50;sp:300;"
											style="z-index:21;background-color:#006dd2;font-family:Roboto;"
										>Show Modules 
										</rs-layer><!--

										--><a
											href="//revolution.themepunch.com/addons/"
											target="_blank"
											class="tp-shape tp-shapewrapper rs-layer"
											data-type="shape"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="fw:700;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-actions='o:mouseenter;a:stoplayer;layer:slide-2449-layer-2;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-1;d:20;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-16;d:40;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-17;d:60;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-18;d:80;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-19;d:100;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-20;d:120;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-21;d:140;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-22;d:160;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-23;d:180;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-24;d:200;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-25;d:220;||o:mouseenter;a:stoplayer;layer:slide-2449-layer-26;d:240;||o:mouseenter;a:startlayer;layer:slide-2449-layer-12;||o:mouseleave;a:stoplayer;layer:slide-2449-layer-12;||o:mouseleave;a:startlayer;layer:slide-2449-layer-2;||o:mouseleave;a:startlayer;layer:slide-2449-layer-1;d:20;||o:mouseleave;a:startlayer;layer:slide-2449-layer-16;d:40;||o:mouseleave;a:startlayer;layer:slide-2449-layer-17;d:60;||o:mouseleave;a:startlayer;layer:slide-2449-layer-18;d:80;||o:mouseleave;a:startlayer;layer:slide-2449-layer-19;d:100;||o:mouseleave;a:startlayer;layer:slide-2449-layer-20;d:120;||o:mouseleave;a:startlayer;layer:slide-2449-layer-21;d:140;||o:mouseleave;a:startlayer;layer:slide-2449-layer-22;d:160;||o:mouseleave;a:startlayer;layer:slide-2449-layer-23;d:180;||o:mouseleave;a:startlayer;layer:slide-2449-layer-24;d:200;||o:mouseleave;a:startlayer;layer:slide-2449-layer-25;d:220;||o:mouseleave;a:startlayer;layer:slide-2449-layer-26;d:240;||o:mouseenter;a:startlayer;layer:slide-2449-layer-28;||o:mouseleave;a:stoplayer;layer:slide-2449-layer-28;'
											data-basealign="slide"
											data-border="bos:solid;boc:rgba(0,0,0,0.5);bow:0px;"
											data-frame_1="sR:10;"
											data-frame_999="o:0;st:w;sR:8690;"
											style="z-index:22;"
											id="2449a"
										> 
										</a><!--
									--></rs-slide>
								</rs-slides>								
							</rs-module>
						</rs-module-wrap>
