<?php

// This file `jetpack_autoload_psr4.php` was auto generated by automattic/jetpack-autoloader.

$vendorDir = dirname(__DIR__);
$baseDir   = dirname($vendorDir);

return array(
	'Composer\\Installers\\' => array(
		'version' => '1.11.0.0',
		'path'    => array( $vendorDir . '/composer/installers/src/Composer/Installers' )
	),
	'Automattic\\WooCommerce\\Blocks\\Tests\\' => array(
		'version' => '6.1.0.0',
		'path'    => array( $baseDir . '/tests/php' )
	),
	'Automattic\\WooCommerce\\Blocks\\' => array(
		'version' => '6.1.0.0',
		'path'    => array( $baseDir . '/src' )
	),
	'Automattic\\Jetpack\\Autoloader\\' => array(
		'version' => '2.10.4.0',
		'path'    => array( $vendorDir . '/automattic/jetpack-autoloader/src' )
	),
);
