.wc-block-checkout-error {
	padding: $gap-largest;
	text-align: center;
	width: 100%;

	.wc-block-checkout-error__image {
		max-width: 150px;
		margin: 0 auto 1em;
		display: block;
		color: inherit;
	}
	.wc-block-checkout-error__title {
		display: block;
		margin: 0;
		font-weight: bold;
	}
	.wc-block-checkout-error__description {
		display: block;
		margin: 0.25em 0 1em 0;
	}
}
