/**
 * External dependencies
 */
import { __ } from '@wordpress/i18n';

/**
 * Internal dependencies
 */
import formStepAttributes from '../../form-step/attributes';

export default {
	...formStepAttributes( {
		defaultTitle: __( 'Payment options', 'woo-gutenberg-products-block' ),
		defaultDescription: '',
	} ),
	lock: {
		type: 'object',
		default: {
			move: true,
			remove: true,
		},
	},
};
