.wp-block-button.wc-block-components-product-button {
	word-break: break-word;
	white-space: normal;
	margin-top: 0;
	margin-bottom: $gap-small;

	.wc-block-components-product-button__button {
		word-break: break-word;
		white-space: normal;
		margin: 0 auto;
		display: inline-flex;
		justify-content: center;
	}

	.wc-block-components-product-button__button--placeholder {
		@include placeholder();
		min-width: 8em;
		min-height: 3em;
	}
}

.is-loading .wc-block-components-product-button > .wc-block-components-product-button__button {
	@include placeholder();
	min-width: 8em;
	min-height: 3em;
}


.theme-twentytwentyone {
	// Prevent buttons appearing disabled in the editor.
	.editor-styles-wrapper .wc-block-components-product-button .wp-block-button__link {
		background-color: var(--button--color-background);
		color: var(--button--color-text);
		border-color: var(--button--color-background);
	}
}
