<?php
namespace Automattic\WooCommerce\Blocks\BlockTypes;

/**
 * ActiveFilters class.
 */
class ActiveFilters extends AbstractBlock {
	/**
	 * Block name.
	 *
	 * @var string
	 */
	protected $block_name = 'active-filters';
}
