/**
 * External dependencies
 */
import { SVG } from 'wordpress-components';

const FormStep = (
	<SVG xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
		<g fillRule="evenodd">
			<path d="M20 6a1 1 0 011 1v13a1 1 0 01-1 1H4a1 1 0 01-1-1V7a1 1 0 011-1h16zm-1 2H5v11h14V8z" />
			<path d="M3 4h8v3H3zM18 10v3H6v-3h12zm-3 1H7v1h8v-1zM18 14v3H6v-3h12zm-3 1H7v1h8v-1z" />
		</g>
	</SVG>
);

export default FormStep;
