.woocommerce-leaderboard.woocommerce-empty-content{margin-bottom:24px}.woocommerce-leaderboard .woocommerce-card__body{padding:0}.woocommerce-leaderboard .woocommerce-table__table{margin-bottom:0}.woocommerce-leaderboard .woocommerce-table__table tr:last-child{border-bottom-style:none}:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}.woocommerce-dashboard__dashboard-leaderboards .components-base-control__field{width:100%}.woocommerce-dashboard__dashboard-leaderboards .components-select-control__input{border:1px solid #ccc;height:34px}.woocommerce-dashboard__dashboard-leaderboards .woocommerce-dashboard__dashboard-leaderboards__select .components-base-control__field{padding:0 12px 4px}.woocommerce-dashboard__dashboard-leaderboards .woocommerce-dashboard__dashboard-leaderboards__select .woocommerce-ellipsis-menu__title{padding:10px 0 14px}