.components-disabled .wc-block-price-filter__range-input-wrapper .wc-block-price-filter__range-input {
	&::-webkit-slider-thumb {
		pointer-events: none;
	}
	&::-moz-range-thumb {
		pointer-events: none;
	}
	&::-ms-thumb {
		pointer-events: none;
	}
}
.wc-block-price-slider {
	.components-placeholder__instructions {
		border-bottom: 1px solid #e0e2e6;
		width: 100%;
		padding-bottom: 1em;
		margin-bottom: 2em;
	}
	.components-placeholder__label svg {
		fill: currentColor;
		margin-right: 1ch;
	}
	.components-placeholder__fieldset {
		display: block; /* Disable flex box */
	}
	.wc-block-price-slider__add-product-button {
		margin: 0 0 1em;
		vertical-align: middle;
		height: auto;
		padding: 0.5em 1em;

		svg {
			fill: currentColor;
			margin-left: 0.5ch;
			vertical-align: middle;
		}
	}
	.wc-block-price-slider__read_more_button {
		display: block;
		margin-bottom: 1em;
	}
}
