.notice.wcs-nux__notice{display:flex;padding:0;border:0}.notice.wcs-nux__notice .wcs-nux__notice-logo{display:flex;flex-direction:column;justify-content:center;margin-left:1em;padding:1.5em 2em}.notice.wcs-nux__notice .wcs-nux__notice-logo .wcs-nux__notice-logo-jetpack{width:135px;margin:-10px 0 .75em 21px}.notice.wcs-nux__notice .wcs-nux__notice-logo .wcs-nux__notice-logo-graphic{width:220px;height:121px}.notice.wcs-nux__notice .wcs-nux__notice-logo.is-compact{padding:1.5em 1em 1.5em 1.5em}.notice.wcs-nux__notice .wcs-nux__notice-logo.is-compact .wcs-nux__notice-logo-graphic{width:100px;height:auto}.notice.wcs-nux__notice .wcs-nux__notice-content{display:flex;flex-direction:column;justify-content:center;padding:1em 1em 2.5em;margin-left:1em;color:#52768f}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-title{color:#4f748e;font-size:1.3em;font-weight:600}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-tos{font-size:.8em;color:#7591a5;margin:0 0 1.5em;padding:0}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-tos a{color:#49a8ce}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-button{align-self:flex-start;background-color:#00aadc;border-color:#0087be;box-shadow:0 1px 0 #0087be;color:#fff;font-size:1.1em;height:auto;padding:.45em 1.5em;text-shadow:none}.notice.wcs-nux__notice .wcs-nux__notice-content-text{margin:0 2.5em 1em 0;line-height:1.6;padding:0}@media (max-width:1000px){.notice.wcs-nux__notice{flex-direction:column}.notice.wcs-nux__notice .wcs-nux__notice-content{margin:0 1em}.notice.wcs-nux__notice .wcs-nux__notice-logo{display:none}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-tos{margin:1.5em 0 .5em;max-width:90%}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-text{margin:1em 0 0;max-width:100%}.notice.wcs-nux__notice .wcs-nux__notice-content .wcs-nux__notice-content-button{-ms-grid-row-align:auto;align-self:auto;margin:1.5em 0 0;text-align:center}.notice.wcs-nux__notice .wcs-nux__notice-jetpack{flex-direction:row;padding:.5em}.notice.wcs-nux__notice .wcs-nux__notice-jetpack img{height:2em;width:2em;margin:0 1em 0 0}.notice.wcs-nux__notice .wcs-nux__notice-jetpack .wcs-nux__notice-jetpack-text{margin:0}}