/**
 * External dependencies
 */
import { Placeholder } from 'wordpress-components';

const GroupedProducts = () => {
	return (
		<Placeholder className="wc-block-components-product-add-to-cart-group-list">
			This is a placeholder for the grouped products form element.
		</Placeholder>
	);
};

export default GroupedProducts;
