(window.webpackWcBlocksJsonp=window.webpackWcBlocksJsonp||[]).push([[15],{364:function(t,c){},391:function(t,c,e){"use strict";e.r(c);var n=e(4),o=e.n(n),a=e(1),u=(e(10),e(6)),s=e.n(u),r=e(101),l=e(249);e(364),c.default=Object(l.withProductDataContext)((function(t){var c=t.className,e=Object(r.useInnerBlockLayoutContext)().parentClassName,n=Object(r.useProductDataContext)().product.sku;return n?React.createElement("div",{className:s()(c,"wc-block-components-product-sku",o()({},"".concat(e,"__product-sku"),e))},Object(a.__)("SKU:",'woocommerce')," ",React.createElement("strong",null,n)):null}))}}]);