/**
 * External dependencies
 */
import { SVG } from 'wordpress-components';
// 🤫
const stonks = (
	<SVG xmlns="http://www.w3.org/2000/SVG" viewBox="0 0 24 24">
		<path fill="none" d="M0 0h24v24H0V0z" />
		<path d="M3.5 18.49l6-6.01 4 4L22 6.92l-1.41-1.41-7.09 7.97-4-4L2 16.99l1.5 1.5z" />
	</SVG>
);

export default stonks;
