/**
 * @typedef {Object} BillingData
 *
 * @property {string} email      The email for the billing address
 * @property {string} first_name First name of billing customer
 * @property {string} last_name  Last name of billing customer
 * @property {string} company    Company name for the billing address
 * @property {string} address_1  First line of the billing address
 * @property {string} address_2  Second line of the billing address
 * @property {string} city       The city of the billing address
 * @property {string} state      The state of the billing address (ISO or name)
 * @property {string} postcode   The postal or zip code for the billing address
 * @property {string} country    The country for the billing address (ISO)
 * @property {string} phone      The phone number for the billing address
 */

export {};
