:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}[data-gateway_id=pre_install_woocommerce_payments_promotion]>.status a{visibility:hidden!important}[data-gateway_id=pre_install_woocommerce_payments_promotion]>.action .is-secondary,[data-gateway_id=pre_install_woocommerce_payments_promotion]>.action.is-secondary:hover{box-shadow:none}[data-gateway_id=pre_install_woocommerce_payments_promotion]>.action a{position:relative;display:none}.wc_gateways .name .wc-payment-gateway-method-name{display:inline-flex}.wc_gateways .name .wc-payment-gateway-method-name img{height:24px;width:auto;margin-right:4px}.wc_gateways .wc-payment-gateway-method_name{display:flex}.wc_gateways .wc-payment-gateway-method_name .pre-install-payment-gateway_subtitle{margin-right:16px}.wc_gateways .wc-payment-gateway-method_name .pre-install-payment-gateway_subtitle>img{height:24px;width:auto;margin-right:4px}