(window.webpackWcBlocksJsonp=window.webpackWcBlocksJsonp||[]).push([[18],{366:function(t,e){},393:function(t,e,c){"use strict";c.r(e);var a=c(4),n=c.n(a),o=c(1),s=(c(10),c(6)),l=c.n(s),u=c(101),r=c(7),i=c(249);c(366),e.default=Object(i.withProductDataContext)((function(t){var e=t.className,c=Object(u.useInnerBlockLayoutContext)().parentClassName,a=Object(u.useProductDataContext)().product;return Object(r.isEmpty)(a.tags)?null:React.createElement("div",{className:l()(e,"wc-block-components-product-tag-list",n()({},"".concat(c,"__product-tag-list"),c))},Object(o.__)("Tags:",'woocommerce')," ",React.createElement("ul",null,Object.values(a.tags).map((function(t){var e=t.name,c=t.link,a=t.slug;return React.createElement("li",{key:"tag-list-item-".concat(a)},React.createElement("a",{href:c},e))}))))}))}}]);