.wc-block-components-shipping-rates-control__package {
	.wc-block-components-panel__button {
		margin-bottom: 0;
		margin-top: 0;
		padding-bottom: em($gap-small);
		padding-top: em($gap-small);
	}

	// Remove panel padding because we are adding bottom padding to `.wc-block-components-radio-control`
	// and `.wc-block-components-radio-control__option-layout` in the next ruleset.
	.wc-block-components-panel__content {
		padding-bottom: 0;
	}

	.wc-block-components-radio-control,
	.wc-block-components-radio-control__option-layout {
		padding-bottom: 0;
	}

	.wc-block-components-radio-control .wc-block-components-radio-control__option-layout {
		padding-bottom: 0;
	}
}

.wc-block-components-shipping-rates-control__package-items {
	@include font-size(small);
	display: block;
	list-style: none;
	margin: 0;
	padding: 0;
}

.wc-block-components-shipping-rates-control__package-item {
	@include wrap-break-word();
	display: inline-block;
	margin: 0;
	padding: 0;
}

.wc-block-components-shipping-rates-control__package-item:not(:last-child)::after {
	content: ", ";
	white-space: pre;
}
