.wcmp-report-abuse-error {
    color: red;
    font-size: 12px;
}

.wcmp_fpm_buttons {
    right:3px;
    left:auto;
}
.wcmp_fpm_buttons {
    position:absolute;
    color:#fff;
    background-color:#222;
    background-color:rgba(16,16,16,0.3);
    top:3px;
    left:50%;
    margin-left:-33px;
    z-index:99;
    width:30px;
    height:auto;
    padding:3px;
    border-radius:3px;
}

.wcmp_fpm_buttons .wcmp_fpm_button {
    display:block;
    float:right;
    width:24px;
    height:24px;
    line-height:24px;
    margin:0px;
    text-align:center;
    border-radius:3px;
    font-size:18px;
    color:#fff;
    text-decoration:none;
}

.wcmp_fpm_buttons .wcmp_fpm_button_separator {
    color: #fff;
}

.wcmp-report-abouse-wrapper .report-abouse-modal{
    display: none; /* Hidden by default */
    position: fixed; /* Stay in place */
    z-index: 100000; /* Sit on top */
    padding-top: 100px; /* Location of the box */

    right: 0;
    top: 0;
    width: 100%; /* Full width */
    height: 100%; /* Full height */
    overflow: auto; /* Enable scroll if needed */
    background-color: rgb(0,0,0); /* Fallback color */
    background-color: rgba(0,0,0,0.4); /* Black w/ opacity */
}
.wcmp-report-abouse-wrapper .modal-content{
    background-color: #fefefe;
    margin: auto;
    padding: 0;
    border: 1px solid #888;
    width: 600px;
    max-width: 90%;
    position: absolute;
    right: 50%;
    top: 50%;
    -webkit-transform: translate(50%, -50%);
        -ms-transform: translate(50%, -50%);
            transform: translate(50%, -50%);
    max-height: 90vh;
    overflow: auto;
    text-align: right;
    box-shadow: 0 4px 8px 0 rgba(0,0,0,0.2),0 6px 20px 0 rgba(0,0,0,0.19);
    -webkit-animation-name: animatetop;
    -webkit-animation-duration: 0.4s;
    animation-name: animatetop;
    animation-duration: 0.4s;
}
.wcmp-report-abouse-wrapper .modal-header button.close {
    padding: 3px;
    cursor: pointer;
    background: transparent;
    border: 0;
    min-height: auto;
    -webkit-appearance: none;
    margin: 0;
    float: left;
    font-size: 20px;
    font-weight: bold;
    line-height: 20px;
    color: #000000;
    text-shadow: 0 1px 0 #ffffff;
    opacity: 0.2;
} 
/* modla header */
.wcmp-report-abouse-wrapper .modal-content .modal-header {
    padding: 16px 20px;
    border-bottom: 1px solid #eee;
    position: relative;
}
.wcmp-report-abouse-wrapper .modal-header h2{
    margin: 0;
    font-size: 18px;
    display: inline-block;
}
.wcmp-report-abouse-wrapper .modal-content .modal-footer {
    padding: 10px 20px;
    margin-bottom: 0;
    text-align: left;
    background-color: #f5f5f5;
    border-top: 1px solid #ddd;
    z-index: 1;
    position: relative;
}
.wcmp-report-abouse-wrapper textarea {
    margin: 0 0 6px;
    font-size: 15px;
    line-height: 16px;
    font-weight: normal;
    color: #616161;
}
.wcmp-report-abouse-wrapper .modal-content .modal-body{
    max-height: 52vh;
    overflow: auto;
    margin: 0;
    width: 100%;
    padding: 20px 20px 0 20px;
}
.wcmp-report-abouse-wrapper .toggle-content{     
    padding: 10px;
    border: 1px solid #ddd; 
}
.wcmp-report-abouse-wrapper .toggle-content .close{ display: none; },.wcmp-report-abuse-error{color:red;font-size:12px}.wcmp_fpm_buttons{right:3px;left:auto}.wcmp_fpm_buttons{position:absolute;color:#fff;background-color:#222;background-color:rgba(16,16,16,.3);top:3px;left:50%;margin-left:-33px;z-index:99;width:30px;height:auto;padding:3px;border-radius:3px}.wcmp_fpm_buttons .wcmp_fpm_button{display:block;float:right;width:24px;height:24px;line-height:24px;margin:0;text-align:center;border-radius:3px;font-size:18px;color:#fff;text-decoration:none}.wcmp_fpm_buttons .wcmp_fpm_button_separator{color:#fff}.wcmp-report-abouse-wrapper .report-abouse-modal{display:none;position:fixed;z-index:100000;padding-top:100px;right:0;top:0;width:100%;height:100%;overflow:auto;background-color:#000;background-color:rgba(0,0,0,.4)}.wcmp-report-abouse-wrapper .modal-content{background-color:#fefefe;margin:auto;padding:0;border:1px solid #888;width:600px;max-width:90%;position:absolute;right:50%;top:50%;-webkit-transform:translate(50%,-50%);-ms-transform:translate(50%,-50%);transform:translate(50%,-50%);max-height:90vh;overflow:auto;text-align:right;box-shadow:0 4px 8px 0 rgba(0,0,0,.2),0 6px 20px 0 rgba(0,0,0,.19);-webkit-animation-name:animatetop;-webkit-animation-duration:.4s;animation-name:animatetop;animation-duration:.4s}.wcmp-report-abouse-wrapper .modal-header button.close{padding:3px;cursor:pointer;background:100% 0;border:0;min-height:auto;-webkit-appearance:none;margin:0;float:left;font-size:20px;font-weight:700;line-height:20px;color:#000;text-shadow:0 1px 0 #fff;opacity:.2}.wcmp-report-abouse-wrapper .modal-content .modal-header{padding:16px 20px;border-bottom:1px solid #eee;position:relative}.wcmp-report-abouse-wrapper .modal-header h2{margin:0;font-size:18px;display:inline-block}.wcmp-report-abouse-wrapper .modal-content .modal-footer{padding:10px 20px;margin-bottom:0;text-align:left;background-color:#f5f5f5;border-top:1px solid #ddd;z-index:1;position:relative}.wcmp-report-abouse-wrapper textarea{margin:0 0 6px;font-size:15px;line-height:16px;font-weight:400;color:#616161}.wcmp-report-abouse-wrapper .modal-content .modal-body{max-height:52vh;overflow:auto;margin:0;width:100%;padding:20px 20px 0 20px}.wcmp-report-abouse-wrapper .toggle-content{padding:10px;border:1px solid #ddd}.wcmp-report-abouse-wrapper .toggle-content .close{display:none}