/* fonts */
@font-face {
  font-family: 'wcmp-frontend-font';
  src: url("../fonts/wcmp-frontend-font.eot?vtl65e");
  src: url("../fonts/wcmp-frontend-font.eot?vtl65e#iefix") format("embedded-opentype"), url("../fonts/wcmp-frontend-font.ttf?vtl65e") format("truetype"), url("../fonts/wcmp-frontend-font.woff?vtl65e") format("woff");
  font-weight: normal;
  font-style: normal; }

/* fonts end */
/************ new *****************/
/*TYPE 1*/
/*TYPE 2*/
/*TYPE 3*/
i.wcmp-font {
  font-family: 'wcmp-frontend-font' !important;
  speak: none;
  font-style: normal;
  font-weight: normal;
  font-variant: normal;
  text-transform: none;
  line-height: 1;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale; }

.ico-facebook-icon:before {
  content: "\e902"; }

.ico-google-plus-icon:before {
  content: "\e903"; }

.ico-instagram-icon:before {
  content: "\e904"; }

.ico-linkedin-icon:before {
  content: "\e905"; }

.ico-twitter-icon:before {
  content: "\e908"; }

.ico-youtube-icon:before {
  content: "\e909"; }

.ico-badge-icon:before {
  content: "\e900"; }

.ico-call-icon:before {
  content: "\e901"; }

.ico-location-icon:before {
  content: "\e906"; }

.ico-mail-icon:before {
  content: "\e907"; }

.vendor_description_background {
  position: relative;
  line-height: 0.8; }
  .vendor_description_background * {
    box-sizing: border-box; }

.vendor_description {
  width: 100%;
  clear: both;
  display: inline-block;
  padding: 15px;
  background-color: rgba(0, 0, 0, 0.5);
  font-weight: bold; }
  .vendor_description .vendor_img_add {
    float: left;
    width: 280px;
    max-width: 100%; }

.wcmp_social_profile {
  /*position: absolute;*/
  right: 15px;
  bottom: 15px;
  z-index: 1; }
  .wcmp_social_profile a {
    display: inline-block;
    margin: 0 0 0 5px; }
  .wcmp_social_profile .wcmp-font {
    background: #fff;
    font-size: 27px; }
  .wcmp_social_profile .ico-facebook-icon {
    color: #4867aa; }
  .wcmp_social_profile .ico-twitter-icon {
    color: #1da1f2; }
  .wcmp_social_profile .ico-linkedin-icon {
    color: #007bb6; }
  .wcmp_social_profile .ico-google-plus-icon {
    color: #dd4c40; }
  .wcmp_social_profile .ico-youtube-icon {
    color: #ff0000; }
  .wcmp_social_profile .ico-instagram-icon {
    color: #517ea3; }

.vendor_address {
  margin-top: 8px;
  word-break: break-word; }
  .vendor_address p {
    margin: 0 0 10px;
    text-align: left;
    line-height: 1;
    font-weight: 400;
    color: inherit; }
    .vendor_address p img {
      height: 12px;
      margin-right: 14px;
      width: 12px;
      display: inline-block; }
  .vendor_address p.wcmp_vendor_name {
    color: inherit;
    font-weight: 600;
    font-size: 18px; }
  .vendor_address label {
    font-size: 14px;
    font-weight: 400;
    color: inherit;
    display: inline; }

a.wcmp_vendor_detail {
  margin: 0 0 10px;
  text-align: left;
  line-height: 1;
  font-weight: 400;
  color: inherit;
  cursor: pointer; }

.wcmp_vendor_rating {
  margin: 5px 0; }
  .wcmp_vendor_rating .star-rating {
    float: none;
    display: inline-block; }
  .wcmp_vendor_rating * {
    font-weight: 400;
    color: inherit;
    font-size: 14px;
    font-weight: 400;
    color: inherit; }
  .wcmp_vendor_rating p {
    margin: 0; }

.woocommerce .wcmp_vendor_rating .star-rating {
  float: none;
  display: inline-block; }

.img_div img {
  height: auto;
  max-width: 100px;
  max-height: 100px;
  width: auto; }

.social_profile {
  float: right; }
  .social_profile a {
    padding: 2px;
    display: inline-block; }

.dc-wpv-quick-info-wrapper #respond {
  padding: 0px; }

.success_review_msg {
  border: 1px solid;
  margin: 10px 0px;
  padding: 15px 10px 15px 50px;
  background-position: 10px center;
  color: #4F8A10;
  background-color: #DFF2BF; }

.error_review_msg {
  border: 1px solid;
  margin: 10px 0px;
  padding: 15px 10px 15px 50px;
  background-position: 10px center;
  color: #D8000C;
  background-color: #FFBABA; }

.wocommerce #wcmp_vendor_reviews {
  margin-top: 20px; }

.description_data {
  margin-top: 10px; }
  .description_data p {
    line-height: 1.2; }

.wcmp_vendor_banner {
  width: 100%;
  background-size: cover;
  background-repeat: no-repeat;
  position: relative; }
  .wcmp_vendor_banner > img {
    width: 100%;
    border-radius: 0; }

.wcmp_vendor_detail i {
  display: inline-block;
  vertical-align: middle;
  margin: 0 6px 0 0; }

.wcmp_vendor_banner_template {
  margin-bottom: 35px; }
  .wcmp_vendor_banner_template .wcmp_total_rating_number {
    display: inline-block;
    vertical-align: middle; }
    .wcmp_vendor_banner_template .wcmp_total_rating_number + a {
      display: inline-block;
      vertical-align: middle; }
      .wcmp_vendor_banner_template .wcmp_total_rating_number + a .star-rating {
        display: inline-block;
        vertical-align: middle; }

.wcmp_vendor_banner_template.template1 .vendor_description {
  position: absolute;
  top: 0;
  left: 0;
  height: 100%; }
  .wcmp_vendor_banner_template.template1 .vendor_description .vendor_img_add {
    width: auto;
    max-width: 100%; }

.wcmp_vendor_banner_template.template1 .img_div img {
  border: 2px solid #fff;
  max-width: 70px;
  max-height: 70px; }

.wcmp_vendor_banner_template.template1 .wcmp_vendor_banner {
  overflow: hidden; }

.wcmp_vendor_banner_template.template1 .vendor_address {
  color: #fff; }

.wcmp_vendor_banner_template.template1 .wcmp_vendor_name {
  display: inline-block;
  vertical-align: middle;
  margin-bottom: 5px; }

.wcmp_vendor_banner_template.template1 .vendor_badge {
  margin: 0 0 5px 10px; }

.wcmp_vendor_banner_template.template1 .description_data {
  border: 1px solid #ddd;
  margin: 0;
  padding: 20px;
  background: #fbfbfb; }
  .wcmp_vendor_banner_template.template1 .description_data p {
    margin: 0; }

.wcmp_vendor_banner_template.template1 .external_store_url {
  margin: 1px 0 0; }
  .wcmp_vendor_banner_template.template1 .external_store_url a {
    color: #fff; }

.wcmp_vendor_banner_template.template2 {
  border: 1px solid #e8e8e8; }
  .wcmp_vendor_banner_template.template2 .vendor_description {
    position: relative;
    padding-top: 0;
    margin-top: -63px;
    background: transparent;
    text-align: center; }
    .wcmp_vendor_banner_template.template2 .vendor_description .vendor_img_add {
      margin: 0 auto;
      float: none;
      text-align: center;
      width: 100%; }
  .wcmp_vendor_banner_template.template2 .vendor_address {
    color: #444; }
    .wcmp_vendor_banner_template.template2 .vendor_address p.wcmp_vendor_name {
      margin: 7px 0 11px 0; }
    .wcmp_vendor_banner_template.template2 .vendor_address p {
      text-align: center; }
  .wcmp_vendor_banner_template.template2 .img_div img {
    border: 2px solid #fff;
    max-width: 125px;
    max-height: 125px;
    margin: 0 auto;
    border-radius: 50%;
    box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
    -moz-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
    -ms-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
    -o-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
    -webkit-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3); }
  .wcmp_vendor_banner_template.template2 .vendor_description_background {
    border: solid 1px #eaeaea; }
  .wcmp_vendor_banner_template.template2 .wcmp_vendor_rating .star-rating {
    float: none;
    display: inline-block; }
  .wcmp_vendor_banner_template.template2 .wcmp_total_rating_number {
    float: none; }
  .wcmp_vendor_banner_template.template2 .wcmp_vendor_detail {
    display: inline-block;
    padding: 0;
    margin: 5px 10px; }
  .wcmp_vendor_banner_template.template2 .external_store_url {
    margin: 0 0 10px 0; }

.wcmp_vendor_banner_template.template3 {
  border: 1px solid #e8e8e8;
  background: #f3f3f3; }
  .wcmp_vendor_banner_template.template3 .vendor_description {
    position: relative;
    padding-top: 0;
    margin-top: -55px;
    background: transparent; }
    .wcmp_vendor_banner_template.template3 .vendor_description .vendor_img_add {
      width: 100%; }
  .wcmp_vendor_banner_template.template3 .img_div {
    float: left; }
    .wcmp_vendor_banner_template.template3 .img_div img {
      border: 2px solid #fff;
      border-radius: 50%;
      box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
      -moz-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
      -ms-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
      -o-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3);
      -webkit-box-shadow: 0px 2px 4px rgba(0, 0, 0, 0.3); }
  .wcmp_vendor_banner_template.template3 .vendor_description_background {
    border: solid 1px #eaeaea; }
  .wcmp_vendor_banner_template.template3 .vendor_address {
    color: #444;
    float: left;
    margin-left: 15px; }
  .wcmp_vendor_banner_template.template3 .wcmp_vendor_rating .star-rating {
    float: none;
    display: inline-block; }
  .wcmp_vendor_banner_template.template3 .wcmp_total_rating_number {
    float: none; }
  .wcmp_vendor_banner_template.template3 .wcmp_vendor_detail {
    display: inline-block;
    padding: 0;
    margin: 10px 15px 0 0; }
  .wcmp_vendor_banner_template.template3 .wcmp_vendor_name {
    color: #fff;
    margin: 16px 0 21px 0;
    font-size: 24px;
    text-shadow: 1px 1px 2px rgba(0, 0, 0, 0.4);
    width: auto; }
  .wcmp_vendor_banner_template.template3 .vendor_img_add {
    display: flex; }
  .wcmp_vendor_banner_template.template3 .description_data {
    margin-top: 15px; }
  .wcmp_vendor_banner_template.template3 .external_store_url {
    margin: 10px 0; }

.wcmp_review_loader {
  width: 100%;
  position: relative;
  z-index: 8888;
  background-color: none;
  opacity: 0.5;
  height: 40px;
  text-align: center;
  display: none; }
  .wcmp_review_loader img {
    margin: 2px auto; }

  .wcmp_store_coupons span.wcmp-store-coupon-single.tips.text_tip {
    border: 1px dashed #17a2b8;
    background-color: #d9f2f6;
    padding: 5px 10px;
    margin-right: 10px;
    float: left;
    margin-top: 10px;
    font-weight: 600;
    color: #222;
    font-size: 13px;
    line-height: 20px;
  }

@media screen and (max-width: 480px) {
  .wcmp_vendor_banner_template.template1 .vendor_description {
    padding-bottom: 50px; }
  .wcmp_vendor_banner_template.template1 .wcmp_social_profile {
    left: 10px;
    right: auto;
    bottom: 14px; }
  .wcmp_vendor_banner_template.template2 .vendor_description {
    margin-top: 12px; }
  .wcmp_vendor_banner_template.template3 .vendor_img_add {
    display: block; }
  .wcmp_vendor_banner_template.template3 .wcmp_vendor_name {
    margin-top: 10px;
    font-size: 20px; } }

@media screen and (max-width: 1100px) {
  .wcmp_vendor_banner_template.template1 .vendor_description {
    position: relative;
    background: #fbfbfb;
    border: 1px solid #ddd;
    border-bottom: 0; }
  .wcmp_vendor_banner_template.template1 .vendor_address {
    color: inherit; }
  body .wcmp_vendor_banner_template.template1 .description_data {
    border-top: 0;
    padding: 0 15px 20px; } }

@media screen and (max-width: 640px) {
  .wcmp_vendor_banner_template.template2 .vendor_description {
    margin-top: 10px; } }

@media screen and (max-width: 991px) {
  .wcmp_vendor_banner_template.template3 .vendor_description {
    margin-top: 10px; }
    .wcmp_vendor_banner_template.template3 .vendor_description .vendor_img_add {
      width: 100%; }
  .wcmp_vendor_banner_template.template3 .wcmp_vendor_name {
    color: inherit;
    text-shadow: none;
    margin: 16px 0 15px 0;
    width: 100%; } }

@media screen and (max-width: 480px) {
  .wcmp_social_profile {
    right: 10px;
    bottom: 10px; }
    .wcmp_social_profile .wcmp-font {
      font-size: 16px; }
  .wcmp_vendor_banner_template .vendor_img_add .img_div img {
    max-width: 70px;
    max-height: 70px; } }

/*# sourceMappingURL=../../../none */