.wc-block-components-shipping-calculator-address {
	margin-bottom: 0;
}

.wc-block-components-shipping-calculator-address__button {
	width: 100%;
	margin-top: em($gap-large);
}

.wc-block-components-shipping-calculator {
	padding: em($gap-smaller) 0 em($gap-small);
}
