.wc-block-components-totals-discount__coupon-list {
	list-style: none;
	margin: 0;
	padding: 0;
}

.wc-block-components-totals-discount .wc-block-components-totals-item__value {
	color: $discount-color;
}
