<?php
/**
 * Template for displaying the sep field
 *
 * @var array $field The field.
 * @package YITH\PluginFramework\Templates\Fields
 */

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

/**
 * Do nothing, allow to stamp the container of this option with sep class:
 * <div class="the-metabox sep clearfix"></div>
 */
