.wc-block-view-switch-control {
	text-align: left;
	background: #f0f2f3;
	box-shadow: 0 0 0 13px #f0f2f3;
	margin: 0 0 27px;
	visibility: hidden;
	color: $gray-700;
}
.has-child-selected,
.is-selected {
	.wc-block-view-switch-control {
		visibility: visible;
	}
}
