:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}.woocommerce-recommended-payments-card{margin:0 15px 10px 0;animation:isLoaded;animation-duration:.25s}.woocommerce-recommended-payments-card .woocommerce-list__item>.woocommerce-list__item-inner{align-items:flex-start}.woocommerce-recommended-payments-card .woocommerce-list__item:hover{background-color:#fff}.woocommerce-recommended-payments-card .woocommerce-list__item:hover .woocommerce-list__item-title{color:#1e1e1e}.woocommerce-recommended-payments-card .woocommerce-list__item-title{font-size:14px;color:#1e1e1e;font-weight:600}.woocommerce-recommended-payments-card .woocommerce-review-activity-card__section-controls{text-align:center}.woocommerce-recommended-payments-card .woocommerce-pill{margin-left:4px;padding:2px 8px}@media(max-width:480px){.woocommerce-recommended-payments-card .woocommerce-pill{margin-top:4px;margin-bottom:4px}}.woocommerce-recommended-payments-card .components-card__footer .gridicon{margin-left:4px}.woocommerce-recommended-payments-card .woocommerce-list__item-enter{opacity:0;max-height:100vh;transform:none}.woocommerce-recommended-payments-card .woocommerce-list__item-enter-active{opacity:1;transition:opacity .2s}.woocommerce-recommended-payments-card .woocommerce-list__item-after .components-button{margin-left:12px}.woocommerce-recommended-payments-card .woocommerce-list__item-text,.woocommerce-recommended-payments-card .woocommerce-recommended-payments__header-heading{max-width:749px}@media(max-width:782px){.woocommerce-recommended-payments-card{margin:0 0 10px}}