.components-toggle-button-control {
	.components-base-control__field {
		flex-wrap: wrap;
	}
	.components-toggle-button-control__label {
		width: 100%;
		margin-bottom: 8px;
		display: block;
	}
	.components-base-control__help {
		margin-top: 0;
	}
}
