this.wc=this.wc||{},this.wc.onboardingTaxNotice=function(e){var t={};function n(r){if(t[r])return t[r].exports;var o=t[r]={i:r,l:!1,exports:{}};return e[r].call(o.exports,o,o.exports,n),o.l=!0,o.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"==typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var o in e)n.d(r,o,function(t){return e[t]}.bind(null,o));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e.default}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=458)}({13:function(e,t,n){"use strict";n.d(t,"a",(function(){return s})),n.d(t,"b",(function(){return a})),n.d(t,"c",(function(){return d})),n.d(t,"f",(function(){return l})),n.d(t,"g",(function(){return f})),n.d(t,"e",(function(){return m})),n.d(t,"d",(function(){return w}));var r=n(2);const o=["wcAdminSettings","preloadSettings"],c="object"==typeof wcSettings?wcSettings:{},i=Object.keys(c).reduce((e,t)=>(o.includes(t)||(e[t]=c[t]),e),{});Object.keys(c.admin||{}).forEach(e=>{o.includes(e)||(i[e]=c.admin[e])});const u=i.adminUrl,s=(i.countries,i.currency),a=i.locale,d=i.orderStatuses;i.siteTitle,i.wcAssetUrl;function l(e,t=!1,n=(e=>e)){if(o.includes(e))throw new Error(Object(r.__)("Mutable settings should be accessed via data store."));return n(i.hasOwnProperty(e)?i[e]:t,t)}function f(e,t,n=(e=>e)){if(o.includes(e))throw new Error(Object(r.__)("Mutable settings should be mutated via data store."));i[e]=n(t)}function m(e){return(u||"")+e}function w(e){return new Promise((t,n)=>{document.querySelector(`#${e.handle}-js`)&&t();const r=document.createElement("script");r.src=e.src,r.id=e.handle+"-js",r.async=!0,r.onload=t,r.onerror=n,document.body.appendChild(r)})}},2:function(e,t){e.exports=window.wp.i18n},458:function(e,t,n){"use strict";n.r(t);var r=n(2),o=n(7),c=n(51),i=n.n(c),u=n(13);const s=e=>{if(e&&!e.disabled){return new Promise(e=>{window.requestAnimationFrame(e)}).then(()=>s(e))}return Promise.resolve(!0)},a=()=>{const e=document.querySelector(".woocommerce-save-button");e.classList.contains("has-tax")||s(e).then(()=>{document.querySelector(".wc_tax_rates .tips")&&(e.classList.add("has-tax"),Object(o.dispatch)("core/notices").createSuccessNotice(Object(r.__)("You've added your first tax rate!",'woocommerce'),{id:"WOOCOMMERCE_ONBOARDING_TAX_NOTICE",actions:[{url:Object(u.e)("admin.php?page=wc-admin"),label:Object(r.__)("Continue setup.",'woocommerce')}]}))})};i()(()=>{const e=document.querySelector(".woocommerce-save-button");window.htmlSettingsTaxLocalizeScript&&window.htmlSettingsTaxLocalizeScript.rates&&!window.htmlSettingsTaxLocalizeScript.rates.length&&e&&e.addEventListener("click",a)})},51:function(e,t){e.exports=window.wp.domReady},7:function(e,t){e.exports=window.wp.data}});