/**
 * External dependencies
 */
import { SVG } from 'wordpress-components';

const notes = (
	<SVG xmlns="http://www.w3.org/2000/svg" viewBox="0 0 16 16">
		<g fill="none" fillRule="evenodd">
			<path
				stroke="currentColor"
				strokeWidth="1.5"
				d="M2 .75h12c.69 0 1.25.56 1.25 1.25v12c0 .69-.56 1.25-1.25 1.25H2c-.69 0-1.25-.56-1.25-1.25V2C.75 1.31 1.31.75 2 .75z"
			/>
			<path
				fill="currentColor"
				d="M12 6H4V4.75h8zM12 9H4V7.75h8zM10 12H4v-1.25h6z"
			/>
		</g>
	</SVG>
);

export default notes;
