// Adjust padding and margins in the editor to improve selected block outlines.
.wp-block-woocommerce-cart-express-payment-block {
	margin: 14px 0 28px;

	.components-placeholder__label svg {
		font-size: 1em;
	}

	.wc-block-components-express-payment-continue-rule--checkout {
		margin-bottom: 0;
	}

	&.wp-block-woocommerce-cart-express-payment-block--has-express-payment-methods {
		padding: 14px 0;
		margin: -14px 0 14px 0 !important;
		position: relative;
	}
}

.wp-block-woocommerce-checkout-express-payment-block-placeholder {
	* {
		pointer-events: all; // Overrides parent disabled component in editor context
	}

	.wp-block-woocommerce-checkout-express-payment-block-placeholder__description {
		display: block;
		margin: 0 0 1em;
	}
}
