.wc-block-product-categories.wc-block-product-categories ul {
	margin-left: 20px;
}
.wc-block-product-categories {
	.components-placeholder__label svg {
		margin-right: 1ch;
		fill: currentColor;
	}
}
