(window.webpackWcBlocksJsonp=window.webpackWcBlocksJsonp||[]).push([[14],{487:function(t,c,e){"use strict";e.r(c);var n=e(5),o=e.n(n),u=e(0),s=e(1),a=(e(2),e(6)),r=e.n(a),l=e(45),b=e(86);e(562),c.default=Object(b.withProductDataContext)((function(t){var c=t.className,e=Object(l.useInnerBlockLayoutContext)().parentClassName,n=Object(l.useProductDataContext)().product.sku;return n?Object(u.createElement)("div",{className:r()(c,"wc-block-components-product-sku",o()({},"".concat(e,"__product-sku"),e))},Object(s.__)("SKU:",'woocommerce')," ",Object(u.createElement)("strong",null,n)):null}))},562:function(t,c){}}]);