/**
 * External dependencies
 */
import { SVG } from 'wordpress-components';

const grid = (
	<SVG xmlns="http://www.w3.org/2000/SVG" viewBox="0 0 24 24">
		<path fill="none" d="M0 0h24v24H0V0z" />
		<path d="M4 8h4V4H4v4zm6 12h4v-4h-4v4zm-6 0h4v-4H4v4zm0-6h4v-4H4v4zm6 0h4v-4h-4v4zm6-10v4h4V4h-4zm-6 4h4V4h-4v4zm6 6h4v-4h-4v4zm0 6h4v-4h-4v4z" />
	</SVG>
);

export default grid;
