!function(e){var n={};function t(o){if(n[o])return n[o].exports;var i=n[o]={i:o,l:!1,exports:{}};return e[o].call(i.exports,i,i.exports,t),i.l=!0,i.exports}t.m=e,t.c=n,t.d=function(e,n,o){t.o(e,n)||Object.defineProperty(e,n,{enumerable:!0,get:o})},t.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},t.t=function(e,n){if(1&n&&(e=t(e)),8&n)return e;if(4&n&&"object"===typeof e&&e&&e.__esModule)return e;var o=Object.create(null);if(t.r(o),Object.defineProperty(o,"default",{enumerable:!0,value:e}),2&n&&"string"!=typeof e)for(var i in e)t.d(o,i,function(n){return e[n]}.bind(null,i));return o},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,n){return Object.prototype.hasOwnProperty.call(e,n)},t.p="",t(t.s=387)}({25:function(e,n){e.exports=jQuery},387:function(e,n,t){e.exports=t(388)},388:function(e,n,t){"use strict";t.r(n);var o=t(25);!function(e){e(window).on("load",function(){!function n(t,o){if(Array.isArray(t)&&t[o]){var i=t[o];if("string"===typeof i.target&&"string"===typeof i.id&&"object"===typeof i.options&&"string"===typeof i.options.content){var r=e(i.target),c=e.extend(i.options,{close:function(){e.post(ajaxurl,{pointer:i.id,action:"dismiss-wp-pointer"}),i.dim?e("#wcs-pointer-page-dimmer").fadeOut(500,function(){return e(i.target).css("z-index","")}):e(i.target).css("z-index",""),n(t,o+1)},open:function(){i.dim&&(r.css("z-index",9999),e("body").append('<div id="wcs-pointer-page-dimmer" class="wcs-pointer-page-dimmer"></div>'),e("#wcs-pointer-page-dimmer").fadeIn(500).one("click",function(){r.pointer("close")}))}});if(i.delayed_opening){var u=e(i.delayed_opening.delegation_container||document);u.one("click",i.delayed_opening.show_button,function(){i.delayed_opening.animating_container?setTimeout(function(){e(i.delayed_opening.animating_container).promise().then(a)},0):a()}),i.delayed_opening.hide_button&&u.one("click",i.delayed_opening.hide_button,function(){r.pointer("close")})}else a()}else n(t,o+1)}function a(){r.pointer(c).pointer("open")}}(wcServicesAdminPointers,0)})}(t.n(o).a)}});