<?php
/**
 * This file was automatically generated by automattic/jetpack-autoloader.
 *
 * @package automattic/jetpack-autoloader
 */

namespace Automattic\Jetpack\Autoloader\jpd8a0da99a02edbc19880f5a1a3c7fa56;

 // phpcs:ignore

require_once __DIR__ . '/jetpack-autoloader/class-autoloader.php';
Autoloader::init();
