.wc-block-cart__submit {
	position: relative;
}

.wc-block-cart__submit-container {
	padding-bottom: $gap;
}

.wc-block-cart__submit-button {
	width: 100%;
	margin: 0 0 $gap;

	&:last-child {
		margin-bottom: 0;
	}
}

.is-mobile,
.is-small,
.is-medium {
	.wc-block-cart__submit-container:not(.wc-block-cart__submit-container--sticky) {
		padding-left: 0;
		padding-right: 0;
		padding-top: 0;
	}
}

@include breakpoint(">782px") {
	.wc-block-cart__submit-container--sticky {
		display: none;
	}
}

@include breakpoint("<782px") {
	.wc-block-cart__submit-container--sticky {
		background: $white;
		bottom: 0;
		left: 0;
		padding: $gap;
		position: fixed;
		width: 100%;
		z-index: 9999;

		&::before {
			box-shadow: 0 -10px 20px 10px currentColor;
			color: transparentize($gray-400, 0.5);
			content: "";
			height: 100%;
			left: 0;
			position: absolute;
			right: 0;
			top: 0;
		}
	}
}

