ul {
    list-style: none;
}
div[class^="col-md"] .date-inp-wrap, div[class^="col-md"] .date-inp-wrap .hasDatepicker{
    width: 100%;
}
.wcs_hidden_label {
    display: none !important;
}
#product_images_container {
    background-color: #fff;
    margin-top: 20px;
}
#product_images_container ul {
    margin: 0 -10px 0 0;
    padding: 0;
}
#product_images_container ul::after {
    clear: both;
}
#product_images_container ul::after, #product_images_container ul::before {
    content: ' ';
    display: table;
}
#product_images_container ul li.image {
    width: 86px;
    float: left;
    cursor: move; 
    margin: 3px 3.33% 8px 0; 
    border-radius: 2px;
    position: relative;
    box-sizing: border-box; 
    text-align: center;
} 
#product_images_container ul li.image img {
    width: 100%;
    height: auto;
    display: block;
    border: 1px solid #d5d5d5;
}
#product_images_container ul li:hover ul.actions {
    display: block;
}
#product_images_container ul ul.actions {
    position: absolute;
    top: -8px;
    right: -8px;
    padding: 2px;
    display: none;
    z-index: 1;
}
#product_images_container ul ul.actions li {
    float: right;
    margin: 0 0 0 2px;
}
#product_images_container ul ul.actions li a.delete {
    display: block;
    text-indent: -9999px;
    position: relative;
    height: 1em;
    width: 1em;
    font-size: 1.4em;
}
#product_images_container ul ul.actions li a.tips {
    cursor: pointer;
}
#product_images_container ul ul.actions li a.delete::before {
    font-family: Dashicons;
    speak: none;
    font-weight: 400;
    text-transform: none;
    -webkit-font-smoothing: antialiased;
    text-indent: 0px;
    position: absolute;
    top: 0px;
    left: 0px;
    text-align: center;
    content: "";
    color: rgb(153, 153, 153);
    height: 1em;
    width: 1em;
    line-height: 1em;
    font-variant: normal;
    margin: 0px;
    background: rgb(255, 255, 255);
    border-radius: 50%;
}

.notice-wrapper {
    position: relative;
}
.notice-wrapper .notice-dismiss {
    position: absolute;
    top: 7px;
    right: 1px;
    border: none;
    margin: 0;
    padding: 9px;
    background: 0 0;
    color: #72777c;
    cursor: pointer;
}
.notice-wrapper .notice-dismiss:before {
    background: 0 0;
    color: #72777c;
    content: "\f153";
    display: block;
    font: 400 17px/20px dashicons;
    speak: none;
    height: 20px;
    text-align: center;
    width: 20px;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}
.notice-dismiss:hover:before {
    color: #c00;
}
/*SORTABLE ICON CSS FOR FRONTEND ADD PRODUCT PAGE*/
.sortable-icon {
  position: relative;
  padding-left: 1.25em;
  margin-right: 0.5em;
}
.sortable-icon:before {
  content: "";
  position: absolute;
  left: 0;
  top: 0.35em;
  width: 1em;
  height: 0.15em;
  background: currentColor;
  box-shadow: 
    0 0.25em 0 0 currentColor,
    0 0.5em 0 0 currentColor;
}
.downloadable_files tr td:first-child {
    cursor: move;
}
.variations-pagenav {
    float: right;
    line-height: 24px;
}
table.dataTable thead .sorting:after, table.dataTable thead .sorting_asc:after, table.dataTable thead .sorting_desc:after, table.dataTable thead .sorting_asc_disabled:after, table.dataTable thead .sorting_desc_disabled:after {
    bottom: auto;
    top: 50%;
    -webkit-transform: translateY(-50%);
        -ms-transform: translateY(-50%);
            transform: translateY(-50%);
}
/*PRODUCT MANAGER EDIT CSS*/
.dc-wp-fields-uploader .button.ele_hide {
    display: none !important;
}
.dc-wp-fields-uploader .button.ele_show {
    display: inline-block !important;
}


/*FRONT END CSS*/
.wrap.woocommerce.export {
    margin-left: 10px;
    margin-right: 10px;
}

/*IMPORT CSS*/
.woocommerce-importer-advanced.hidden {
    display:none;
}
.woocommerce-exporter-wrapper .wc-progress-steps li::before, .woocommerce-importer-wrapper .wc-progress-steps li::before, .woocommerce-progress-form-wrapper .wc-progress-steps li::before {
    margin-bottom:-7px !important;
    height: 11px !important;
    width:11px !important;
}
.wrap.woocommerce{
    margin-left: 10px;
    margin-right: 10px;
}

.audio_rm,
.video_rm{
    background: #ddd;
    padding: 2px 5px 0 5px;
    border-radius: 100%;
    position: absolute;
    right: 14px;
    top: 3px;
    font-weight: bold;
}

.multi_input_block p {
    margin: 10px 0 5px;
}

.hide_visiblity, .hide_variation, .hide_variation + .select2-container {
    display:none !important;
}
.woocommerce-Tabs-panel--video_audio_tab .frame-wrap:nth-child(2n+2){
    margin-right: 0;
}
@media screen and (max-width: 767px){
    .frame-wrap{
        width: 100%;
        text-align: center;
        margin: 0 0 25px;
    }
    .woocommerce-Tabs-panel--video_audio_tab .frame-wrap:last-child{
        margin-bottom: 0;
    }

}
.multiSelectBox-right{ 
    margin-top: 6px;	
}
@media screen and (max-width: 640px){
    .multiSelectBox-right{
        margin-left: 0;
    }
}
.paid_checkbox[type="checkbox"]{
    width: auto !important;
} 
.video_uploader_label, .blockClass {
    display: block !important;
}
.single-product .ywcfav_thumbnails_video_container, .single-product .ywcfav_thumbnails_audio_container {
    display: none;
}
body .remove_display,body .hide {
    display: none !important;
}


/*CALENDER CSS*/
.clear{clear:both}.nobr{white-space:nowrap}.wc_bookings_calendar_form{margin:1em 0 0!important}.wc_bookings_calendar_form .calendar_days{position:relative;height:1440px;overflow:hidden;overflow-x:scroll;border:1px solid #ddd;}.wc_bookings_calendar_form .calendar_days .hours{position:absolute;top:0;left:0;margin:-.5em 0 0 0}.wc_bookings_calendar_form .calendar_days .hours li{height:60px;vertical-align:top;padding:0;margin:0}.wc_bookings_calendar_form .calendar_days .hours li label{float:left;background:#f9f9f9;padding:0 8px}.wc_bookings_calendar_form .calendar_days .bookings{margin:0 0 0 100px;padding:0;height:100%;white-space:nowrap;position:relative}.wc_bookings_calendar_form .calendar_days .bookings li{position:absolute;width:98px;background:#000;border-radius:2px;margin:0;overflow:hidden;color:#fff}.wc_bookings_calendar_form .calendar_days .bookings li a{display:block;padding:2px 4px;height:100%;width:100%;position:relative;color:#fff;text-decoration:none}.wc_bookings_calendar_form .tablenav{border:1px solid #dedede;background:#fff;height:auto;box-shadow:0 1px 1px -1px rgba(0,0,0,.1);-webkit-box-shadow:0 1px 1px -1px rgba(0,0,0,.1)}.wc_bookings_calendar_form .tablenav:after,.wc_bookings_calendar_form .tablenav:after{clear:both}.wc_bookings_calendar_form .tablenav .filters{float:left;line-height:30px;padding:5px;border-right:1px solid #ddd}.wc_bookings_calendar_form .tablenav .filters .chosen-container{width:12em}.wc_bookings_calendar_form .tablenav .views{float:right;padding:0;border-left:1px solid #ddd}.wc_bookings_calendar_form .tablenav .views a{padding:11px 20px;display:block;text-decoration:none}.wc_bookings_calendar_form .tablenav .date_selector{margin-left:5px;display:block;float:left}.wc_bookings_calendar_form .tablenav .date_selector div{padding:5px 0;float:left}.wc_bookings_calendar_form .tablenav .date_selector div:first-of-type select{margin-right:5px}.wc_bookings_calendar_form .tablenav .date_selector select{vertical-align:middle;width:6em}.wc_bookings_calendar_form .tablenav .date_selector a{display:block;text-indent:-9999px;position:relative;height:1em;width:1em;font-size:1.3em;padding:.6em}.wc_bookings_calendar_form .tablenav .date_selector a.prev{float:left}.wc_bookings_calendar_form .tablenav .date_selector a.next{float:right}.wc_bookings_calendar_form .wc_bookings_calendar{width:100%;margin:1em 0 0!important}.wc_bookings_calendar_form .wc_bookings_calendar th{text-align:center}.wc_bookings_calendar_form .wc_bookings_calendar td{color:#bbb;text-align:right;padding:8px 8px 0 0;font-size:20px;border-right:1px solid #ddd;border-bottom:1px solid #ddd;height:120px}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings{overflow:auto;height:100px;width:100%;text-align:left;font-size:14px;}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings ul{padding:0;margin:0}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings ul li{padding:0;margin:0 0 4px 0;list-style:none}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings ul li a{border-left:4px solid #eee;padding:3px 3px 3px 8px;display:block}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings ul li a ul{padding:0;margin:0}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings ul li a ul li{list-style:none;font-size:.75em;padding:0;margin:0;border:0}.wc_bookings_calendar_form .wc_bookings_calendar td .bookings ul li a:hover{border-left-color:#ccc;background:#fafafa}.wc_bookings_calendar_form .wc_bookings_calendar tr{background: #fff}.wc_bookings_calendar_form .wc_bookings_calendar .calendar-diff-month{background:#efefef}.wc_bookings_calendar_form .wc_bookings_calendar tbody tr td:last-child{border-right:0}.wc_bookings_calendar_form .wc_bookings_calendar tbody tr:last-child td{border-bottom:0}#tiptip_holder{display:none;z-index:8675309;position:absolute;top:0;left:0}#tiptip_holder.tip_top{padding-bottom:5px}#tiptip_holder.tip_top #tiptip_arrow_inner{margin-top:-7px;margin-left:-6px;border-top-color:#333}#tiptip_holder.tip_bottom{padding-top:5px}#tiptip_holder.tip_bottom #tiptip_arrow_inner{margin-top:-5px;margin-left:-6px;border-bottom-color:#333}#tiptip_holder.tip_right{padding-left:5px}#tiptip_holder.tip_right #tiptip_arrow_inner{margin-top:-6px;margin-left:-5px;border-right-color:#333}#tiptip_holder.tip_left{padding-right:5px}#tiptip_holder.tip_left #tiptip_arrow_inner{margin-top:-6px;margin-left:-7px;border-left-color:#333}#tiptip_content,.chart-tooltip,.wc_error_tip{color:#fff;font-size:.8em;max-width:150px;background:#333;text-align:center;border-radius:3px;padding:.618em 1em;box-shadow:0 1px 3px rgba(0,0,0,.2)}#tiptip_content code,.chart-tooltip code,.wc_error_tip code{padding:1px;background:#888}#tiptip_arrow,#tiptip_arrow_inner{position:absolute;border-color:transparent;border-style:solid;border-width:6px;height:0;width:0}.wc_error_tip{max-width:20em;line-height:1.8em;position:absolute;white-space:normal;background:#d82223;margin:1.5em 1px 0 -1em;z-index:9999999}.wc_error_tip::after{content:'';display:block;border:8px solid #d82223;border-right-color:transparent;border-left-color:transparent;border-top-color:transparent;position:absolute;top:-3px;left:50%;margin:-1em 0 0 -3px}

/*Backup end*/


.inline-select {
    width: auto;
    display: inline-block;
    vertical-align: middle; }

.row-padding {
    padding: 20px; }

.select-group {
    display: inline-block;
    vertical-align: middle; }
.select-group label {
    font-weight: normal;
    margin: 0 10px 0 0; }

.add-product-wrapper .product-primary-info .panel.panel-default {
    margin: 0; }
.add-product-wrapper .product-primary-info > div{
    float: left;
}
.right-primary-info{
    width: calc(100% - 350px);
    margin-right: 40px;
}
.left-primary-info{
    width: 310px;
}
@media screen and (max-width: 1080px){
    .right-primary-info{
        width: 100%;
        margin: 0 0 20px;
    }
    .left-primary-info{
        width: 100%;
    }
    .featured-img .upload_image_button.remove::before, .woocommerce_variable_attributes .upload_image_button.remove::before, #product_images_container ul ul.actions{
        display: block !important;
    }
}
.product-gallery-wrapper figure {
    background: #fff;
    border: 1px solid #d3dbe2;
    padding: 10px;
    height: 200px;
    position: relative; }
.product-gallery-wrapper figure .gallery-img-placeholder {
    position: absolute;
    margin: auto;
    top: 50%;
    left: 0;
    right: 0;
    -webkit-transform: translateY(-50%);
        -ms-transform: translateY(-50%);
            transform: translateY(-50%); }
.product-gallery-wrapper figure .gallery-img-placeholder i {
    font-size: 55px;
    color: #cfcfcf; }
.product-gallery-wrapper figure .gallery-img-placeholder p {
    margin: 10px 0 0;
    color: #0c0c0c; }

.product-img-thumbnail {
    list-style: none; }
.product-img-thumbnail li {
    display: inline-block;
    width: 80px;
    height: 80px;
    vertical-align: top;
    margin: 0 2% 0 0;
    background: #fff;
    border: 1px solid #d3dbe2; }
.product-img-thumbnail li:last-of-type {
    margin: 0; }
.product-img-thumbnail .add-gal-img {
    position: relative; }
.product-img-thumbnail .add-gal-img i {
    position: absolute;
    margin: auto;
    top: 50%;
    left: 0;
    right: 0;
    -webkit-transform: translateY(-50%);
        -ms-transform: translateY(-50%);
            transform: translateY(-50%);
    font-size: 45px;
    color: #cfcfcf; }

.gal-img-wrap {
    position: relative; }
.gal-img-wrap .remove-img {
    position: absolute;
    top: 5px;
    right: 5px;
    -webkit-transform: rotate(45deg);
        -ms-transform: rotate(45deg);
            transform: rotate(45deg); }
.gal-img-wrap .remove-img .wcmp-font {
    background: #fff;
    border-radius: 50%;
    color: #ff0000;
    font-size: 15px;
    cursor: pointer; }

.mcs-horizontal-example {
    overflow-x: auto; }
.mcs-horizontal-example .mCSB_dragger_bar {
    top: 7px; }
.mcs-horizontal-example #mCSB_2_scrollbar_horizontal {
    bottom: -7px; }

body .mcs-horizontal-example .mCSB_draggerRail {
    background: #dae1e7 !important;
    height: 4px !important; }

body .mcs-horizontal-example .mCSB_dragger_bar {
    background: #b1bfca !important; }

.mcs-horizontal-example ul {
    white-space: nowrap;
    list-style: none;
    padding: 0;
    margin: 15px 0 0; }

.mcs-horizontal-example ul > li {
    display: inline-block; }

.hidden-file-uploader {
    position: absolute;
    width: 100%;
    height: 100%;
    top: 0;
    left: 0; }
.hidden-file-uploader input[type="file"] {
    position: absolute;
    width: 100%;
    height: 100%;
    top: 0;
    left: 0;
    opacity: 0;
    cursor: pointer; }

.add-product-info-holder {
    background: #fff;
    border: 1px solid #d3dbe2;
    overflow: hidden;
    margin: 25px 0 0; } 
.add-product-info-holder .form-group input[type=checkbox] + label {
    margin: 0 0 0 5px; }

.add-product-info-header .form-group {
    display: inline-block;
    margin: 0 0 0 15px; }
.add-product-info-holder .form-group .checkbox-holder {
    margin-top: 3px; }

.product-info-tab-wrapper {
    clear: both;
    background: #fff; }

.product-tab-nav-holder {
    background: transparent;
    height: 63px;
    overflow: hidden; 
}
.product-tab-nav-holder ul {
    display: -webkit-box;
    display: flex;
    -webkit-box-orient: horizontal;
    -webkit-box-direction: normal;
            flex-flow: row wrap;
    -webkit-box-pack: start;
            justify-content: flex-start;
    -webkit-box-align: end;
            align-items: flex-end;
    position: relative;
    z-index: 1;
    border: 0; }
.product-tab-nav-holder ul li {
    border-right: 1px solid #fff;
    margin: 0; }
.product-tab-nav-holder ul li:last-of-type {
    border: 0;
}
.product-tab-nav-holder ul li a {
    color: #383838;
    border-radius: 0;
    border-width: 2px 1px 2px 1px !important;
    border-color: transparent;
    background: #bfc3c4;
    margin: 0; 
    -webkit-transition: background-color 0.3s; 
    transition: background-color 0.3s;
    padding: 13px 22px; }
.product-tab-nav-holder ul li a:hover {
    background: #b1becb;
    border-color: #9eacbd;
    border-top-color: transparent;
}
.product-tab-nav-holder ul li.active a{
    position: relative;
    top: 1px;
    border-bottom-color: transparent !important;
}
.product-tab-nav-holder ul li.active a, .product-tab-nav-holder ul li.active a:hover, .product-tab-nav-holder ul li.active a:focus {
    padding: 20px 22px;
    background-color: #fff;
    color: #222; 
    border-color: #bfc3c4;
    box-shadow: none; }

.add-veriaton-wrapper .select-group:last-of-type {
    margin: 0 10px 0 25px; }

.wcmp-metabox-title {
    overflow: hidden;
    padding: 12px 20px!important;
}
.product-variations-wrapper .wc-metabox .variation-title .ico-up-arrow-icon::before {
    -webkit-transition: all 0.3s;
    transition: all 0.3s; }
.wcmp-metabox-title[aria-expanded='true'] {
    background: #ddd; }
.product-variations-wrapper .wc-metabox .variation-title[aria-expanded='true'] .ico-up-arrow-icon::before {
    content: '\e90f'; }
.wcmp-metabox-title div[class*="group"] {
    float: left;
    word-wrap: break-word;
    width: calc(100% - 150px); }
.product-variations-wrapper .wc-metabox .variation-select-group select {
    margin: 3px; }
.wcmp-metabox-action {
    float: right;
    margin: 0;
    width: 150px;
    text-align: right;
    visibility: hidden; }
.product-variations-wrapper .wc-metabox:hover .variation-action, .wcmp-metabox-wrapper:hover .wcmp-metabox-action {
    visibility: visible;
}
.product-variations-wrapper .wc-metabox .variation-action i {
    margin-right: 15px; }
.product-variations-wrapper .wc-metabox .variation-content {
    overflow: hidden;
    padding: 20px;  }
.product-variations-wrapper .wc-metabox .variation-content input[type=checkbox] {
    background: #fff; }
.product-variations-wrapper .wc-metabox .variation-content .checkbox-holder {
    background: #fff; }

.featured-img, .variation-img {
    background: #fff;
    border: 1px solid #d3dbe2;
    text-align: center;
    font-size: 40px;
    position: relative; 
}

.variation-img {
    display: inline-block;
    vertical-align: top;
}
.variation-img .upload_image_button{
    width: 80px;
    height: 80px;
    min-height: 80px !important;
}

.featured-img .upload_image_button, .woocommerce_variable_attributes .upload_image_button {
    display: block;
    position: relative; 
}
.featured-img .upload_image_button{
    height: 215px;
}
.featured-img .upload_image_button.remove{
    height: auto; 
}
.featured-img .upload_image_button img{
    padding: 5px;
    margin: 0 auto;
    width: 100%;
}
.featured-img .upload_image_button.remove img, .woocommerce_variable_attributes .upload_image_button.remove img {
    display: block;
}

.featured-img .upload_image_button:not(.remove) img, .woocommerce_variable_attributes .upload_image_button:not(.remove) img {
    display: none;
}

.featured-img .upload_image_button.remove::before, .woocommerce_variable_attributes .upload_image_button.remove::before {
    content: '\f335';
    font-family: 'dashicons';
    display: none;
    line-height: 52px;
    font-size: 50px;
    position: absolute;
    left: 50%;
    top: 50%;
    background: rgb(153, 153, 153);
    -webkit-transform: translateY(-50%)translateX(-50%);
        -ms-transform: translateY(-50%)translateX(-50%);
            transform: translateY(-50%)translateX(-50%);
    border-radius: 50%;
    height: 50px;
    width: 50px;
    text-align: center;
    color: #fff;
}
.featured-img .upload_image_button.remove .upload-placeholder{
    display: none;  
}
.upload-placeholder p{
    font-size: 15px;
}
.remove .upload-placeholder{
    display: none;
}
.featured-img .upload_image_button.remove:hover::before, .woocommerce_variable_attributes .upload_image_button.remove:hover::before {
    display: block;
}


.attribute-chk-option .form-group {
    width: 50%; 
    vertical-align: top;
    float: left;
    margin: 5px 0 0; }

.collapse {
    overflow: hidden; }

label.col-md-12.control-label {
    margin-bottom: 6px; }


/* downloadable product */
.border-wrapper{
    border: 1px solid #c5c5c5;
    padding: 15px 20px;
}
.downloadable-files tr td:not(.sort){
    padding: 0 6px 6px;
}
.downloadable-files .delete.btn{
    background: #c6c6c6;
    padding: 5px;
    border-radius: 50%;
    width: 30px;
    font-size: 16px;
    line-height: 20px;
    height: 30px;
    color: #fff;
}
.downloadable-files .delete.btn:hover {
    background: #e55959;
}
.downloadable-files tfoot button{
    margin-top: 10px;
}
/*attribute*/
.product_attributes.wc-metaboxes .woocommerce_attribute input[type=checkbox]{
    vertical-align: middle;
}
.product_attributes.wc-metaboxes .woocommerce_attribute label{
    font-weight: normal;
}
.product_attributes.wc-metaboxes .woocommerce_attribute .woocommerce_attribute_data{
    padding: 0;
}
td.attribute_name {
    width: 230px;
    padding: 0 6px 10px 0;
} 
.selectTowrap .select2.select2-container{
    margin-bottom: 5px;
}

/* N E W */
/* global */
.margin-0{
    margin: 0;
}
label{
    font-weight: normal;
}
.wcmp-2x{
    font-size: 2rem;
}
.wcmp-3x{
    font-size: 3rem;
}
#woocommerce-product-data hr{
    margin-bottom: 0;
    margin-top: 15px;
}
body div.dataTables_wrapper div.dataTables_length select{
    width: auto;
}
.form-horizontal .control-label.col-md-12{
    padding-right: 40px;
}
.form-horizontal .control-label.col-md-12 .img_tip{
    margin: 10px 17px 0 0;
}
/* elements start */
.custom-panel{
    background-color: #fff;
    padding: 15px;
    border: 1px solid #d3dbe2;
    overflow: hidden;
}
.custom-panel h3 {
    margin: -7px 0 10px 0;
    color: #565656;
    padding: 0 0 10px;
    position: relative;
}
.custom-panel h3:after {
    content: '';
    height: 1px;
    background: #d3dbe2;
    position: absolute;
    left: 0;
    margin: 0 -15px;
    right: 0;
    bottom: 0;
}
.button-group{
    margin-top: 15px;
}
/* elements end */

/* General Tab Start */
.add_product_images{
    margin: 10px 0;
}
.form-text {
    display: block;
    margin: 6px 0 0;
}
.form-text i{
    vertical-align: middle;
}
.add-product-info-header .pull-right label{
    margin: 6px 0 0 15px;
    font-weight: normal;
}
.add-product-info-header .pull-right .checkbox-holder{
    margin-right: 3px;
}
.add-product-info-header .select-group label{
    color: #333;
}
.upload_file_button .wcmp-font, .delete .wcmp-font{
    font-size: 22px;
}
.downloadable_files{
    margin: 5px 0;
}
.downloadable_files table{
    margin: 0;
}
.downloadable_files table.table>tbody>tr>td{
    vertical-align: middle;
}
.downloadable_files table.table thead th{
    padding: 8px 20px;
}
/* General Tab End */

/* Attributeds start */
.product_attributes table.table{
    margin: 15px 0;
}
.product_attributes table.table td{
    padding: 4px 15px;
    border: 0;
}
.wcmp-metabox-wrapper{
    border: 1px solid #dddddd;
    margin: 15px 0 0;
}
.woocommerce_attribute table{
    margin: 0;
}
/* Attributeds End */
/* veriations */
.woocommerce_variations{
    margin-top: 15px;
} 
.woocommerce_variations.product-variations-wrapper .wc-metabox .variation-action {
    margin-top: 11px;
}
/* rental */
@media screen and (min-width:991px){
    .table.wcmp-metabox-table{
        width: 75%;
    }
}
.table.wcmp-metabox-table td, .table.wcmp-metabox-table th{
    border: 0;
    padding: 12px 20px;
}
#variable_product_options_inner hr{
    margin-bottom: 15px;    
}
/* booking */
.booking_range_pricing table .select + .form-control{
    margin-top: 10px;
}
.wc_booking_restricted_days_field{
    padding: 15px 20px;
    border: solid 1px #ccc;
    text-align: center;
}
.wc_booking_restricted_days_field li {
    display: inline-block;
    vertical-align: middle;
    margin-right: 30px;
}
.wc_booking_restricted_days_field li label, .add-product-info-holder .form-group .wc_booking_restricted_days_field li input[type=checkbox] {
    margin: 0;
}
.create-order-label-wrap label{
    display: block;
    margin-bottom: 10px;    
}
.create-order-label-wrap label input[name="booking_order_id"]{
    width: 150px;
    display: inline-block;
    vertical-align: middle;
    margin-left: 6px;
}
.wc-bookings-booking-form .block-picker li:nth-child(odd) a {
    background: #efefef;
}
.wc-bookings-booking-form .block-picker li:nth-child(even) a {
    background: #f5f5f5;
}
.wc-bookings-date-picker .ui-datepicker td.bookable a {
    background-color: #3f85b9!important;
}
.wc-bookings-date-picker .ui-datepicker td.ui-datepicker-current-day a {
    background: #316fa8!important;
}
.wc-bookings-booking-form .block-picker li:nth-child(odd) a:focus, .wc-bookings-booking-form .block-picker li:nth-child(odd) a:hover {
    background: #316fa8;
}
.wc-bookings-booking-form .block-picker li:nth-child(even) a:focus, .wc-bookings-booking-form .block-picker li:nth-child(even) a:hover {
    background: #316fa8; 
}
.wc-bookings-date-picker .ui-datepicker td.partial_booked a:before{
    border-top: 2.5em solid #7eb5de; 
}
.fc-row .fc-content-skeleton td, .fc-row .fc-helper-skeleton td{
    border-color: #d3dbe2;
}
.fc-content-skeleton td {
    padding: 10px 0 0 10px;
}
.table.vertical-th tr th{
    padding: 20px;
}
.fc-basic-view .fc-body .fc-row {
    min-height: 1em;
}
.woocommerce_booking_resource .wcmp-metabox-title div[class*="group"]{
    width: calc(100% - 190px);
}
.woocommerce_booking_resource .wcmp-metabox-action{
    width: 190px;
}
.woocommerce_booking_resource .wcmp-metabox-action .wcmp-font.ico-up-arrow-icon{
    margin-right: 5px;
}
/* Auction */
.auction_status_wrapper{
    margin-bottom: 15px;
    font-weight: bold;
}
.auction_status_wrapper p span{
    font-weight: normal;
}
/* bundle */
.item_visibility .col-md-6 > div {
    margin-bottom: 20px;
}
.item_visibility .col-md-6 input[type="checkbox"]{
    margin-top: 0;
}

/* add on */
.inline-description{
    display: inline-block;
    vertical-align: middle;
    margin-left: 7px;
}
.addon_required label{
    display: block;
}
/* Rental product settings */
.sub-tab-wrapper{
    border: 1px solid #e2e1e1;
    overflow: hidden;
    display: -webkit-box;
    display: flex;
}
.sub-tab-nav{
    width: 200px;
    max-width: 100%;
    display: inline-block;
    vertical-align: top;
    box-shadow: 0px 2px 20px rgba(0,0,0,0.20);
}
.sub-tab-nav li{ 
    border-bottom: 1px solid #e2e1e1;
}
.sub-tab-nav li a{
    display: block; 
    color: #212121;
    font-size: 14px;
    padding: 12px 16px;
}
.sub-tab-nav li.active a, .sub-tab-nav li a:hover, .sub-tab-nav li a:focus{
    background: #f4f4f4;
}
.sub-tab-nav li a i {
    font-size: 22px;
    vertical-align: middle;
    margin-right: 6px;
}
.sub-tab-content{
    width: calc(100% - 200px);
    float: right;
    /* border-left: 1px solid #e2e1e1; */
    min-height: 200px;
}

/* select2 */
.select2-container--default .select2-selection--single{
    height: 100%;
    border: 1px solid #c5c5c5;
}
.select2-container{
    min-height: 34px;
}
.select2-container--default .select2-selection--single .select2-selection__rendered{
    padding-top: 3px;
}
.select2-container--default .select2-selection--single .select2-selection__arrow{
    top: 4px;
    right: 9px;
}
.select2-container--default.select2-container--focus .select2-selection--multiple {
    border: solid #c5c5c5 1px; 
}
.select2-container .select2-search--inline {
    margin-top: 8px;
}
.select2-container .select2-selection.select2-selection--multiple{
    min-height: 34px;
}
.select2-container .select2-selection.select2-selection--multiple .select2-selection__rendered .select2-selection__choice { 
    margin: 6px 8px 0 0;
}
.select2-results__options [data-selected="true"]{
    background: #ddd;
} 
/* gupis */
.product-info-tab-wrapper{
    position: relative;
}
.product-info-tab-wrapper > .tab-content{
    position: relative;
    top: -1px;
    border: 1px solid #bfc3c4;
}
.product-info-tab-wrapper{
    padding: 40px 20px 20px;
}

.nav-tabs{
    -webkit-transition: all 0.3s;
    transition: all 0.3s;
}
.tab-nav-direction-wrapper{
    position: absolute;
    top: 5px;
    right: 20px;
    bottom: 0;
    width: 51px;
    height: 25px;
}
.tab-nav-direction-wrapper .nav-btn {
    position: absolute;
    top: 0;
    bottom: 0;
    cursor: pointer;
    z-index: 1;
    background: #979b9c;
    border: 0;
    color: #fff;
    width: 25px;
    height: 25px;
    font-size: 11px;
    text-align: center;
    cursor: pointer;
    border-radius: 3px;
    z-index: 1;
}
.tab-nav-direction-wrapper .nav-btn:hover {
    background: #3f85b9;
}
.tab-nav-direction-wrapper .nav-btn.next {
    right: 0;
}
.title-with-type .wcmp-metabox-action{
    margin-top: 11px;
}
/* rental toggle */
.days-range-wrapper .redq_days_range .days-range-title[aria-expanded='true'] .ico-up-arrow-icon::before,
.price-discount-wrapper .redq_price_discount .price-discount-title[aria-expanded='true'] .ico-up-arrow-icon::before,
.rental-inventory-wrapper .redq_inventory .inventory-title[aria-expanded='true'] .ico-up-arrow-icon::before {
    content: '\e90f';
}
body .select2-container--default .select2-search--inline .select2-search__field[placeholder="Search for a product…"]{
    min-width: 150px;
}
/* responsive */
@media screen and (max-width: 767px){
    .control-label + div[class*="col-md"] .row div[class*="col-md"]:not(:last-of-type) {
        margin-bottom: 10px;
    }
}

@media screen and (max-width: 640px){
    .add-product-info-header{
        text-align: center;
    }
    .add-product-info-header .select-group + .pull-right{
        float: none !important;
        margin: 14px auto 0;
    }
    .form-horizontal .form-group input[type="checkbox"] {
        margin-top: 0;
    }
    .product_attributes table.table td {
        padding: 4px 8px; 
    }
}

@media screen and (max-width: 480px){
    .product-info-tab-wrapper {
        padding: 40px 10px 20px;
    }
    .row-padding {
        padding: 10px;
    }
}