.woocommerce-dismissable-list{margin:0 20px 10px;animation:isLoaded;animation-duration:.25s}@media(min-width:782px){.woocommerce-dismissable-list{margin-left:0;margin-right:0}}.woocommerce-dismissable-list .woocommerce-dismissable-list__controls{text-align:center}.woocommerce-services-item__logo{width:36px;height:auto}:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}.woocommerce-recommended-shipping-extensions__more_options_cta .gridicon{margin-left:4px}.woocommerce-recommended-shipping-extensions .woocommerce-list__item>.woocommerce-list__item-inner{align-items:flex-start}.woocommerce-recommended-shipping-extensions .woocommerce-list__item:hover{background-color:#fff}.woocommerce-recommended-shipping-extensions .woocommerce-list__item:hover .woocommerce-list__item-title{color:#1e1e1e}.woocommerce-recommended-shipping-extensions .woocommerce-list__item-title{font-size:14px;color:#1e1e1e;font-weight:600}.woocommerce-recommended-shipping-extensions .woocommerce-pill{margin-left:4px;margin-top:4px;margin-bottom:4px;padding:2px 8px}@media(min-width:480px){.woocommerce-recommended-shipping-extensions .woocommerce-pill{margin-top:0;margin-bottom:0}}.woocommerce-recommended-shipping-extensions .woocommerce-list__item-after .components-button{margin-left:12px}.woocommerce-recommended-shipping-extensions .woocommerce-list__item-text,.woocommerce-recommended-shipping-extensions .woocommerce-recommended-shipping__header-heading{max-width:749px}