.wc-block-components-textarea {
	@include font-size(regular);
	background-color: #fff;
	border: 1px solid $input-border-gray;
	border-radius: 4px;
	color: $input-text-active;
	font-family: inherit;
	line-height: 1.375; // =22px when font-size is 16px.
	margin: 0;
	padding: em($gap-small) $gap;
	width: 100%;

	.has-dark-controls & {
		background-color: $input-background-dark;
		border-color: $input-border-dark;
		color: $input-text-dark;

		&::placeholder {
			color: $input-placeholder-dark;
		}
	}
}

.theme-twentytwentyone {
	.has-dark-controls .wc-block-components-textarea {
		background-color: $input-background-dark;
		color: $input-text-dark;
	}
}
