/**
 * @typedef {Object} ShippingAddress
 *
 * @property {string} first_name First name of recipient
 * @property {string} last_name  Last name of recipient
 * @property {string} company    Company name for the shipping address
 * @property {string} address_1  First line of the shipping address
 * @property {string} address_2  Second line of the shipping address
 * @property {string} city       The city of the shipping address
 * @property {string} state      The state of the shipping address (ISO or name)
 * @property {string} postcode   The postal or zip code for the shipping address
 * @property {string} country    The country for the shipping address (ISO)
 */

export {};
