.woocommerce-dashboard__chart-block-wrapper{cursor:pointer}.woocommerce-dashboard__chart-block-wrapper:hover .woocommerce-card__header,.woocommerce-dashboard__chart-block-wrapper:hover .woocommerce-chart{background:#f8f9f9}.woocommerce-dashboard__chart-block-wrapper:hover .woocommerce-legend__item button{background:transparent}.woocommerce-dashboard__chart-block-wrapper .woocommerce-chart{margin-top:0;margin-bottom:0;border:0}.woocommerce-dashboard__chart-block-wrapper .woocommerce-chart__footer{position:relative}.woocommerce-dashboard__chart-block-wrapper .woocommerce-chart__footer:after{content:"";position:absolute;width:100%;height:100%;right:0;top:0;cursor:pointer;z-index:1}.woocommerce-dashboard__chart-block .woocommerce-card__body{padding:0;position:relative}.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart{border:none;margin:0}.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart .woocommerce-legend__item>button{cursor:default}.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart .woocommerce-legend__item>button:hover{background:#f0f0f0}.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart .woocommerce-legend__item>button .woocommerce-legend__item-container{cursor:default}.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart .woocommerce-legend__item>button .woocommerce-legend__item-container .woocommerce-legend__item-checkmark.woocommerce-legend__item-checkmark-checked:after{display:none}.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart:hover,.woocommerce-dashboard__chart-block .woocommerce-card__body .woocommerce-chart:hover .woocommerce-legend__item>button,.woocommerce-dashboard__chart-block:hover{background:#f0f0f0}.woocommerce-dashboard__chart-block .screen-reader-text:focus{clip:auto;-webkit-clip-path:none;clip-path:none;z-index:1;right:6px;top:7px;height:auto;width:auto;display:block;font-size:14px;font-size:.875rem;font-weight:600;padding:15px 23px 14px;background:#f1f1f1;color:#0073aa;text-decoration:none;box-shadow:0 0 2px 2px rgba(0,0,0,.6)}:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}.woocommerce-dashboard__dashboard-charts{border-bottom:0;border-left:0}.woocommerce-dashboard__dashboard-charts .woocommerce-section-header__actions{flex-grow:0}.woocommerce-dashboard__dashboard-charts .woocommerce-card__body{padding:0}.woocommerce-dashboard__dashboard-charts .woocommerce-summary{margin:0}