<?php
if(!defined('ABSPATH')) exit();
?>
	<!-- START RVSLDR Support REVOLUTION SLIDER 6.0.0 -->
						<rs-module-wrap id="rev_slider_733_1_wrapper" data-alias="rvsldr-support" data-source="gallery" style="background:#2a2c2f;padding:0px;margin:0px auto;margin-top:0px;margin-bottom:0px;">
							<rs-module id="rev_slider_733_1" class="feature_slider" style="display:none;" data-version="6.0.0">
								<rs-slides>
									<rs-slide data-key="rs-2447" data-title="Slide" data-anim="ei:d;eo:d;s:300;r:0;t:fade;sl:d;">
										<img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/transparent.png" data-bg="p:center center;c:#2a2c2f;" class="rev-slidebg" data-no-retina>
										<rs-layer
											id="slide-2447-layer-1" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:c;y:b;"
											data-text="l:22;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-border="bow:0px;"
											data-frame_0="sX:1.2;sY:1.2;"
											data-frame_1="st:100;sp:1000;sR:100;"
											data-frame_999="y:bottom;o:1;rX:90deg;st:a;sp:700;sR:7900;"
											style="z-index:5;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="full-proportional" data-hh="full-proportional" width="640" height="370" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/support_bg2.jpg" data-no-retina> 
										</rs-layer>
										<rs-layer
											id="slide-2447-layer-11" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="l:22;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-border="bow:0px;"
											data-frame_0="sX:1.2;sY:1.2;"
											data-frame_1="st:a;sp:1000;sR:100;"
											data-frame_999="y:top;o:1;rX:-90deg;st:a;sp:700;sR:7900;"
											style="z-index:6;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="full-proportional" data-hh="full-proportional" width="640" height="370" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/support_bg-1.jpg" data-no-retina> 
										</rs-layer>
										<rs-layer
											id="slide-2447-layer-15" 
											class="tp-shape tp-shapewrapper"
											data-type="shape"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="fw:700;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-border="bos:solid;boc:rgba(0,0,0,0.5);bow:0px;"
											data-frame_1="sR:10;"
											data-frame_999="o:0;st:w;sR:8690;"
											style="z-index:7;background:linear-gradient(180deg, rgba(0,0,0,0) 0%, rgba(0,0,0,0.00) 0%, rgba(0,0,0,0.65) 100%);"
										> 
										</rs-layer>
										<rs-layer
											id="slide-2447-layer-2" 
											data-type="text"
											data-color="#ffffff"
											data-rsp_ch="on"
											data-xy="x:c;y:c;yo:-20px;"
											data-text="s:25;l:30;fw:700;a:center;"
											data-border="bow:0px;"
											data-frame_0="y:-50;"
											data-frame_1="e:power4.out;st:100;sp:1000;sR:100;"
											data-frame_999="y:50;o:0;e:power4.out;st:a;sp:600;sR:7900;"
											style="z-index:8;font-family:Roboto;"
										>Premium 1on1<br/>Support Center 
										</rs-layer>
										<rs-layer
											id="slide-2447-layer-12" 
											data-type="text"
											data-color="#ffffff"
											data-rsp_ch="on"
											data-xy="x:c;y:c;yo:-20px;"
											data-text="s:25;l:30;fw:700;a:center;"
											data-border="bow:0px;"
											data-frame_0="y:-50;"
											data-frame_1="e:power4.out;st:a;sp:1000;sR:100;"
											data-frame_999="y:50px;o:0;e:power4.out;st:a;sp:600;sR:7900;"
											style="z-index:9;font-family:Roboto;"
										>We are here<br/>to Help! 
										</rs-layer>
										<rs-layer
											id="slide-2447-layer-4" 
											class="rev-btn"
											data-type="button"
											data-color="rgba(255,255,255,1)"
											data-xy="x:c;y:c;yo:40px;"
											data-text="s:15;l:34;fw:500;a:inherit;"
											data-rsp_bd="off"
											data-padding="r:35;l:35;"
											data-border="bos:solid;boc:rgba(0,0,0,1);bow:0px;bor:3px,3px,3px,3px;"
											data-frame_0="y:50;"
											data-frame_1="e:power4.out;st:500;sp:1000;sR:500;"
											data-frame_999="o:0;e:power4.inOut;st:w;sp:1000;sR:7500;"
											data-frame_hover="c:rgba(255,255,255,1);bgc:#0a447a;boc:rgba(0,0,0,1);bor:3px,3px,3px,3px;bos:solid;oX:50;oY:50;sp:300;"
											style="z-index:10;background-color:#006dd2;font-family:Roboto;"
										>Submit a Ticket 
										</rs-layer>
										<a 
											href="//themepunch.com/support-center" 
											target="_blank" 
											class="tp-shape tp-shapewrapper rs-layer rs-waction"
											data-type="shape"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="fw:700;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-actions='o:mouseenter;a:stoplayer;layer:slide-2447-layer-2;||o:mouseenter;a:stoplayer;layer:slide-2447-layer-1;||o:mouseenter;a:startlayer;layer:slide-2447-layer-12;||o:mouseenter;a:startlayer;layer:slide-2447-layer-11;||o:mouseenter;a:startlayer;layer:slide-2447-layer-13;||o:mouseleave;a:stoplayer;layer:slide-2447-layer-12;||o:mouseleave;a:stoplayer;layer:slide-2447-layer-11;||o:mouseleave;a:stoplayer;layer:slide-2447-layer-13;||o:mouseleave;a:startlayer;layer:slide-2447-layer-2;||o:mouseleave;a:startlayer;layer:slide-2447-layer-1;'
											data-basealign="slide"
											data-border="bos:solid;boc:rgba(0,0,0,0.5);bow:0px;"
											data-frame_1="sR:10;"
											data-frame_999="o:0;st:w;sR:8690;"
											style="z-index:11;"
											id="2447a"
										> 
										</a>
									</rs-slide>
								</rs-slides>								
							</rs-module>	
						</rs-module-wrap>
