.wc-block-featured-product {
	.components-resizable-box__handle {
		z-index: 10;
	}
}
.wc-block-featured-product__message {
	margin-bottom: 16px;
}

.wc-block-featured-product__selection {
	width: 100%;
}
