<?php
/**
 * Marketplace suggestions container
 *
 * @package  WooCommerce\Templates
 * @version  3.6.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

?>
<div class="marketplace-suggestions-container"
	data-marketplace-suggestions-context="<?php echo esc_attr( $context ); ?>"
	>
</div>
