<?php
/**
 * Single Product Multiple vendors Load More Button
 *
 * This template can be overridden by copying it to yourtheme/dc-product-vendor/single-product/load-more-button.php.
 *
 * HOWEVER, on occasion WCMp will need to update template files and you (the theme developer).
 * will need to copy the new files to your theme to maintain compatibility. We try to do this.
 * as little as possible, but it does happen. When this occurs the version of the template file will.
 * be bumped and the readme will list any important changes.
 *
 * 
 * @author  WC Marketplace
 * @package dc-woocommerce-multi-vendor/Templates
 * @version 2.3.4
 */ 
global $WCMp; 
?>
<div class = "wcmp-load-more-button-wrapper" ><button id="wcmp-load-more-button" class = "wcmp-load-more-button button" ><?php echo esc_html_e(' Load More ','dc-woocommerce-multi-vendor');?> </button></div>

