/*SORTABLE ICON CSS FOR FRONTEND ADD PRODUCT PAGE*/
/*PRODUCT MANAGER EDIT CSS*/
/*FRONT END CSS*/
/*IMPORT CSS*/
/*CALENDER CSS*/
/*Backup end*/
/* downloadable product */
/*attribute*/
/* N E W */
/* global */
/* elements start */
/* elements end */
/* General Tab Start */
/* General Tab End */
/* Attributeds start */
/* Attributeds End */
/* veriations */
/* rental */
/* booking */
/* Auction */
/* bundle */
/* add on */
/* Rental product settings */
/* select2 */
/* gupis */
/* rental toggle */
/* responsive */
ul {
	list-style: none;
}
div[class^="col-md"] {
	.date-inp-wrap {
		width: 100%;
		.hasDatepicker {
			width: 100%;
		}
	}
}
.wcs_hidden_label {
	display: none !important;
}
#product_images_container {
	background-color: #fff;
	margin-top: 20px;
	ul {
		margin: 0 -10px 0 0;
		padding: 0;
		&::after {
			clear: both;
			content: ' ';
			display: table;
		}
		&::before {
			content: ' ';
			display: table;
		}
		li.image {
			width: 86px;
			float: left;
			cursor: move;
			margin: 3px 3.33% 8px 0;
			border-radius: 2px;
			position: relative;
			-webkit-box-sizing: border-box;
			box-sizing: border-box;
			text-align: center;
			img {
				width: 100%;
				height: auto;
				display: block;
				border: 1px solid #d5d5d5;
			}
		}
		li {
			&:hover {
				ul.actions {
					display: block;
				}
			}
		}
		ul.actions {
			position: absolute;
			top: -8px;
			right: -8px;
			padding: 2px;
			display: none;
			z-index: 1;
			li {
				float: right;
				margin: 0 0 0 2px;
				a.delete {
					display: block;
					text-indent: -9999px;
					position: relative;
					height: 1em;
					width: 1em;
					font-size: 1.4em;
					&::before {
						font-family: Dashicons;
						speak: none;
						font-weight: 400;
						text-transform: none;
						-webkit-font-smoothing: antialiased;
						text-indent: 0px;
						position: absolute;
						top: 0px;
						left: 0px;
						text-align: center;
						content: "";
						color: rgb(153, 153, 153);
						height: 1em;
						width: 1em;
						line-height: 1em;
						font-variant: normal;
						margin: 0px;
						background: rgb(255, 255, 255);
						border-radius: 50%;
					}
				}
				a.tips {
					cursor: pointer;
				}
			}
		}
	}
}
.notice-wrapper {
	position: relative;
	.notice-dismiss {
		position: absolute;
		top: 7px;
		right: 1px;
		border: none;
		margin: 0;
		padding: 9px;
		background: 0 0;
		color: #72777c;
		cursor: pointer;
		&:before {
			background: 0 0;
			color: #72777c;
			content: "\f153";
			display: block;
			font: 400 17px/20px dashicons;
			speak: none;
			height: 20px;
			text-align: center;
			width: 20px;
			-webkit-font-smoothing: antialiased;
			-moz-osx-font-smoothing: grayscale;
		}
	}
}
.notice-dismiss {
	&:hover {
		&:before {
			color: #c00;
		}
	}
}
.sortable-icon {
	position: relative;
	padding-left: 1.25em;
	margin-right: 0.5em;
	&:before {
		content: "";
		position: absolute;
		left: 0;
		top: 0.35em;
		width: 1em;
		height: 0.15em;
		background: currentColor;
		box-shadow: 0 0.25em 0 0 currentColor, 0 0.5em 0 0 currentColor;
	}
}
.downloadable_files {
	tr {
		td {
			&:first-child {
				cursor: move;
			}
		}
	}
	margin: 5px 0;
	table {
		margin: 0;
	}
	table.table {
		>tbody {
			>tr {
				>td {
					vertical-align: middle;
				}
			}
		}
		thead {
			th {
				padding: 8px 20px;
			}
		}
	}
}
.variations-pagenav {
	float: right;
	line-height: 24px;
}
table.dataTable {
	thead {
		.sorting {
			&:after {
				bottom: auto;
				top: 50%;
				transform: translateY(-50%);
			}
		}
		.sorting_asc {
			&:after {
				bottom: auto;
				top: 50%;
				transform: translateY(-50%);
			}
		}
		.sorting_desc {
			&:after {
				bottom: auto;
				top: 50%;
				transform: translateY(-50%);
			}
		}
		.sorting_asc_disabled {
			&:after {
				bottom: auto;
				top: 50%;
				transform: translateY(-50%);
			}
		}
		.sorting_desc_disabled {
			&:after {
				bottom: auto;
				top: 50%;
				transform: translateY(-50%);
			}
		}
	}
}
.dc-wp-fields-uploader {
	.button.ele_hide {
		display: none !important;
	}
	.button.ele_show {
		display: inline-block !important;
	}
}
.wrap.woocommerce.export {
	margin-left: 10px;
	margin-right: 10px;
}
.woocommerce-importer-advanced.hidden {
	display: none;
}
.woocommerce-exporter-wrapper {
	.wc-progress-steps {
		li {
			&::before {
				margin-bottom: -7px !important;
				height: 11px !important;
				width: 11px !important;
			}
		}
	}
}
.woocommerce-importer-wrapper {
	.wc-progress-steps {
		li {
			&::before {
				margin-bottom: -7px !important;
				height: 11px !important;
				width: 11px !important;
			}
		}
	}
}
.woocommerce-progress-form-wrapper {
	.wc-progress-steps {
		li {
			&::before {
				margin-bottom: -7px !important;
				height: 11px !important;
				width: 11px !important;
			}
		}
	}
}
.wrap.woocommerce {
	margin-left: 10px;
	margin-right: 10px;
}
.audio_rm {
	background: #ddd;
	padding: 2px 5px 0 5px;
	border-radius: 100%;
	position: absolute;
	right: 14px;
	top: 3px;
	font-weight: bold;
}
.video_rm {
	background: #ddd;
	padding: 2px 5px 0 5px;
	border-radius: 100%;
	position: absolute;
	right: 14px;
	top: 3px;
	font-weight: bold;
}
.multi_input_block {
	p {
		margin: 10px 0 5px;
	}
}
.hide_visiblity {
	display: none !important;
}
.hide_variation {
	display: none !important;
	+ {
		.select2-container {
			display: none !important;
		}
	}
}
.woocommerce-Tabs-panel--video_audio_tab {
	.frame-wrap {
		&:nth-child(2n+2) {
			margin-right: 0;
		}
	}
}
.multiSelectBox-right {
	margin-top: 6px;
}
.paid_checkbox[type="checkbox"] {
	width: auto !important;
}
.video_uploader_label {
	display: block !important;
}
.blockClass {
	display: block !important;
}
.single-product {
	.ywcfav_thumbnails_video_container {
		display: none;
	}
	.ywcfav_thumbnails_audio_container {
		display: none;
	}
}
body {
	.remove_display {
		display: none !important;
	}
	.hide {
		display: none !important;
	}
	.mcs-horizontal-example {
		.mCSB_draggerRail {
			background: #dae1e7 !important;
			height: 4px !important;
		}
		.mCSB_dragger_bar {
			background: #b1bfca !important;
		}
	}
	div.dataTables_wrapper {
		div.dataTables_length {
			select {
				width: auto;
			}
		}
	}
	.select2-container--default {
		.select2-search--inline {
			.select2-search__field[placeholder="Search {
				for {
					a {
						product…"] {
							min-width: 150px;
						}
					}
				}
			}
		}
	}
}
.clear {
	clear: both;
}
.nobr {
	white-space: nowrap;
}
.wc_bookings_calendar_form {
	margin: 1em 0 0 !important;
	.calendar_days {
		position: relative;
		height: 1440px;
		overflow: hidden;
		overflow-x: scroll;
		border: 1px solid #ddd;
		.hours {
			position: absolute;
			top: 0;
			left: 0;
			margin: -.5em 0 0 0;
			li {
				height: 60px;
				vertical-align: top;
				padding: 0;
				margin: 0;
				label {
					float: left;
					background: #f9f9f9;
					padding: 0 8px;
				}
			}
		}
		.bookings {
			margin: 0 0 0 100px;
			padding: 0;
			height: 100%;
			white-space: nowrap;
			position: relative;
			li {
				position: absolute;
				width: 98px;
				background: #000;
				border-radius: 2px;
				margin: 0;
				overflow: hidden;
				color: #fff;
				a {
					display: block;
					padding: 2px 4px;
					height: 100%;
					width: 100%;
					position: relative;
					color: #fff;
					text-decoration: none;
				}
			}
		}
	}
	.tablenav {
		border: 1px solid #dedede;
		background: #fff;
		height: auto;
		box-shadow: 0 1px 1px -1px rgba(0,0,0,.1);
		-webkit-box-shadow: 0 1px 1px -1px rgba(0,0,0,.1);
		&:after {
			clear: both;
			clear: both;
		}
		.filters {
			float: left;
			line-height: 30px;
			padding: 5px;
			border-right: 1px solid #ddd;
			.chosen-container {
				width: 12em;
			}
		}
		.views {
			float: right;
			padding: 0;
			border-left: 1px solid #ddd;
			a {
				padding: 11px 20px;
				display: block;
				text-decoration: none;
			}
		}
		.date_selector {
			margin-left: 5px;
			display: block;
			float: left;
			div {
				padding: 5px 0;
				float: left;
				&:first-of-type {
					select {
						margin-right: 5px;
					}
				}
			}
			select {
				vertical-align: middle;
				width: 6em;
			}
			a {
				display: block;
				text-indent: -9999px;
				position: relative;
				height: 1em;
				width: 1em;
				font-size: 1.3em;
				padding: .6em;
			}
			a.prev {
				float: left;
			}
			a.next {
				float: right;
			}
		}
	}
	.wc_bookings_calendar {
		width: 100%;
		margin: 1em 0 0 !important;
		th {
			text-align: center;
		}
		td {
			color: #bbb;
			text-align: right;
			padding: 8px 8px 0 0;
			font-size: 20px;
			border-right: 1px solid #ddd;
			border-bottom: 1px solid #ddd;
			height: 120px;
			.bookings {
				overflow: auto;
				height: 100px;
				width: 100%;
				text-align: left;
				font-size: 14px;
				ul {
					padding: 0;
					margin: 0;
					li {
						padding: 0;
						margin: 0 0 4px 0;
						list-style: none;
						a {
							border-left: 4px solid #eee;
							padding: 3px 3px 3px 8px;
							display: block;
							ul {
								padding: 0;
								margin: 0;
								li {
									list-style: none;
									font-size: .75em;
									padding: 0;
									margin: 0;
									border: 0;
								}
							}
							&:hover {
								border-left-color: #ccc;
								background: #fafafa;
							}
						}
					}
				}
			}
		}
		tr {
			background: #fff;
		}
		.calendar-diff-month {
			background: #efefef;
		}
		tbody {
			tr {
				td {
					&:last-child {
						border-right: 0;
					}
				}
				&:last-child {
					td {
						border-bottom: 0;
					}
				}
			}
		}
	}
}
#tiptip_holder {
	display: none;
	z-index: 8675309;
	position: absolute;
	top: 0;
	left: 0;
}
#tiptip_holder.tip_top {
	padding-bottom: 5px;
	#tiptip_arrow_inner {
		margin-top: -7px;
		margin-left: -6px;
		border-top-color: #333;
	}
}
#tiptip_holder.tip_bottom {
	padding-top: 5px;
	#tiptip_arrow_inner {
		margin-top: -5px;
		margin-left: -6px;
		border-bottom-color: #333;
	}
}
#tiptip_holder.tip_right {
	padding-left: 5px;
	#tiptip_arrow_inner {
		margin-top: -6px;
		margin-left: -5px;
		border-right-color: #333;
	}
}
#tiptip_holder.tip_left {
	padding-right: 5px;
	#tiptip_arrow_inner {
		margin-top: -6px;
		margin-left: -7px;
		border-left-color: #333;
	}
}
#tiptip_content {
	color: #fff;
	font-size: .8em;
	max-width: 150px;
	background: #333;
	text-align: center;
	border-radius: 3px;
	padding: .618em 1em;
	-webkit-box-shadow: 0 1px 3px rgba(0,0,0,.2);
	box-shadow: 0 1px 3px rgba(0,0,0,.2);
	code {
		padding: 1px;
		background: #888;
	}
}
.chart-tooltip {
	color: #fff;
	font-size: .8em;
	max-width: 150px;
	background: #333;
	text-align: center;
	border-radius: 3px;
	padding: .618em 1em;
	-webkit-box-shadow: 0 1px 3px rgba(0,0,0,.2);
	box-shadow: 0 1px 3px rgba(0,0,0,.2);
	code {
		padding: 1px;
		background: #888;
	}
}
.wc_error_tip {
	color: #fff;
	font-size: .8em;
	max-width: 150px;
	background: #333;
	text-align: center;
	border-radius: 3px;
	padding: .618em 1em;
	-webkit-box-shadow: 0 1px 3px rgba(0,0,0,.2);
	box-shadow: 0 1px 3px rgba(0,0,0,.2);
	max-width: 20em;
	line-height: 1.8em;
	position: absolute;
	white-space: normal;
	background: #d82223;
	margin: 1.5em 1px 0 -1em;
	z-index: 9999999;
	code {
		padding: 1px;
		background: #888;
	}
	&::after {
		content: '';
		display: block;
		border: 8px solid #d82223;
		border-right-color: transparent;
		border-left-color: transparent;
		border-top-color: transparent;
		position: absolute;
		top: -3px;
		left: 50%;
		margin: -1em 0 0 -3px;
	}
}
#tiptip_arrow {
	position: absolute;
	border-color: transparent;
	border-style: solid;
	border-width: 6px;
	height: 0;
	width: 0;
}
#tiptip_arrow_inner {
	position: absolute;
	border-color: transparent;
	border-style: solid;
	border-width: 6px;
	height: 0;
	width: 0;
}
.inline-select {
	width: auto;
	display: inline-block;
	vertical-align: middle;
}
.row-padding {
	padding: 20px;
}
.select-group {
	display: inline-block;
	vertical-align: middle;
	label {
		font-weight: normal;
		margin: 0 10px 0 0;
	}
}
.add-product-wrapper {
	.product-primary-info {
		.panel.panel-default {
			margin: 0;
		}
		>div {
			float: left;
		}
	}
}
.right-primary-info {
	width: calc(100% - 350px);
	margin-right: 40px;
}
.left-primary-info {
	width: 310px;
}
.product-gallery-wrapper {
	figure {
		background: #fff;
		border: 1px solid #d3dbe2;
		padding: 10px;
		height: 200px;
		position: relative;
		.gallery-img-placeholder {
			position: absolute;
			margin: auto;
			top: 50%;
			left: 0;
			right: 0;
			transform: translateY(-50%);
			i {
				font-size: 55px;
				color: #cfcfcf;
			}
			p {
				margin: 10px 0 0;
				color: #0c0c0c;
			}
		}
	}
}
.product-img-thumbnail {
	list-style: none;
	li {
		display: inline-block;
		width: 80px;
		height: 80px;
		vertical-align: top;
		margin: 0 2% 0 0;
		background: #fff;
		border: 1px solid #d3dbe2;
		&:last-of-type {
			margin: 0;
		}
	}
	.add-gal-img {
		position: relative;
		i {
			position: absolute;
			margin: auto;
			top: 50%;
			left: 0;
			right: 0;
			transform: translateY(-50%);
			font-size: 45px;
			color: #cfcfcf;
		}
	}
}
.gal-img-wrap {
	position: relative;
	.remove-img {
		position: absolute;
		top: 5px;
		right: 5px;
		transform: rotate(45deg);
		.wcmp-font {
			background: #fff;
			border-radius: 50%;
			color: #ff0000;
			font-size: 15px;
			cursor: pointer;
		}
	}
}
.mcs-horizontal-example {
	overflow-x: auto;
	.mCSB_dragger_bar {
		top: 7px;
	}
	#mCSB_2_scrollbar_horizontal {
		bottom: -7px;
	}
	ul {
		white-space: nowrap;
		list-style: none;
		padding: 0;
		margin: 15px 0 0;
		>li {
			display: inline-block;
		}
	}
}
.hidden-file-uploader {
	position: absolute;
	width: 100%;
	height: 100%;
	top: 0;
	left: 0;
	input[type="file"] {
		position: absolute;
		width: 100%;
		height: 100%;
		top: 0;
		left: 0;
		opacity: 0;
		cursor: pointer;
	}
}
.add-product-info-holder {
	background: #fff;
	border: 1px solid #d3dbe2;
	overflow: hidden;
	margin: 25px 0 0;
	.form-group {
		input[type=checkbox] {
			+ {
				label {
					margin: 0 0 0 5px;
				}
			}
		}
		.checkbox-holder {
			margin-top: 3px;
		}
		.wc_booking_restricted_days_field {
			li {
				input[type=checkbox] {
					margin: 0;
				}
			}
		}
	}
}
.add-product-info-header {
	.form-group {
		display: inline-block;
		margin: 0 0 0 15px;
	}
	.pull-right {
		label {
			margin: 6px 0 0 15px;
			font-weight: normal;
		}
		.checkbox-holder {
			margin-right: 3px;
		}
	}
	.select-group {
		label {
			color: #333;
		}
	}
}
.product-info-tab-wrapper {
	clear: both;
	background: #fff;
	position: relative;
	padding: 40px 20px 20px;
	>.tab-content {
		position: relative;
		top: -1px;
		border: 1px solid #bfc3c4;
	}
}
.product-tab-nav-holder {
	background: transparent;
	height: 63px;
	overflow: hidden;
	ul {
		display: flex;
		-webkit-flex-flow: row wrap;
		flex-flow: row wrap;
		justify-content: flex-start;
		align-items: flex-end;
		position: relative;
		z-index: 1;
		border: 0;
		li {
			border-right: 1px solid #fff;
			margin: 0;
			&:last-of-type {
				border: 0;
			}
			a {
				color: #383838;
				border-radius: 0;
				border-width: 2px 1px 2px 1px !important;
				border-color: transparent;
				background: #bfc3c4;
				margin: 0;
				transition: background-color 0.3s;
				padding: 13px 22px;
				&:hover {
					background: #b1becb;
					border-color: #9eacbd;
					border-top-color: transparent;
				}
			}
		}
		li.active {
			a {
				position: relative;
				top: 1px;
				border-bottom-color: transparent !important;
				padding: 20px 22px;
				background-color: #fff;
				color: #222;
				border-color: #bfc3c4;
				box-shadow: none;
				&:hover {
					padding: 20px 22px;
					background-color: #fff;
					color: #222;
					border-color: #bfc3c4;
					box-shadow: none;
				}
				&:focus {
					padding: 20px 22px;
					background-color: #fff;
					color: #222;
					border-color: #bfc3c4;
					box-shadow: none;
				}
			}
		}
	}
}
.add-veriaton-wrapper {
	.select-group {
		&:last-of-type {
			margin: 0 10px 0 25px;
		}
	}
}
.wcmp-metabox-title {
	overflow: hidden;
	padding: 12px 20px !important;
	div[class*="group"] {
		float: left;
		word-wrap: break-word;
		width: calc(100% - 150px);
	}
}
.product-variations-wrapper {
	.wc-metabox {
		.variation-title {
			.ico-up-arrow-icon {
				&::before {
					transition: all 0.3s;
				}
			}
		}
		.variation-title[aria-expanded='true'] {
			.ico-up-arrow-icon {
				&::before {
					content: '\e90f';
				}
			}
		}
		.variation-select-group {
			select {
				margin: 3px;
			}
		}
		&:hover {
			.variation-action {
				visibility: visible;
			}
		}
		.variation-action {
			i {
				margin-right: 15px;
			}
		}
		.variation-content {
			overflow: hidden;
			padding: 20px;
			input[type=checkbox] {
				background: #fff;
			}
			.checkbox-holder {
				background: #fff;
			}
		}
	}
}
.wcmp-metabox-title[aria-expanded='true'] {
	background: #ddd;
}
.wcmp-metabox-action {
	float: right;
	margin: 0;
	width: 150px;
	text-align: right;
	visibility: hidden;
}
.wcmp-metabox-wrapper {
	&:hover {
		.wcmp-metabox-action {
			visibility: visible;
		}
	}
	border: 1px solid #dddddd;
	margin: 15px 0 0;
}
.featured-img {
	background: #fff;
	border: 1px solid #d3dbe2;
	text-align: center;
	font-size: 40px;
	position: relative;
	.upload_image_button {
		display: block;
		position: relative;
		height: 215px;
		img {
			padding: 5px;
			margin: 0 auto;
			width: 100%;
		}
		&:not(.remove) {
			img {
				display: none;
			}
		}
	}
	.upload_image_button.remove {
		height: auto;
		img {
			display: block;
		}
		&::before {
			content: '\f335';
			font-family: 'dashicons';
			display: none;
			line-height: 52px;
			font-size: 50px;
			position: absolute;
			left: 50%;
			top: 50%;
			background: rgb(153, 153, 153);
			transform: translateY(-50%)translateX(-50%);
			border-radius: 50%;
			height: 50px;
			width: 50px;
			text-align: center;
			color: #fff;
		}
		.upload-placeholder {
			display: none;
		}
		&:hover {
			&::before {
				display: block;
			}
		}
	}
}
.variation-img {
	background: #fff;
	border: 1px solid #d3dbe2;
	text-align: center;
	font-size: 40px;
	position: relative;
	display: inline-block;
	vertical-align: top;
	.upload_image_button {
		width: 80px;
		height: 80px;
		min-height: 80px !important;
	}
}
.woocommerce_variable_attributes {
	.upload_image_button {
		display: block;
		position: relative;
		&:not(.remove) {
			img {
				display: none;
			}
		}
	}
	.upload_image_button.remove {
		img {
			display: block;
		}
		&::before {
			content: '\f335';
			font-family: 'dashicons';
			display: none;
			line-height: 52px;
			font-size: 50px;
			position: absolute;
			left: 50%;
			top: 50%;
			background: rgb(153, 153, 153);
			transform: translateY(-50%)translateX(-50%);
			border-radius: 50%;
			height: 50px;
			width: 50px;
			text-align: center;
			color: #fff;
		}
		&:hover {
			&::before {
				display: block;
			}
		}
	}
}
.upload-placeholder {
	p {
		font-size: 15px;
	}
}
.remove {
	.upload-placeholder {
		display: none;
	}
}
.attribute-chk-option {
	.form-group {
		width: 50%;
		vertical-align: top;
		float: left;
		margin: 5px 0 0;
	}
}
.collapse {
	overflow: hidden;
}
label.col-md-12.control-label {
	margin-bottom: 6px;
}
.border-wrapper {
	border: 1px solid #c5c5c5;
	padding: 15px 20px;
}
.downloadable-files {
	tr {
		td {
			&:not(.sort) {
				padding: 0 6px 6px;
			}
		}
	}
	.delete.btn {
		background: #c6c6c6;
		padding: 5px;
		border-radius: 50%;
		width: 30px;
		font-size: 16px;
		line-height: 20px;
		height: 30px;
		color: #fff;
		&:hover {
			background: #e55959;
		}
	}
	tfoot {
		button {
			margin-top: 10px;
		}
	}
}
.product_attributes.wc-metaboxes {
	.woocommerce_attribute {
		input[type=checkbox] {
			vertical-align: middle;
		}
		label {
			font-weight: normal;
		}
		.woocommerce_attribute_data {
			padding: 0;
		}
	}
}
td.attribute_name {
	width: 230px;
	padding: 0 6px 10px 0;
}
.selectTowrap {
	.select2.select2-container {
		margin-bottom: 5px;
	}
}
.margin-0 {
	margin: 0;
}
label {
	font-weight: normal;
}
.wcmp-2x {
	font-size: 2rem;
}
.wcmp-3x {
	font-size: 3rem;
}
#woocommerce-product-data {
	hr {
		margin-bottom: 0;
		margin-top: 15px;
	}
}
.form-horizontal {
	.control-label.col-md-12 {
		padding-right: 40px;
		.img_tip {
			margin: 10px 17px 0 0;
		}
	}
}
.custom-panel {
	background-color: #fff;
	padding: 15px;
	border: 1px solid #d3dbe2;
	overflow: hidden;
	h3 {
		margin: -7px 0 10px 0;
		color: #565656;
		padding: 0 0 10px;
		position: relative;
		&:after {
			content: '';
			height: 1px;
			background: #d3dbe2;
			position: absolute;
			left: 0;
			margin: 0 -15px;
			right: 0;
			bottom: 0;
		}
	}
}
.button-group {
	margin-top: 15px;
}
.add_product_images {
	margin: 10px 0;
}
.form-text {
	display: block;
	margin: 6px 0 0;
	i {
		vertical-align: middle;
	}
}
.upload_file_button {
	.wcmp-font {
		font-size: 22px;
	}
}
.delete {
	.wcmp-font {
		font-size: 22px;
	}
}
.product_attributes {
	table.table {
		margin: 15px 0;
		td {
			padding: 4px 15px;
			border: 0;
		}
	}
}
.woocommerce_attribute {
	table {
		margin: 0;
	}
}
.woocommerce_variations {
	margin-top: 15px;
}
.woocommerce_variations.product-variations-wrapper {
	.wc-metabox {
		.variation-action {
			margin-top: 11px;
		}
	}
}
.table.wcmp-metabox-table {
	td {
		border: 0;
		padding: 12px 20px;
	}
	th {
		border: 0;
		padding: 12px 20px;
	}
}
#variable_product_options_inner {
	hr {
		margin-bottom: 15px;
	}
}
.booking_range_pricing {
	table {
		.select {
			+ {
				.form-control {
					margin-top: 10px;
				}
			}
		}
	}
}
.wc_booking_restricted_days_field {
	padding: 15px 20px;
	border: solid 1px #ccc;
	text-align: center;
	li {
		display: inline-block;
		vertical-align: middle;
		margin-right: 30px;
		label {
			margin: 0;
		}
	}
}
.create-order-label-wrap {
	label {
		display: block;
		margin-bottom: 10px;
		input[name="booking_order_id"] {
			width: 150px;
			display: inline-block;
			vertical-align: middle;
			margin-left: 6px;
		}
	}
}
.wc-bookings-booking-form {
	.block-picker {
		li {
			&:nth-child(odd) {
				a {
					background: #efefef;
					&:focus {
						background: #316fa8;
					}
					&:hover {
						background: #316fa8;
					}
				}
			}
			&:nth-child(even) {
				a {
					background: #f5f5f5;
					&:focus {
						background: #316fa8;
					}
					&:hover {
						background: #316fa8;
					}
				}
			}
		}
	}
}
.wc-bookings-date-picker {
	.ui-datepicker {
		td.bookable {
			a {
				background-color: #3f85b9 !important;
			}
		}
		td.ui-datepicker-current-day {
			a {
				background: #316fa8 !important;
			}
		}
		td.partial_booked {
			a {
				&:before {
					border-top: 2.5em solid #7eb5de;
				}
			}
		}
	}
}
.fc-row {
	.fc-content-skeleton {
		td {
			border-color: #d3dbe2;
		}
	}
	.fc-helper-skeleton {
		td {
			border-color: #d3dbe2;
		}
	}
}
.fc-content-skeleton {
	td {
		padding: 10px 0 0 10px;
	}
}
.table.vertical-th {
	tr {
		th {
			padding: 20px;
		}
	}
}
.fc-basic-view {
	.fc-body {
		.fc-row {
			min-height: 1em;
		}
	}
}
.woocommerce_booking_resource {
	.wcmp-metabox-title {
		div[class*="group"] {
			width: calc(100% - 190px);
		}
	}
	.wcmp-metabox-action {
		width: 190px;
		.wcmp-font.ico-up-arrow-icon {
			margin-right: 5px;
		}
	}
}
.auction_status_wrapper {
	margin-bottom: 15px;
	font-weight: bold;
	p {
		span {
			font-weight: normal;
		}
	}
}
.item_visibility {
	.col-md-6 {
		>div {
			margin-bottom: 20px;
		}
		input[type="checkbox"] {
			margin-top: 0;
		}
	}
}
.inline-description {
	display: inline-block;
	vertical-align: middle;
	margin-left: 7px;
}
.addon_required {
	label {
		display: block;
	}
}
.sub-tab-wrapper {
	border: 1px solid #e2e1e1;
	overflow: hidden;
	display: flex;
}
.sub-tab-nav {
	width: 200px;
	max-width: 100%;
	display: inline-block;
	vertical-align: top;
	box-shadow: 0px 2px 20px rgba(0,0,0,0.20);
	li {
		border-bottom: 1px solid #e2e1e1;
		a {
			display: block;
			color: #212121;
			font-size: 14px;
			padding: 12px 16px;
			&:hover {
				background: #f4f4f4;
			}
			&:focus {
				background: #f4f4f4;
			}
			i {
				font-size: 22px;
				vertical-align: middle;
				margin-right: 6px;
			}
		}
	}
	li.active {
		a {
			background: #f4f4f4;
		}
	}
}
.sub-tab-content {
	width: calc(100% - 200px);
	float: right;
	min-height: 200px;
}
.select2-container--default {
	.select2-selection--single {
		height: 100%;
		border: 1px solid #c5c5c5;
		.select2-selection__rendered {
			padding-top: 3px;
		}
		.select2-selection__arrow {
			top: 4px;
			right: 9px;
		}
	}
}
.select2-container {
	min-height: 34px;
	.select2-search--inline {
		margin-top: 8px;
	}
	.select2-selection.select2-selection--multiple {
		min-height: 34px;
		.select2-selection__rendered {
			.select2-selection__choice {
				margin: 6px 8px 0 0;
			}
		}
	}
}
.select2-container--default.select2-container--focus {
	.select2-selection--multiple {
		border: solid #c5c5c5 1px;
	}
}
.select2-results__options {
	[data-selected="true"] {
		background: #ddd;
	}
}
.nav-tabs {
	transition: all 0.3s;
}
.tab-nav-direction-wrapper {
	position: absolute;
	top: 5px;
	right: 20px;
	bottom: 0;
	width: 51px;
	height: 25px;
	.nav-btn {
		position: absolute;
		top: 0;
		bottom: 0;
		cursor: pointer;
		z-index: 1;
		background: #979b9c;
		border: 0;
		color: #fff;
		width: 25px;
		height: 25px;
		font-size: 11px;
		text-align: center;
		cursor: pointer;
		border-radius: 3px;
		z-index: 1;
		&:hover {
			background: #3f85b9;
		}
	}
	.nav-btn.next {
		right: 0;
	}
}
.title-with-type {
	.wcmp-metabox-action {
		margin-top: 11px;
	}
}
.days-range-wrapper {
	.redq_days_range {
		.days-range-title[aria-expanded='true'] {
			.ico-up-arrow-icon {
				&::before {
					content: '\e90f';
				}
			}
		}
	}
}
.price-discount-wrapper {
	.redq_price_discount {
		.price-discount-title[aria-expanded='true'] {
			.ico-up-arrow-icon {
				&::before {
					content: '\e90f';
				}
			}
		}
	}
}
.rental-inventory-wrapper {
	.redq_inventory {
		.inventory-title[aria-expanded='true'] {
			.ico-up-arrow-icon {
				&::before {
					content: '\e90f';
				}
			}
		}
	}
}
@media screen and (max-width: 767px) {
	.frame-wrap {
		width: 100%;
		text-align: center;
		margin: 0 0 25px;
	}
	.woocommerce-Tabs-panel--video_audio_tab {
		.frame-wrap {
			&:last-child {
				margin-bottom: 0;
			}
		}
	}
	.control-label {
		+ {
			div[class*="col-md"] {
				.row {
					div[class*="col-md"] {
						&:not(:last-of-type) {
							margin-bottom: 10px;
						}
					}
				}
			}
		}
	}
}
@media screen and (max-width: 640px) {
	.multiSelectBox-right {
		margin-left: 0;
	}
	.add-product-info-header {
		text-align: center;
		.select-group {
			+ {
				.pull-right {
					float: none !important;
					margin: 14px auto 0;
				}
			}
		}
	}
	.form-horizontal {
		.form-group {
			input[type="checkbox"] {
				margin-top: 0;
			}
		}
	}
	.product_attributes {
		table.table {
			td {
				padding: 4px 8px;
			}
		}
	}
}
@media screen and (max-width: 1080px) {
	.right-primary-info {
		width: 100%;
		margin: 0 0 20px;
	}
	.left-primary-info {
		width: 100%;
	}
	.featured-img {
		.upload_image_button.remove {
			&::before {
				display: block !important;
			}
		}
	}
	.woocommerce_variable_attributes {
		.upload_image_button.remove {
			&::before {
				display: block !important;
			}
		}
	}
	#product_images_container {
		ul {
			ul.actions {
				display: block !important;
			}
		}
	}
}
@media screen and (min-width:991px) {
	.table.wcmp-metabox-table {
		width: 75%;
	}
}
@media screen and (max-width: 480px) {
	.product-info-tab-wrapper {
		padding: 40px 10px 20px;
	}
	.row-padding {
		padding: 10px;
	}
}
