$light_blue: #0073aa;
$light_grey: #f7f7f7;
$mid_grey: #f1f1f1;
$grey: #DFDFDF;
$dark_grey: #999;
$text_grey: #444;
$white: #fff;
$black: #000;
$red: #D0011B;
$orange: #ffb900;
$green: #46b450;
$header_height: 75px;

@mixin tablet {
    @media only screen and (max-width : 1200px) {
        @content;
    }
}

.rtl.nav-menus-php {
	.mm_launch {
		margin-left: 0;
		margin-right: 10px;

		&:before {
			margin-left: 2px;
			margin-right: 0;
		}
	}
	#cboxContent .mm_tab_container {
		float: right;
	}
	#cboxContent .mega_menu .mm_panel_options {
		float: left;
	}
	#cboxContent .mm_header_container .mm_title {
		float: right;
	}
	#cboxContent table .mega-menu-item-align td.mega-value select {
		float: right;
	}
	#cboxContent table .mega-menu-item-align td.mega-value .mega-description {
		float: right;
		margin-right: 10px;
	}
	#cboxContent .mm_tab_container .mm_tab.active:after {
		right: auto;
		left: 0;
	}
	#cboxContent .mega_menu .widget-title-action {
		direction: ltr;
	}
	.mega_menu_meta_box div table td:nth-child(2) {
		text-align: left;
	}
}

body[class*='_page_maxmegamenu_'],
body.toplevel_page_maxmegamenu,
.nav-menus-php {

	.sp-container {
		border: 1px solid #7e8993;
	}

	input.button-primary.is-busy {
		pointer-events: none;
		background-size: 100px 100% !important;
	    background-image: linear-gradient(-45deg,#007cba 28%,#006395 0,#006395 72%,#007cba 0) !important;
	    border-color: #007cba !important;
	    animation: busy-animation 2.5s linear infinite;
	}

	.mega-tooltip[data-tooltip],
	.mega-tooltip[data-tooltip-enabled],
	.mega-tooltip[data-tooltip-disabled] {
		position: relative;

		&.mega-enabled:before {
			content: attr(data-tooltip-enabled);
		}

		&.mega-disabled:before {
			content: attr(data-tooltip-disabled);
		}

		&:before {
			position: absolute;
			content: attr(data-tooltip);
			top: -32px;
			background: $black;
			color: $white;
			padding: 3px 0;
			font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif;
			font-size: 10px;
			font-weight: normal;
			border-radius: 3px;
			margin-left: -10px;
			width: 160px;
			z-index: 2;
		}

		&:after {
			top: -8px;
			left: 8px;
			border: solid transparent;
			content: " ";
			height: 0;
			width: 0;
			position: absolute;
			border-color: transparent;
			border-top-color: $black;
			border-width: 4px;
			margin-left: -4px;
		}

		&:before,
		&:after {
			opacity: 0;
			pointer-events: none;
			text-align: center;
			transition: all 0.3s;
			transition-delay: 0.3s;
		}

		&:focus:before,
		&:focus:after,
		&:hover:before,
		&:hover:after {
			opacity: 1;
		}

	}
}

.nav-menus-php {
	#cboxOverlay {
		left: 160px;
	}

	#colorbox {
		transform: translateX(80px);
	}

	&.mega-colorbox-open {
		ul#adminmenu a.wp-has-current-submenu:after, 
		ul#adminmenu>li.current>a.current:after {
			border-right-color: #373737;
		}
	}

	#cboxContent {
		padding: 0;
		user-select: none;

		.CodeMirror {
			user-select: auto;
		}

		.notice {
			margin: 10px 0 0 0;

			.notice-dismiss:focus {
				box-shadow: none;
			}
		}

		.mm_tabs {
			li.mm_tab_horizontal {
				float: left;
				cursor: pointer;
				font-size: 12px;
				font-weight: normal;
				padding: 3px 5px;
				border: 0;
				margin: 0 15px 0 0;
				background: white;
				color: #444;
				border-radius: 3px;
				background: #f7f7f7;
				border: 1px solid #DFDFDF;

				&.active,
				&:hover {
					background: $light_blue;
					border-color: $light_blue;
					color: white;
				}
			}

			&.horizontal {
				width: 85%;
				float: left;
				margin-top: 10px;
				margin-bottom: 20px;
			}
		}


		.filter_icons {
			width: 14%;
			float: right;
			margin-top: 0;
			font-size: 0.9em;
		}

		.mm_header_container {
			float: left;
			width: 100%;
			border-bottom: 1px solid $mid_grey;
			background: $mid_grey;

			.mm_title {
				padding: 0 20px;
				font-size: 1.3em;
			    align-items: center;
			    display: inline-flex;
			    line-height: 25px;
				float: left;
				height: $header_height;

				br {
					display: none;
				}
			}

			.mm_saving {
				text-align: right;
				padding: 0 20px;
				display: none;
				line-height: $header_height;
				text-transform: uppercase;
				color: $light_blue;
				font-size: 11px;
				font-weight: bold;
				float: right;
				background: $white;
				position: absolute;
				right: 54px;
				
				&:before {
					color: $light_blue;
					display: inline-block;
					font: normal 20px/#{$header_height} 'dashicons';
					speak: none;
					-webkit-font-smoothing: antialiased;
					-moz-osx-font-smoothing: grayscale;
					vertical-align: top;
					margin-right: 5px;
					content: '\f463';
					-webkit-animation: rotation 2s infinite linear;
					animation: rotation 2s infinite linear;
				}
			}

		}

		.mm_tab_container {
			width: 15%;
			float: left;
			position: relative;
			height: 504px;
			background: white;
			-webkit-box-shadow: inset -10px 0px 7px -12px rgba(0, 0, 0, 0.2);
			-moz-box-shadow: inset -10px 0px 7px -12px rgba(0, 0, 0, 0.2);
			box-shadow: inset -10px 0px 7px -12px rgba(0, 0, 0, 0.2);

			.mm_tab {
				text-decoration: none;
				color: $text_grey;
				width: 100%;
				display: block;
				padding: 10px 10px 10px 15px;
				box-sizing: border-box;
				outline: 0;
				position: relative;
				transition: all 0.1s linear;
				font-size: 12px;
				cursor: pointer;
			}

			.mm_tab:hover {
				background: $light_grey;
			}

			.mm_tab:before {
				color: $dark_grey;
				display: inline-block;
				font-family: 'dashicons';
				speak: none;
				-webkit-font-smoothing: antialiased;
				-moz-osx-font-smoothing: grayscale;
				vertical-align: top;
				margin-right: 10px;
				content: '\f463';
				font-size: 15px;
				width: 14px;
			}

			.mm_tab.mega_menu:before {
				content: "\f116";
			}

			.mm_tab.general_settings:before {
				content: "\f111";
			}
			
			.mm_tab.menu_icon:before {
				content: "\f128";
			}
			.mm_tab.styling:before {
				content: "\f540";
			}
			
			.mm_tab.roles:before {
				content: "\f110";
			}
			.mm_tab.active {
				color: white;
				background: $light_blue;

				&:before {
					color: white;
				}
				&:after {
					right: 0;
					border: solid 5px transparent;
					content: " ";
					height: 0;
					width: 0;
					position: absolute;
					pointer-events: none;
					border-right-color: white;
					top: 50%;
					margin-top: -5px;
				}
			}


		}

		p.submit {
			margin-bottom: 0;
			padding-bottom: 20px;
			position: sticky;
		    bottom: 0;
		    background: white;
		}

		.mm_content_container {
			padding: 20px 20px 0 20px;
			float: right;
			height: 504px;
			width: 85%;
			overflow: auto;
			box-sizing: border-box;
			position: relative;
		}

		.mm_content.mega_menu {
			#widgets,
			#mm_widget_selector,
			#mm_number_of_columns,
			#megamenu-grid {
				display: none;
			}
		}

		.mm_content.mega_menu[data-type='grid'] {
			#megamenu-grid,
			#mm_widget_selector {
				display: block;
			}
		}

		.mm_content.mega_menu[data-type='megamenu'] {
			#widgets,
			#mm_widget_selector,
			#mm_number_of_columns {
				display: block !important;/* @todo remove !important once MMM Pro 1.7.1 is released */
			}
		}

		.mm_content.mega_menu {

			.mm_panel_options {
				float: right;
			}

			#megamenu-grid {
				margin-top: 20px;

				button.mega-add-row,
				button.mega-add-column {
					height: 23px;
					min-height: 23px;
					line-height: 22px;
					font-size: 12px;
					padding-left: 4px;

					.dashicons-plus {
						margin-right: 4px;
						line-height: 22px;
					}
				}

				> .drop-area {
					margin-top: 20px;
					border: 1px dashed $dark_grey;
					max-height: 100px !important;
				}

				.mega-row {
					width: 100%;
					display: inline-block;
					background-color: $light_grey;
					border: 1px solid $grey;
					clear: both;
					padding: 10px 5px;
					margin-top: 20px;
					box-sizing: border-box;
					position: relative;
					min-height: 100%;

					&:first-child {
						margin-top: 0;
					}

					.mega-row-header {
						display: inline-block;
						width: 100%;
						padding: 5px 0;
						box-sizing: border-box;

						.mega-disabled .dashicons {
							opacity: 1;
						}

						.dashicons.mega-settings-open {
							opacity: 1;
						}

						.dashicons {
							opacity: 0.2;
							transition: opacity 0.5s;
						}

						.dashicons-plus {
							opacity: 1;
						}
						
						.mega-row-actions {
							display: inline-block;
							
							.dashicons-trash {
								display: none;
								color: #D0011B;
							}
						}


						.mega-row-actions:hover .dashicons {
							opacity: 1;
						}
					}

					&[data-used-cols='0'] {
						.mega-row-header .mega-row-actions .dashicons-trash {
							display: inline-block;
							opacity: 1;
						}
					}

					.mega-too-many-cols {
						display: none;
					}

					&[data-too-many-cols='true'] {
						.mega-too-many-cols {
							display: block;
						}
					}

					.mega-row-is-full {
						display: none;
					}

					&[data-row-is-full='true'] .mega-col-expand,
					.mega-col[data-span='1'] .mega-col-contract {
						pointer-events: none;
						color: $grey;
					}
					
					&[data-row-is-full='true'] {
						.mega-col-expand {
							pointer-events: none;
							color: $grey;
						}
					}

	                @for $of from 1 through 12 {
	                    @for $cols from 1 through $of {
	                        &[data-available-cols='#{$of}'] .mega-col[data-span='#{$cols}'] {
	                            width: (100% / $of) * $cols;
	                        }
	                    }
	                }

					.notice {
						margin: 10px 5px;
					    cursor: pointer;
					}
				}

				.mega-col {
					display: inline-block;
					position: relative;
					vertical-align: top;
					float: left;
					transition: width 0.1s;
					width: 100%;

					.mega-col-wrap {
						background-color: $white;
						border: 1px solid $grey;
						margin: 3px;
						padding: 8px;
					}

					.mega-col-widgets {
						min-height: 50px;

						.widget {
							float: none;
							width: 100%;
							border: 0;
							margin: 3px 0;

							.widget-top {
								background: $light_grey;
								border: 1px solid $grey;
								height: 42px;
							}

							&.open .widget-top {
								border: 1px solid $dark_grey;
								border-bottom: 0;
							}


							&.open .widget-inner {
								border: 1px solid $dark_grey;
							}

							.widget-title-action {
								background: $light_grey;
								border: 0;
								padding-left: 0;
								padding-right: 10px;
								position: absolute;
								top: 0;
							}

							&[data-type=item] form {
								input, select {
									float: right;
								}

								p {
									float: left;
									width: 100%;
									margin-bottom: 0;
								}

								.widget-content {
									padding-top: 10px;
								}
							}

							.widget-title {

								h4 {
									font-weight: normal;
									margin-top: 5px;
									line-height: 16px;
								}

								.widget-desc {
									clear: both;
									font-weight: normal;
									font-size: 10px;
									opacity: 0.6;
									float: left;
									font-style: italic;
								}

							}

						}

					}

					.mega-col-description {
						float: left;

						.mega-disabled .dashicons {
							opacity: 1;
						}

						.dashicons.mega-settings-open {
							opacity: 1;
						}
						
						.dashicons {
							opacity: 0.2;
							transition: opacity 0.5s;
						}

						.dashicons-trash {
							display: none;
							color: #D0011B;
						}
					}

					&[data-total-blocks='0'] {
						.dashicons-trash {
							display: inline-block;
							opacity: 1;
						}
					}

					.mega-col-header {
						display: inline-block;
						width: 100%;

						.mega-col-description:hover .dashicons {
							opacity: 1;
						}

						.mega-col-actions {
							float: right;

							.dashicons {
								margin-right: 0;
								width: 10px;
								cursor: pointer;
							}
							> span {
								font-size: 10px;
								position: relative;
								top: -3px;
							}

						}

					}

				}

				button.mega-add-column {
					position: absolute;
				    right: 10px;
				    top: 10px;
				}

				button.mega-add-row {
					margin-top: 20px;
				}

				.mega-col-settings {
					float: left;
					width: 100%;
					display: none;
					margin: 5px 0 20px;
					padding: 10px;
					box-sizing: border-box;

					label {
						width: 100%;
						font-size: 11px;
						text-transform: uppercase;
						margin-right: 5px;
					}

					input {
						font-size: 11px;
						max-width: 100%;
						display: block;
						width: 100%;
						margin: 5px 0 10px 0;
					}

					button {
						display: block;
					}
				}

				.mega-row-settings {
					float: left;
					width: 100%;
					display: none;
					margin: 5px 0 20px;
					padding: 10px;
					box-sizing: border-box;

					.mega-settings-row {
						margin-bottom: 5px;
					}

					label {
						width: 150px;
						font-size: 11px;
						text-transform: uppercase;
						display: inline-block;
					}

					input,
					select {
						font-size: 11px;
					}

					button {
						display: block;
					}
				}
				.mega-row-header .dashicons,
				.mega-col-header .dashicons {
					font-size: 13px;
					width: 15px;
					cursor: pointer;
				}

				.dashicons-sort,
				.dashicons-move {
					cursor: move;
				}

				.mega-disabled .dashicons {
					&:before {
						color: $red;
					}
				}

				.mega-add-row {
					clear: both;
				}

				.drop-area {
					display: inline-block;
					-webkit-box-sizing: border-box;
					-moz-box-sizing: border-box;
					box-sizing: border-box;
				}

				.ui-sortable-helper {
					opacity: 0.5;
				}

			}

			#widgets {
				float: left;
				background: $light_grey;
				border: 1px solid $grey;
				width: 760px;
				position: relative;
				margin-top: 15px;
				padding: 5px;
				float: left;
				width: 100%;
				box-sizing: border-box;
				min-height: 200px;

				.no_widgets {
					font-size: 0.9em;
					margin-left: 10px;
					font-style: italic;
					margin-top: 10px;
					clear: both;
					color: $text_grey;
					float: left;
				}

				.drop-area {
					display: block;
					border: 1px solid $mid_grey;
					float: left;
					-webkit-box-sizing: border-box;
					-moz-box-sizing: border-box;
					box-sizing: border-box;
				}

				.widget {
					width: 100%;

					h4 {
						max-width: 700px;
					}
				}
			}

			select#mm_enable_mega_menu {
				margin-bottom: 0;
				font-size: 0.9em;
				margin: 0 5px;
			}

			select#mm_widget_selector {
				font-size: 0.9em;
				float: right;
			}

			select#mm_number_of_columns {
				font-size: 0.9em;
				float: right;
				margin-right: 10px;
			}

			.mm_header {
				float: right;
			}

			.widget {
				-webkit-box-sizing: border-box;
				-moz-box-sizing: border-box;
				box-sizing: border-box;
				width: 160px;
				margin: 0;
				color: $text_grey;
				font-size: 12px;
				display: inline-block;
				float: left;
				border: 5px solid transparent;

				&:hover {
					.widget-top {
						border: 1px solid $text_grey;
					}
					.widget-inner {
						border: 1px solid $text_grey;
					}
				}

				textarea {
					max-height: 100px;
				}
			}

			.widget.open {
				z-index: 999;

				.widget-top {
					border: 1px solid $text_grey;
					cursor: move;
					z-index: 1001;
					position: relative;
					border-bottom: 0;
				}

				.widget-inner {
					border: 1px solid $text_grey;
					position: absolute;
					z-index: 1000;
					-webkit-box-shadow: 0 1px 1px rgba(0, 0, 0, 0.04);
					box-shadow: 0 1px 1px rgba(0, 0, 0, 0.04);
					margin-top: -1px;
					background: white;
					min-width: 340px;
					box-sizing: border-box;
					display: block;
				}

				.widget-action {
					&::after {
						content: '\f142';
					}
				}
			}

			.widget-top {
				border: 0;
				-webkit-box-shadow: 0 0 0;
				box-shadow: 0 0 0;
				border: 1px solid $grey;
				padding: 0 8px;
				background: white;
				overflow: hidden;
			}



			.widget.sub_menu {
				.widget-top {
					cursor: default;
				}
			}

			.widget-title {
				h4 {
					color: $text_grey;
					float: left;
					margin: 0;
					padding: 0;
					overflow: hidden;
					white-space: nowrap;
					border-bottom: 0;
					font-size: 12px;
					text-overflow: ellipsis;
					line-height: 42px;

					&:after {
						color: #d54e21;
						display: none;
						font: normal 20px/1 'dashicons';
						speak: none;
						-webkit-font-smoothing: antialiased;
						-moz-osx-font-smoothing: grayscale;
						vertical-align: top;
						margin-left: 5px;
						content: '\f463';
						-webkit-animation: rotation 2s infinite linear;
						animation: rotation 2s infinite linear;
						vertical-align: middle;
					}
				}

				h4.loading {
					&:after {
						display: inline-block;
					}
				}
			}

			.widget[data-type='menu_item'] {
				.widget-title {
					h4 {
						&:before {
							color: $text_grey;
							font: normal 16px/1 'dashicons';
							speak: none;
							-webkit-font-smoothing: antialiased;
							-moz-osx-font-smoothing: grayscale;
							vertical-align: top;
							margin-right: 2px;
							content: "\f333";
							vertical-align: middle;
							top: -1px;
							position: relative;
						}
					}
				}
				form {
					input, select {
						float: right;
					}

					p {
						float: left;
						width: 100%;
						margin-bottom: 0;
					}
				}
			}

			.widget-controls {

				float: left;

				.delete:hover {
					text-decoration: none;
					color: $red;
				}
			}

			.widget-inner {
				display: none;
				float: left;
				width: 100%;
				background: white;

				form {
					padding: 0 15px 15px 15px;
					margin-bottom: 30px;
				}
				p {
					font-size: 12px;
				}
				select {
					font-size: 12px;
				}
				input {
					font-size: 12px;
				}
			}

			.widget-title-action {
				padding: 0;
				position: relative;
				font-weight: normal;
				position: absolute;
				right: 1px;
				background: white;
				border-right: 7px solid white;
				padding-left: 10px;
				opacity: 1;
				background: linear-gradient(to right, rgba(255, 255, 255, 0) 0%, white 9%, white 100%);

				a {
					display: inline-block;
					line-height: 40px;
				}
			}

			.widget-option {
				right: 0;
				border: 0;
				background: 0 0;
				font: 400 16px/1 dashicons;
				speak: none;
				display: block;
				padding: 0;
				text-indent: 0;
				text-align: center;
				position: relative;
				-webkit-font-smoothing: antialiased;
				-moz-osx-font-smoothing: grayscale;
				text-decoration: none !important;
				color: $dark_grey;

				&:hover {
					color: $text_grey;
					cursor: pointer;
				}

				&::after {
					padding: 0;
					font: 400 14px/40px dashicons;
					display: inline;
				}
			}

			.widget-cols {
				line-height: 40px;
				vertical-align: top;
				display: inline-block;
				color: $dark_grey;
				font-size: 11px;
				vertical-align: bottom;
			}

			h5 {
				margin: 10px 0 0 10px;
				text-transform: uppercase;
				float: left;
			}

			.widget-expand {
				&::after {
					content: '\f345';
				}
			}

			.widget-contract {
				&::after {
					content: '\f341';
				}
			}

			.widget-action {
				&::after {
					content: '\f107';
					margin: 0;
				}
			}

			.widget-option.disabled {
				&:hover {
					color: $dark_grey;
				}
			}

			.widget[data-columns='1'] {
				.widget-contract {
					display: none;
				}
			}

			#widgets {
                @for $of from 1 through 12 {
                    @for $cols from 1 through $of {
                        &[data-columns='#{$of}'] .widget[data-columns='#{$cols}'] {
                            width: (100% / $of) * $cols;
                        }
                    }
                }
			}

		}

		label[for='mm_enable_mega_menu'] {
			font-size: 12px;
		}

		.menu_icon {
			.icon_selector {
				clear: both;

				>div {
					float: left;
					padding: 0 19px 19px 0;
					width: 24px;
					height: 24px;
					box-sizing: initial;
				}

				label {
					float: left;
					display: block;
					width: 24px;
					height: 24px;

					&:before {
						width: 24px;
						height: 24px;
						display: inline-block;
						content: attr(rel);
						border: 1px solid $grey;
						padding: 6px;
						background: $light_grey;
						color: $dark_grey;
					}
				}

				input.radio {
					&:empty {
						margin-left: -9999px;

						~ {
							label {
								position: relative;
								cursor: pointer;
								-webkit-user-select: none;
								-moz-user-select: none;
								-ms-user-select: none;
								user-select: none;
							}
						}
					}

					&:checked {
						~ {
							label {
								&:before {
									border: 1px solid #0074a2;
									color: #0074a2;
									background: white;
								}
								color: $dark_grey;
							}
						}
					}

					&:hover {
						&:checked {
							~ {
								label {
									&:before {
										border: 1px solid #0074a2;
										color: #0074a2;
										background: white;
									}
								}
							}
						}
						~ {
							label {
								&:before {
									color: #0074a2;
								}
							}
						}
					}
				}

				.dash {
					label {
						&:before {
							font: 400 24px/1 dashicons;
						}
					}
				}
			}
		}

		table {

			td {
				padding-bottom: 10px;
			}

			td.mega-name {
				width: 27%;
				line-height: 2em;
				vertical-align: top;
				font-size: 12px;

				.mega-description {
					font-size: 0.9em;
					color: $dark_grey;
					line-height: 1.5em;

					p {
						font-size: 0.9em;
					}
				}
			}

			td.mega-value {
				font-size: 12px;

				select {
					font-size: 0.9em;
				}
				textarea {
					font-size: 0.9em;
					height: 28px;
					line-height: 28px;
				}
				input[type=text] {
					font-size: 0.9em;
					height: 28px;
					line-height: 28px;
				}
				input[type=number] {
					font-size: 0.9em;
					height: 28px;
					line-height: 28px;
				}
				.mega-description {
					font-size: 0.9em;
					color: $dark_grey;
					line-height: 1.5em;
					text-indent: 2px;
					margin-top: 5px;
					display: inline;
					font-style: italic;

					p {
						font-size: 0.9em;
					}
				}
			}

			.mega-menu-item-align {
				td.mega-value {
					select {
						float: left;
					}
					.mega-description {
						float: left;
						margin-left: 10px;
					}
				}
			}

			.mega-sub-menu-align {
				td.mega-value {
					select {
						float: left;
						margin-right: 10px;
					}
					.mega-description {
						float: left;
						margin-right: 10px;
					}
				}
			}

			input {
				width: 70px;
				margin-right: 20px;
			}

			label {
				font-size: 0.9em;
			}

			input[type=checkbox] {
				width: 16px;
				height: 16px;
				margin-right: 10px;
				vertical-align: middle;
			}
			width: 100%;
			border-collapse: collapse;
		}

		.general_settings {
			table {
				td {
					padding-bottom: 3px;
				}
			}
		}

		form {
			> h4.first {
				margin: 0em 0 0.7em 0;
			}
			> h4 {
				clear: both;
				border-bottom: 1px solid $grey;
				padding: 0 0 10px 0;
				font-size: 1em;
				margin: 1.33em 0 0.7em 0;
			}
		}

		.mm_content {
			> h4.first {
				margin: 0em 0 0.7em 0;
			}
			> h4 {
				clear: both;
				border-bottom: 1px solid $grey;
				padding: 0 0 10px 0;
				font-size: 1em;
				margin: 1.33em 0 0.7em 0;
			}
		}

		img {
			height: auto;
			max-width: 100%;
		}

		div[id^=rommeled_image] {
			.non-sortable {
				display: block !important;
			}
		}
	}

	#cboxClose {
		text-indent: 0;
		background: none;
		width: auto;
		height: auto;
	    right: 14px;
	    top: 25px;
		color: $dark_grey;

		&:before {
			content: "\f335";
			font: normal 24px/29px 'dashicons';
		}
		&:hover {
			color: #0074a2;
		}
	}

	&.megamenu_enabled {
		.mm_prefix {
			display: block;
			font-size: 0.9em;
			border-left: 3px solid $orange;
			padding-left: 10px;
			margin-top: 3px;
		}
	}

	.mm_prefix {
		display: none;
	}

	.mm_launch {
		margin-left: 10px;
		font-size: 12px;
		cursor: pointer;
		color: white;
		display: inline-block;
		width: auto;
		font-weight: normal;
		height: auto;
		background: $light_blue;
		border-color: #0074a2;
		border-width: 1px;
		padding: 1px 5px;
		border-radius: 3px;
		opacity: 0;
		-webkit-transition: all 0.3s ease;
		-moz-transition: all 0.3s ease;
		-o-transition: all 0.3s ease;
		-ms-transition: all 0.3s ease;
		transition: all 0.3s ease;
		z-index: 1;
		position: relative;

		&:before {
			font-family: dashicons;
			content: "\f111";
			font-size: 10px;
			margin-right: 4px;
			display: inline-block;
			vertical-align: middle;
		}
	}

	.menu-item-bar {
		.menu-item-handle {
			&:hover {
				.mm_launch {
					opacity: 1;
				}
			}
		}
		.dashicons-admin-generic {
			&:before {
				font-size: 1.2em;
				margin-right: 1px;
				height: auto;
			}
		}
	}

	#nav-menu-meta .mega_menu_meta_box {
		.inside {
			margin: 0;
		}
		div.accordion-section-content {
			padding: 0;

			.mega-accordion {
				float: left;
				width: 100%;
			}

			.mega-accordion-content {
				padding: 10px 14px 0 14px;
				border-bottom: 1px solid #ccc;

				&:last-child {
					border-bottom: 0;
				}
			}

			.mega-accordion-title {
				border-bottom: 1px solid #ccc;
				padding: 10px 3px 10px 10px;
				box-sizing: border-box;

				&.mega-last:not(.mega-accordion-open) {
					border-bottom: 0;
				}
				.dashicons {
					margin-right: 4px;
				}
				.dashicons-yes {
					display: none;
				}

				&.mega-location-enabled {
					.dashicons-yes {
						display: inline-block;
						margin-left: 4px;
						color: $green;
					}

					&.mega-has-active-location .dashicons-yes {
						color: #F56E28;
					}
				}
			}

			h4 {
				outline: none;
				border: 0;
				margin: 0;
				padding: 0;
				white-space: nowrap;
				overflow: hidden;
				text-overflow: ellipsis;

				.dashicons {
					margin-right: 0;
				}
			}

			.mega-location-info {
				font-size: 10px;
				margin-left: 28px;
				line-height: 18px;
			}

			table {
				font-size: 0.9em;
				width: 100%;
				margin-bottom: 5px;
				border-collapse: collapse;

				tr {
					height: 29px;
					vertical-align: middle;

					td {
						&:nth-child(2) {
							text-align: right;
							min-width: 150px;
							padding: 2px 0;
						}
					}

					select {
						font-size: 0.9em;
						max-width: 120px;
						margin: 0;

						&.megamenu_effect_mobile {
							max-width: 80px;
						}
					}

					input[type=checkbox] {
						margin: 0;
					}
				}

				&.sticky_settings {
					td {
						&:nth-child(2) {
							text-align: right;
							min-width: 100px;
						}
					}
				}
			}

			.mega-ellipsis {
				margin-right: 5px;
			}
		}

		p.submit {
			float: right;
			text-align: right;
			width: auto;
			margin: 0 0 15px 0;
			padding: 0;
		}

		.spinner {
			float: right;
		}
	}


	.menu-item-handle {
		.item-title {
			margin-right: 0;
		}
	}
}

body[class*='_page_maxmegamenu_'],
body.toplevel_page_maxmegamenu {

	.megamenu_outer_wrap {
		display: inline-block;
		width: 100%;
		padding-right: 20px;
		padding-top: 10px;
		box-sizing: border-box;

		.notice {
			margin: 0 0 20px 0;
			clear: both;
		}
	}
	*:focus {
		box-shadow: 0 0 0;
	}

	.icon_dropdown {

		.select2-choice > .select2-chosen {
		    line-height: 28px;
		    margin-right: 0px;
		}

		.select2-choice .select2-arrow {
		    display: none;
		}

		.select2-choice {
		    padding: 0 5px 0 5px;
		    height: 28px;
		    color: $grey;
		    border: 1px solid #7e8993;
		}

		.select2-chosen i:before {
		    line-height: 28px;
		    font-size: 16px;
		    color: $text_grey;
		}

	}

	.tpx-select2-drop {
	    box-shadow: 0 0 0;
	    min-width: 150px;
	}

	.select2-results li {
	    float: left;

	    &:first-child {
	    	float: none;

	    	i {
		    	display: none;
			}
	    }

		.select2-result-label {
			font-size: 0.9em;
		}
	}

	.icon_dropdown .select2-choice > .select2-chosen i,
	.select2-result-label i {
		margin: 0;
	    display: inline-block;
	    width: 20px;
	    height: 20px;
	    font-size: 20px;
	    line-height: 1;
	    font-family: dashicons;
	    text-decoration: inherit;
	    font-weight: normal;
	    font-style: normal;
	    vertical-align: top;
	    text-align: center;
	    transition: color .1s ease-in 0;
	    -webkit-font-smoothing: antialiased;
	    -moz-osx-font-smoothing: grayscale;
	}

	.megamenu_header_top {
		font-size: 0.9em;
		display: block;
		width: 100%;
		height: 30px;
		line-height: 30px;
		margin-bottom: 10px;
		box-sizing: border-box;

		ul {
			margin: 0;
			float: right;
			position: relative;
		
			li {
				display: inline-block;
				margin: 0 0 0 14px;
				line-height: 30px;

				&.mega-highlight {
					background: #35b1df;
					border-radius: 5px;

					a {
						color: white;
						line-height: 30px;
						padding: 0 10px;
					}
				}
			}
		}
	}

	.megamenu_header {
		background: white;
		padding: 10px 17px;
		border-bottom: 1px solid $mid_grey;
		float: left;
		width: 100%;
		box-sizing: border-box;

		.version {
			color: $dark_grey;
			margin-bottom: 15px;
			font-size: 12px;
		}

		@include tablet {
			text-align: center;
			border-bottom: 0;
		}
	}

	.megamenu_wrap {
		width: 100%;
		float: left;
		display: flex;
		flex-wrap: wrap;
		background: white;

		@include tablet {
			flex-direction: column;
		}
	}


	.megamenu_left {
		flex-basis: 190px;
		flex-grow: 1;

		@include tablet {
			flex-basis: auto;
			border-bottom: 0;
			margin-bottom: 20px;
		}

		ul.mega-page-navigation {
			@include tablet {
				text-align: center;
				margin: 0;
			}

			li {
				@include tablet {
					display: inline-flex;
					margin-bottom: 0;
				}

				a {
					text-decoration: none;
					color: $text_grey;
					width: 100%;
					display: block;
					padding: 5px 0px 5px 17px;
					box-sizing: border-box;
					border-left: 2px solid white;
					outline: 0;

					&:hover,
					&.active {
					    border-left: 2px solid $light_blue;
					    color: #0074a2;
					}

					@include tablet {
						border-left: 0;
					    padding: 1rem;
					    margin: 0;
					    margin-left: 1rem;
					    margin-right: 1rem;

						&.active {
					    	border-left: 0;
						    box-shadow: inset 0 -3px $light_blue;
						    font-weight: 600;
						}

						&:hover {
					    	border-left: 0;
						}
					}
				}
			}
		}
	}

	.megamenu_right {
		flex-basis: 0;
		flex-grow: 999;

		@include tablet {
			flex-basis: auto;
		}
	}

	h3.first {
		margin-top: 0;
	}

	h3 {
		clear: both;
		font-size: 1.1em;
		margin: 17px 0 5px 0;
		text-indent: 1px;
	    border-bottom: 1px solid $grey;
	    padding: 0 0 10px 0;
	}

	h3 span {
		margin-right: 10px;
	}

	a {
		text-decoration: none;
	}

	.duplicate {
		margin-left: 15px;
	}

	.megamenu_submit {
		margin-bottom: 40px;
		padding-top: 35px;

		.mega_left,
		.mega_right {
		    float: left;
		    width: 50%;
		}

		.spinner {
		    float: left;
		}

		p.submit {
			margin: 0;
			padding: 0;
		    float: left;
		}

		.mega_right {
			text-align: right;
		}

		.mega_right a {
			color: $red;
		}
	}





	.megamenu_wrap .megamenu_submit .saved {
	    background: transparent;
	    line-height: 28px;
	    margin: 0 0 0 5px;
	    float: left;
	    border-radius: 0;
	    color: #0074a2;
	    text-transform: uppercase;
	    font-weight: bold;
	}

	.megamenu_wrap .megamenu_submit .saved .dashicons {
	    color: #0074a2;
	    width: 26px;
	    line-height: 28px;
	    height: 28px;
	}

	.megamenu_wrap .success,
	.megamenu_wrap .warning,
	.megamenu_wrap .fail {
	    background: $white;
	    border-left: 4px solid $green;
		border-top: 1px solid $grey;
		border-bottom: 1px solid $grey;
		border-right: 1px solid $grey;
	    box-shadow: 0 1px 1px 0 rgba( 0, 0, 0, 0.1 );
	    padding: 10px;
	    margin: 5px 0 30px;
	    line-height: 22px;
	    clear: both;
	}

	.megamenu_wrap .fail {
		border-left-color: $red;
	}
	.megamenu_wrap .warning {
		border-left-color: #ffb900;
	}

	.mm-picker-container {
		border-radius: 3px;
		border: 1px solid #8c8f94;
		font-size: 0.9em;
		float: left;

		.sp-replacer {
			margin: 0;
			float: left;
			border: 0;
			background: transparent;
			padding: 0;
		}

		.sp-dd {
			display: none;
		}
		.sp-palette-container {
		    width: 40px;
		}

		.sp-replacer .sp-preview {
			border: 0;
			border-right: 1px solid #8c8f94;
			height: 26px;
			width: 26px;
			margin: 0;
		}

		input.mm_colorpicker {
			float: left;
			border: 0;
			height: 23px;
			box-shadow: 0 0 0;
			margin: 0;
		}

		div.chosen-color {
			border-radius: 0 2px 2px 0;
			color: $text_grey;
			font-size: 11px;
			line-height: 26px;
			padding: 0 6px;
			text-align: center;
			float: left;
			cursor: default;
		}
	}

	.block .mm-picker-container {
	    float: right;
	    margin-right: 0;
	}

	.megamenu_right {
		padding: 20px;
		box-sizing: border-box;
		border-left: 1px solid $mid_grey;
		min-height: 400px;

		textarea.mega-export {
			width: 100%;
			height: 150px;
			font-family: monospace;
			margin-bottom: 10px;
			font-size: 0.9em;
		}

		h2.nav-tab-wrapper {
		    margin: 0;

			a.nav-tab {
			    cursor: pointer;

			    @include tablet {
			    	font-size: 12px;
			    }
			}

			a.nav-tab-active {
				background: white;
			    position: relative;
			    z-index: 1;
			    border-bottom: 1px solid white;
			}
		}

		h3.editing_theme {
			margin: 0 0 30px 0;
		    padding: 0;
		    border: 0;
		    float: left;
		    width: 100%;
		}

		.menu_settings {
			
			.mega-tooltip {
				&:before {
					top: -29px;
					margin-left: -5px;
				}

				&:after {
					margin-left: 2px;
				}
			}

			&.menu_settings_general_settings {
				.mega-inner-table {
					width: auto;

					td {
						padding-top: 3px;
						padding-bottom: 3px;
					}
				}
			}

			&.menu_settings_menu_locations {

				.mega-accordion-title {
					h4:before {
						font: 400 14px/1 dashicons;
						content: "\f347";
						speak: none;
						padding: 0;
						margin-right: 10px;
						display: inline-block;
						vertical-align: middle;
						-webkit-font-smoothing: antialiased;
						-moz-osx-font-smoothing: grayscale;
						transition: all 0.4s ease;
					}

					&.mega-accordion-open h4:before {
						transform: rotateZ(-180deg);
					}
				}

				.mega-add-location {
					margin-top: 20px;
				}
				div.mega-tab-content {
					border: 0;
				}
				.nav-tab-wrapper {
					border: 0;

					a.nav-tab,
					a.nav-tab-active {
						font-size: 12px;
						font-weight: normal;
						padding: 3px 5px;
						border: 0;
						margin: 0 15px 0 0;
						background: white;
						color: $text_grey;
						border-radius: 3px;
						background: $light_grey;
						border: 1px solid $grey;
					}

					a.nav-tab-active {
						background: $light_blue;
						border-color: $light_blue;
						color: white;
					}
				}

				table td {
					padding-bottom: 0;
				}

				table.output_options td.mega-value {
					textarea {
						width: 100%;
					}
					label {
						float: none;
					}
				}

				table.sticky_settings tr td:first-child {
				    width: 40%;
				}

				.mega-tab-content table td:first-child {
					width: 40%;
					font-size: 13px;

					.mega-description {
						margin-top: 0;
					}
					.dashicons {
						margin-right: 6px;
						font-size: 14px;
						vertical-align: middle;
					}
				}

				.mega-vartical-align-top {
					vertical-align: top;
				}

				.megamenu_submit {
					margin-bottom: 0;
				}
			}

			&.menu_settings_menu_themes {

				.theme_selector {
					margin-bottom: 20px;
					float: left;
					
					select {
						max-width: 500px;
						font-size: 12px;
					}
				}

				.mega-ellipsis {
					float: left;
					height: 30px;
					margin-left: 6px;

					svg {
						height: 30px;
						border: 1px solid #7e8993;
						border-radius: 3px;
						box-sizing: border-box;
					}

					.mega-export-theme .dashicons,
					.mega-import-theme .dashicons {
						transform: rotate(90deg);
					}
				}

				td.mega-name {
					padding-left: 10px;
					font-size: 14px;
				}

				h5 {
					clear: both;
					font-size: 1.1em;
					margin: 15px 0 0 10px;
					padding: 0;
				}
			}

			&.menu_settings_tools {
				label {
					clear: both;
					width: 100%;
				}
			}

			&.menu_settings_menu_locations {

				.mega-location-disabled-assign-menu .mega-accordion-content {
					padding: 10px 20px;
				}

				.mega-enabled-locations {
					float: left;
					width: 100%;

					&:empty {
						display: none;
					}
				}
				.mega-location {
					font-size: 14px;
					border: 0;
					border: 1px solid $grey;
					margin-bottom: 10px;
					float: left;
    				width: 100%;

					.mega-accordion-title.mega-accordion-open,
					.mega-accordion-title:hover {
						background: $light_grey;
					}

					.mega-accordion-title {
						padding: 8px;
						font-size: 0.9em;

						.dashicons-yes {
							color: transparent;
							display: none;
						}
					}

					&.mega-location-enabled {
						.mega-accordion-title .dashicons-yes {
							color: $green;
							margin-left: 4px;
							display: inline-block;
						}

						&.mega-has-active-location {
							.mega-accordion-title .dashicons-yes {
								color: #F56E28;
							}
						}

					}

					.mega-accordion-content {
						border-top: 1px solid $grey;

						.megamenu-edit-theme {
							opacity: 0.6;
							line-height: 26px;
							margin-left: 5px;

							&:hover {
								opacity: 1;
							}
						}
						.megamenu_submit {
							background: $light_grey;
							padding: 20px 10px;
							margin-top: 20px;
							box-sizing: border-box;
						}

						.mega-accordion-content-wrapper {
							padding: 10px 15px;
							float: left;
							width: 100%;
							box-sizing: border-box;
						}
					}
				}

				.warning {
					margin: 10px;
				}

				.mega-tab-content {
					.warning {
					    margin: 20px 0 0 0;
				    }
				}
				table .mega-value textarea {
					height: 40px;
					width: 500px;
					padding: 10px;
				}

				> table td:first-child {
					width: 25%;
				}

				h5 {
					font-size: 12px;
				}

				.accordion-section-content {
					width: 100%;
					box-sizing: border-box;
					padding: 0;

					.accordion-wrapper {
						padding: 10px 20px;
					}

					.megamenu_submit {
						border-top: 1px solid #ddd;
						box-sizing: border-box;
						padding: 20px;
						background: $light_grey;
						margin-top: 20px;
					}
				}
			}

			table {
				width: 100%;
				border-collapse: collapse;

				label {
				    font-size: 0.9em;
				    float: left;
				    margin-right: 10px;
				    margin-bottom: 10px;
				}

				.mega-radio-row {
					margin-bottom: 3px;

					label {
						float: none;
						top: -3px;
	    				position: relative;
	    				font-size: 1em;
					}
				}

				textarea,
				input[type=text],
				input[type=number],
				select {
					font-size: 11px;
					min-height: 28px;
				}

				select {
					max-width: 200px;
				}
				
				textarea {
					width: 100%;
					height: 150px;
					font-family: monospace;
					line-height: 1.5em;
					margin-bottom: 10px;
				}

				input[type=number] {
					height: auto;
					line-height: 1.2em;
				}

				input[name=export] {
					margin-top: 20px;
				}

				input[type=text] {
					width: 70px;
				}

				input[type=text].wide {
					width: 300px;
				}

				input[type=submit] {
					width: auto;
				}

				input[type=checkbox] {
					width: 16px;
					height: 16px;
					vertical-align: middle;
				}

				input.mega-setting-panel_width,
				input.mega-setting-panel_inner_width {
					width: 120px;
				}
				input.mega-setting-title {
					width: 100px;
				}

				.wide input[type=text] {
					width: 300px;
				}
				
				.CodeMirror {
					height: auto;
				    min-height: 150px;
					border: 1px solid #aaa;
				}

				div.CodeMirror span.CodeMirror-matchingbracket {
					color: green;
				}

				.CodeMirror-scroll {
					min-height: 150px;
				}

				tr:first-child td {
					border-top: 0;
					padding-top: 20px;
				}

				th {
					text-align: left;
				}

				td {
					position: relative;
					padding-bottom: 25px;
					padding-top: 15px;
					border-top: 1px solid $grey; 

					&.mega-name {
						width: 33%;
						line-height: 2em;
						padding-right: 30px;
						vertical-align: top;
					}

					&.mega-value {
						padding-top: 10px;

						.mega-description {
							margin-top: 5px;
						}

						.mega-info {
							clear: both;
							margin-top: 15px;
							display: block;
							float: left;
							font-size: 0.9em;
							color: #777;
							line-height: 20px;
							width: 100%;

							&:before {
								font: 400 20px/1 dashicons;
								content: "\f348";
								speak: none;
								padding: 0;
								margin-right: 6px;
								display: inline-block;
								vertical-align: top;
								-webkit-font-smoothing: antialiased;
								-moz-osx-font-smoothing: grayscale;
							}
						}
					}

					&.mega-custom_css {
						width: 100%;
						margin-bottom: 0;
					}

				}	

				td.mega-value.mega-instances {
				    vertical-align: top;

					table th,
					table td {
					    font-weight: normal;
					    padding: 5px;
					}

					.mega-description {
					    padding-left: 5px;
					}

				}

				td.mega-name .mega-description,
				td.mega-value .mega-description {
					font-size: 12px;
					color: #777;
					line-height: 1.5em;

					p {
						font-size: 12px;
					}
				}

				ul.custom_styling_tips {
					list-style-type: disc;
					list-style-position: inside;

					code {
						margin: 10px 0 0 0;
						font-size: 0.9em;
					}
				}

				label.mega-error input {
					border: 1px solid $red;
					background: #FFDDDD;
				}

				label.mega-toggle_blocks {
					width: 99%;
					cursor: auto;

					.mega-value {
					    padding-bottom: 0;
					}

					.mega-description {
					    float: left;
					    width: 100%;
					    margin-top: 20px;
					}

	 				.toggle-bar-designer {

	 					.mega-blocks {
	 						display: flex;
	 						box-sizing: border-box;
	 					}

						.mega-blocks .mega-left,
						.mega-blocks .mega-center,
						.mega-blocks .mega-right {
						    box-sizing: border-box;
						    border: 1px solid #ccc;
						    background: #f7f7f7;
						    height: 60px;
						    padding: 10px;
						    margin-top: 15px;
						    display: flex;
						    flex-basis: 33.3333%;
						}
						.mega-blocks .mega-left {
							border-right: 0px;
							flex: 1;
						}
						.mega-blocks .mega-center {
						    border-left: 1px dashed #ccc;
						    border-right: 1px dashed #ccc;
						    justify-content: center;
						}
						.mega-blocks .mega-right {
						    border-left: 0px;
						    flex: 1;
						    justify-content: flex-end;
						}
						.mega-blocks .mega-right .block {
						    float: right;
						    margin: 0 0 0 5px;
						}
						.mega-blocks .block.mega-open .block-title {
						    border-bottom: 1px solid white;
						    border-radius: 3px 3px 0 0;
						}
						.mega-blocks .block .block-title {
						    padding: 0 10px;
						    display: inline-block;
						    height: 35px;
						    line-height: 35px;
						    font-size: 0.9em;
						    background: white;
						    color: $text_grey;
						    cursor: move;
						    border: 1px solid #ccc;
						    position: relative;
						    z-index: 2;
						    border-radius: 3px;
						    text-align: left;
						    box-sizing: border-box;
						    white-space: nowrap;
						}

						.mega-blocks .block.ui-sortable-helper {
						    width: auto !important;
						}

						.mega-blocks .block .block-title .dashicons {
						    line-height: 35px;
						    cursor: pointer;
						    font-size: 20px;
						}

						.mega-blocks .block .block-settings {
						    padding: 10px;
						    display: none;
						    width: 300px;
						    vertical-align: top;
						    background: white;
						    text-align: left;
						    color: #ccc;
						    position: absolute;
						    left: 0px;
						    top: 34px;
						    z-index: 1;
						    border: 1px solid #ccc;
						}

						.mega-blocks .mega-right .block .block-settings {
						    left: auto;
						    right: 0;
						}

						.mega-blocks .block h3 {
						    margin-top: 5px;
						    border-bottom: 1px solid #ccc;
						    padding-bottom: 10px;
						}

						.block-settings label {
						    width: 100%;
						    clear: both;
						    color: $text_grey;
						    font-size: 1em;
						}

						.block-settings label input,
						.block-settings label select,
						.block-settings label input[type=number],
						.block-settings label .icon_dropdown {
							float: right;
							font-size: 0.9em;
						}

						.block-settings label input[type=number] {
							height: 28px;
							line-height: 28px;
						}

						.toggle-left {
							border-right: 1px dashed #ccc;
						}

						.toggle-right {
						    border-left: 0px solid #ccc;
						    text-align: right;

						}

						.mega-blocks .block {
						    position: relative;
						    display: inline-block;
						    vertical-align: top;
						    margin: 0 5px 0 0;

						    input[type=text] {
							    width: 50%;
							}
						}
	 				}
				}


				.mega-validation-message {
				    display: none;
				    clear: both;
				    font-size: 0.8em;
				    color: $red;
				}

				label span.mega-short-desc {
					font-size: 0.8em;
					text-transform: uppercase;
					color: #777;
					display: block;
					margin: 0 0 2px 1px;
				}

				label.mega-copy_color {
				    margin-right: 3px;

				    span.mega-short-desc {
					    visibility: hidden;
					}

					span.dashicons {
					    opacity: 0.5;
					    margin-top: 6px;
					    font-size: 16px;
					}
				}
			}
			
			.row h5 {
				margin: 0;
				font-weight: normal;
				float: left;
				width: 250px;
				font-size: 1em;
			}

			.megamenu_submit {
			    float: left;
			    width: 100%;

			}

			div.mega-tab-content {
			    position: relative;
			    top: -1px;
			    clear: both;
			    border: 1px solid #ccc;
			    float: left;
			    width: 100%;
			    padding: 0;
			    box-sizing: border-box;

			    &.mega-toggle-disabled {
			    	.mega-toggle_blocks td:after,
			    	.mega-toggle_bar_background td:after,
			    	.mega-toggle_bar_height td:after,
			    	.mega-mobile_menu_overlay td:after,
			    	.mega-mobile_menu_force_width td:after {
			    		background: $mid_grey;
					    display: block;
					    position: absolute;
					    top: 0;
					    left: 0;
					    height: 100%;
					    width: 100%;
					    content: "";
					    opacity: 0.5;
					    z-index: 9;
			    	}

				    .mega-mobile_toggle_disabled {
				    	display: table-row;
				    }
			    }

			    &.mega-mobile-disabled {
			    	tr:not(.mega-responsive_breakpoint):not(.mega-responsive_breakpoint_disabled) td:after {
			    		background: $mid_grey;
					    display: block;
					    position: absolute;
					    top: 0;
					    left: 0;
					    height: 100%;
					    width: 100%;
					    content: "";
					    opacity: 0.5;
					    z-index: 9;
			    	}

				    .mega-responsive_breakpoint_disabled {
				    	display: table-row;
				    }

				    .mega-mobile_toggle_disabled {
				    	display: none;
				    }
			    }


			    .mega-responsive_breakpoint_disabled,
			    .mega-mobile_toggle_disabled {
			  		display: none;

			    	td {
			    		padding-top: 10px;
			    		padding-bottom: 10px;
			    		border-top: 0;
			    	}

			    	h5 {
				    	border: 1px solid #ccc;
					    padding: 10px;
					    box-sizing: border-box;
					    border-left: 3px solid $green;
					    font-size: 12px;
					    font-weight: normal;
					    margin: 0 10px;
			    	}

			    }
			}
		}

		.mega-description .fail {
			margin: 20px 0 0 0;
			color: $dark_grey;
		}

	}

	.mega-delete {
		margin-bottom: 15px;
	    cursor: pointer;
	}

}


.mega-accordion-title {
	transition: all .1s ease-in;
	margin: 0;
	color: $text_grey;
	cursor: pointer;
	float: left;
	width: 100%;
	box-sizing: border-box;
	line-height: 24px;

	h4 {
		display: inline;
	}

	.dashicons {
		font-size: 18px;
	    vertical-align: middle;
	    margin-right: 6px;
	    transition: none;
	}

	&.mega-accordion-open + .mega-accordion-content {
		display: block;
	}
}

.mega-ellipsis {
	float: right;
	position: relative;
	height: 24px;

	svg {
		cursor: pointer;
		fill: black;
		height: 24px;
		width: 24px;
	}

	&.mega-ellipsis-horizontal svg {
		transform: rotate(90deg);
	}

	&.mega-ellipsis-open {
		ul.mega-ellipsis-content {
			display: block;
		}
	}

	&.mega-ellipsis-left {
		ul.mega-ellipsis-content {
			left: -177px;
			top: -5px;

		    &:after {
			    top: 8px;
			    right: -14px;
			    transform: rotate(90deg);
		    }

		    &:before {
				top: 7px;
				right: -16px;
			    transform: rotate(90deg);
		    }

			li {
		    	a {
		    		line-height: 30px;
		    	}
		    }
		}
	}

	ul.mega-ellipsis-content {
		display: none;
		position: absolute;
	    width: 170px;
	    background: white;
	    left: -141px;
	    top: 33px;
	    z-index: 9;
	    border: 1px solid $dark_grey;
	    margin: 0;
	    box-shadow: 0 8px 24px rgba(149,157,165,.2);
	    border-radius: 3px;

	    &:after {
		    top: -14px;
		    right: 10px;
		    left: auto;
		    border: 7px solid transparent;
		    border-bottom-color: #fff;
		    position: absolute;
		    display: inline-block;
		    content: "";
	    }

	    &:before {
		    top: -16px;
		    right: 9px;
		    left: auto;
		    border: 8px solid transparent;
		    border-bottom-color: $dark_grey;
		    position: absolute;
		    display: inline-block;
		    content: "";
	    }
	    li {
	    	font-size: 12px;
	    	margin-bottom: 0;

	    	a {
	    		display: block;
	    		text-decoration: none;
	    		line-height: 40px;
	    		padding-left: 10px;
	    		color: $text_grey;
	    		transition: all 0.2s;

	    		&:hover {
	    			background: $light_grey;
	    			color: $light_blue;
	    		}

				.dashicons {
				    font-size: 18px;
				    vertical-align: middle;
				    margin-right: 6px;
				    transition: none;
				}
	    	}
	    }
	}
}

.mega-accordion-content {
	display: none;
	float: left;
	width: 100%;
	box-sizing: border-box;
}

.mega-warning {
	padding: 10px 0;

	.dashicons {
		margin-right: 6px;
	}
}

@keyframes "rotation" {
	0% {
		-webkit-transform: rotate(0deg);
		transform: rotate(0deg);
	}
	100% {
		-webkit-transform: rotate(359deg);
		transform: rotate(359deg);
	}
}
@keyframes busy-animation {
    0% {
        background-position: 200px 0;
    }
}
