<?php
if(!defined('ABSPATH')) exit();
?>
	<rs-module-wrap id="rev_slider_737_1_wrapper" data-alias="rvsldr-object-library" data-source="gallery" style="background:#2a2c2f;padding:0px;margin:0px auto;margin-top:0px;margin-bottom:0px;">
							<rs-module id="rev_slider_737_1" class="feature_slider" style="display:none;" data-version="6.0.0">
								<rs-slides>
									<rs-slide data-key="rs-2451" data-title="Slide" data-anim="ei:d;eo:d;s:300;r:0;t:fade;sl:d;">
										<img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png"  data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/transparent.png" data-bg="p:center center;c:#2a2c2f;" class="rev-slidebg" data-no-retina>
						<!--
										--><rs-layer
											id="slide-2451-layer-18" 
											data-type="text"
											data-color="#b7bbc0"
											data-rsp_ch="on"
											data-xy="x:98px;y:37px;"
											data-text="s:30;l:30;a:center;"
											data-dim="w:30px;h:30px;"
											data-border="bow:0px;"
											data-frame_0="x:-200px;"
											data-frame_1="e:power4.out;st:400;sp:2000;sR:400;"
											data-frame_999="y:-200px;st:a;sp:700;sR:6600;"
											data-loop_0="yR:4px;y:-4;oX:50;oY:50;"
											data-loop_999="xR:4px;yR:4px;y:4;sp:10000;yym:t;yys:t;yyf:t;"
											style="z-index:5;font-family:Open Sans;"
										><i class="fa-diamond"></i> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-20" 
											data-type="text"
											data-color="#b7bbc0"
											data-rsp_ch="on"
											data-xy="x:31px;y:109px;"
											data-text="s:30;l:30;a:center;"
											data-dim="w:30px;h:30px;"
											data-border="bow:0px;"
											data-frame_0="x:-200px;"
											data-frame_1="e:power4.out;st:500;sp:2000;sR:500;"
											data-frame_999="y:-200px;st:a;sp:700;sR:6500;"
											data-loop_0="yR:1px;y:-2;oX:50;oY:50;"
											data-loop_999="xR:1px;yR:1px;y:2;sp:10000;yym:t;yys:t;yyf:t;"
											style="z-index:6;font-family:Open Sans;"
										><i class="fa-comment-o"></i> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-1" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:c;xo:-119px;y:b;yo:-39px;"
											data-text="l:22;a:inherit;"
											data-dim="w:200px;h:200px;"
											data-border="bow:0px;"
											data-frame_0="x:left;o:1;"
											data-frame_1="e:power4.out;st:160;sp:2000;sR:160;"
											data-frame_999="st:a;sp:1000;sR:6840;auto:true"
											style="z-index:7;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="200px" data-hh="200px" width="200" height="200" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/object_library_headphones.png" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-15" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:c;xo:144px;y:b;yo:-38px;"
											data-text="l:22;a:inherit;"
											data-dim="w:200px;h:230px;"
											data-border="bow:0px;"
											data-frame_0="x:right;o:1;"
											data-frame_1="e:power4.out;st:160;sp:2000;sR:160;"
											data-frame_999="st:a;sp:1000;sR:6840;;auto:true"
											style="z-index:8;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="200px" data-hh="230px" width="200" height="230" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/object_library_phone.png" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-17" 
											data-type="text"
											data-color="#b7bbc0"
											data-rsp_ch="on"
											data-xy="x:49px;y:20px;"
											data-text="s:30;l:30;a:center;"
											data-dim="w:30px;h:30px;"
											data-border="bow:0px;"
											data-frame_0="x:-200px;"
											data-frame_1="e:power4.out;st:300;sp:2000;sR:300;"
											data-frame_999="y:-200px;st:a;sp:700;sR:6700;"
											data-loop_0="yR:3px;y:-2;oX:50;oY:50;"
											data-loop_999="xR:3px;yR:3px;y:2;sp:10000;yym:t;yys:t;yyf:t;"
											style="z-index:9;font-family:Open Sans;"
										><i class="fa-flash"></i> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-19" 
											data-type="text"
											data-color="#b7bbc0"
											data-rsp_ch="on"
											data-xy="x:19px;y:59px;"
											data-text="s:30;l:30;a:center;"
											data-dim="w:30px;h:30px;"
											data-border="bow:0px;"
											data-frame_0="x:-200px;"
											data-frame_1="e:power4.out;st:200;sp:2000;sR:200;"
											data-frame_999="y:-200px;st:a;sp:700;sR:6800;"
											data-loop_0="yR:2px;y:-3;oX:50;oY:50;"
											data-loop_999="xR:2px;yR:2px;y:3;sp:10000;yym:t;yys:t;yyf:t;"
											style="z-index:10;font-family:Open Sans;"
										><i class="fa-cloud-download"></i> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-11" 
											data-type="image"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="l:22;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-border="bow:0px;"
											data-frame_0="y:bottom;sX:2;sY:2;"
											data-frame_1="st:a;sp:1000;sR:160;"
											data-frame_999="auto:true;st:a;sp:700;sR:7840;"
											style="z-index:11;"
										><img src="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/dummy.png" data-ww="full-proportional" data-hh="full-proportional" width="640" height="427" data-lazyload="<?php echo RS_PLUGIN_URL;?>admin/views/features/assets/object_library_bg-1.jpg" data-no-retina> 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-2" 
											data-type="text"
											data-color="#ffffff"
											data-rsp_ch="on"
											data-xy="x:c;y:c;yo:-20px;"
											data-text="s:25;l:30;fw:700;a:center;"
											data-border="bow:0px;"
											data-frame_0="y:-50;"
											data-frame_1="e:power4.out;st:160;sp:1000;sR:160;"
											data-frame_999="y:50;o:0;e:power4.out;st:a;sp:600;sR:7840;"
											style="z-index:12;font-family:Roboto;"
										>2500+<br/>
						Images, Videos & Icons 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-12" 
											data-type="text"
											data-color="#ffffff"
											data-rsp_ch="on"
											data-xy="x:c;y:c;yo:-20px;"
											data-text="s:25;l:30;fw:700;a:center;"
											data-border="bow:0px;"
											data-frame_0="y:-50;"
											data-frame_1="e:power4.out;st:a;sp:1000;sR:100;"
											data-frame_999="y:50px;o:0;e:power4.out;st:a;sp:600;sR:7900;"
											style="z-index:13;font-family:Roboto;"
										>Powerful GFX<br/>
						At Your Fingertips 
										</rs-layer><!--

										--><rs-layer
											id="slide-2451-layer-4" 
											class="rev-btn"
											data-type="button"
											data-color="rgba(255,255,255,1)"
											data-xy="x:c;y:c;yo:40px;"
											data-text="s:15;l:34;fw:500;a:inherit;"
											data-rsp_bd="off"
											data-padding="r:35;l:35;"
											data-border="bos:solid;boc:rgba(0,0,0,1);bow:0px;bor:3px,3px,3px,3px;"
											data-frame_0="y:50;"
											data-frame_1="e:power4.out;st:500;sp:1000;sR:500;"
											data-frame_999="o:0;e:power4.inOut;st:w;sp:1000;sR:7500;"
											data-frame_hover="c:rgba(255,255,255,1);bgc:#0a447a;boc:rgba(0,0,0,1);bor:3px,3px,3px,3px;bos:solid;oX:50;oY:50;sp:300;"
											style="z-index:14;background-color:#006dd2;font-family:Roboto;"
										>Object Library 
										</rs-layer><!--

										--><a
											href="//revolution.themepunch.com/direct-customer-benefits/"
											target="_blank"
											class="tp-shape tp-shapewrapper rs-layer"
											data-type="shape"
											data-rsp_ch="on"
											data-xy="x:c;y:c;"
											data-text="fw:700;a:inherit;"
											data-dim="w:100%;h:100%;"
											data-actions='o:mouseenter;a:stoplayer;layer:slide-2451-layer-2;||o:mouseenter;a:stoplayer;layer:slide-2451-layer-1;||o:mouseenter;a:stoplayer;layer:slide-2451-layer-15;||o:mouseenter;a:stoplayer;layer:slide-2451-layer-17;d:150;||o:mouseenter;a:stoplayer;layer:slide-2451-layer-18;d:200;||o:mouseenter;a:stoplayer;layer:slide-2451-layer-19;d:50;||o:mouseenter;a:stoplayer;layer:slide-2451-layer-20;d:100;||o:mouseenter;a:startlayer;layer:slide-2451-layer-12;||o:mouseenter;a:startlayer;layer:slide-2451-layer-11;||o:mouseleave;a:stoplayer;layer:slide-2451-layer-12;||o:mouseleave;a:stoplayer;layer:slide-2451-layer-11;||o:mouseleave;a:startlayer;layer:slide-2451-layer-2;||o:mouseleave;a:startlayer;layer:slide-2451-layer-1;||o:mouseleave;a:startlayer;layer:slide-2451-layer-15;||o:mouseleave;a:startlayer;layer:slide-2451-layer-17;d:100;||o:mouseleave;a:startlayer;layer:slide-2451-layer-18;d:50;||o:mouseleave;a:startlayer;layer:slide-2451-layer-19;d:200;||o:mouseleave;a:startlayer;layer:slide-2451-layer-20;d:150;'
											data-basealign="slide"
											data-border="bos:solid;boc:rgba(0,0,0,0.5);bow:0px;"
											data-frame_1="st:160;sR:160;"
											data-frame_999="o:0;st:w;sR:8540;"
											style="z-index:15;"
											id="2451a"
										> 
										</a><!--
									--></rs-slide>
								</rs-slides>								
							</rs-module>
						</rs-module-wrap>
