(window.webpackWcBlocksJsonp=window.webpackWcBlocksJsonp||[]).push([[23],{398:function(e,c,t){"use strict";t.r(c);var n=t(32),o=t(0),r=t(137);c.default=function(e){var c=e.children,t=Object(n.a)(),a=t.cartItems,l=t.cartIsLoading;return Object(o.useEffect)((function(){Object(r.a)("wc-blocks_render_blocks_frontend",{element:document.body.querySelector(".wp-block-woocommerce-cart")})}),[]),l||0!==a.length?null:React.createElement(React.Fragment,null,c)}}}]);