.yith-plugin-fw span.description {
    display    : block;
    margin-top : 5px;
    font-size  : 12px;
    color      : #777;
    font-style : italic;
    clear      : both;
}

.yith-plugin-fw span.description.inline {
    display   : inline-block;
    margin    : 0 0 0 5px;
    min-width : 0;
    max-width : 100%;
}

.yith-disabled {
    opacity        : 0.3;
    pointer-events : none;
}

.yith-plugin-fw-field-wrapper:not(.yith-plugin-fw-title-field-wrapper) {
    display : block;
    width   : 100%;
}

.yith-plugin-fw-field-wrapper .yith-inline-fields > div {
    display        : inline-block;
    vertical-align : middle;
    margin-right   : 5px;
}

.yith-plugin-fw-field-wrapper span.desc {
    display : block;
    width   : 100%;
}

.yith-plugin-fw-title-field-wrapper h3.title {
    width   : 100%;
    padding : 0 !important;
}

.the-metabox .yith-plugin-fw-title-field-wrapper h3.title {
    float : left;
}

.yith-plugin-fw span.color-alpha {
    height : 100% !important;
    width  : 100% !important;
    border : 1px solid #d9d9d9;
    margin : -1px;
}

.yith-plugin-fw .wp-color-result-text {
    height : 100%;
}

.yith-plugin-fw input[type=email],
.yith-plugin-fw input[type=number],
.yith-plugin-fw input[type=text]:not(.select2-search__field),
.yith-plugin-fw select,
.woocommerce .yith-plugin-fw table.form-table input[type=email],
.woocommerce .yith-plugin-fw table.form-table input[type=number],
.yith-plugin-ui input[type=password],
.woocommerce .yith-plugin-fw table.form-table input[type=text]:not(.select2-search__field),
.woocommerce .yith-plugin-fw table.form-table select {
    width       : 400px;
    max-width   : 100%;
    height      : 30px;
    padding     : 5px;
    line-height : 1;
}

.yith-plugin-fw input[type=number],
.woocommerce .yith-plugin-fw table.form-table input[type=number] {
    width     : 70px;
    min-width : 70px;
}

.woocommerce .yith-plugin-fw table.form-table input[type=text].wp-color-picker,
.woocommerce .yith-plugin-fw table.form-table input[type=text].yith-plugin-fw-colorpicker,
.yith-plugin-fw input[type=text].wp-color-picker,
.yith-plugin-fw input[type=text].yith-plugin-fw-colorpicker {
    width     : 6em;
    max-width : none;
    height    : auto;
    padding   : 3px 5px;
}

.wp-core-ui .yith-plugin-fw-colorpicker-field-wrapper button {
    border-color : #7e8993;
}

.woocommerce .yith-plugin-fw table.form-table input[type=text].colorpick,
.yith-plugin-fw input[type=text].colorpick {
    width     : 6em;
    max-width : none;
    height    : auto;
    padding   : 6px;
}

.yith-plugin-fw input.select2-search__field,
.yith-plugin-fw .select2-container .select2-search--inline .select2-search__field,
.woocommerce .yith-plugin-fw input.select2-search__field,
.woocommerce .yith-plugin-fw table.form-table input.select2-search__field {
    padding   : 3px;
    min-width : 380px;
}

.yith-plugin-fw .select2-container .select2-selection--single .select2-selection__rendered {
    line-height : 27px;
}

.yith-plugin-fw textarea:not(.wp-editor-area) {
    width     : 400px;
    max-width : 100%;
    padding   : 5px;
}

/* ------------- ICONS ------------- */
.yit-icons-manager-wrapper {
    width     : 100%;
    max-width : 400px;
}

.yit-icons-manager-text {
    width : 100%;
}

.yit-icons-manager-icon-text {
    width  : calc(100% - 35px) !important;
    height : 30px;
    float  : left;
    margin : 0;
}

.yit-icons-manager-icon-preview {
    width        : 30px;
    height       : 30px;
    margin-right : 5px;
    float        : left;
    padding-top  : 6px;
    box-sizing   : border-box;
    text-align   : center;
    background   : #f1f1f1;
}

.yit-icons-manager-list-wrapper {
    margin     : 10px 0;
    box-shadow : 0 0 0 1px #ddd;
}

ul.yit-icons-manager-list {
    width      : 100%;
    height     : 300px;
    overflow-y : scroll;
    margin     : 0;
}

ul.yit-icons-manager-list li {
    float       : left;
    width       : 30px;
    height      : 30px;
    padding-top : 6px;
    box-sizing  : border-box;
    text-align  : center;
    background  : #f1f1f1;
    margin      : 1px;
    cursor      : pointer;
}

ul.yit-icons-manager-list li:hover {
    background : #c5dcf6;
}

ul.yit-icons-manager-list li.active {
    background : #d4f0ff;
}

.yit-icons-manager-action-set-default {
    margin-bottom : 0;
}

.yit-icons-manager-default-icon-preview {
    height       : 100%;
    display      : inline-block;
    padding-left : 10px;
    border-left  : 1px solid #ccc;
    margin-left  : 10px;
}

/* ------- Text Array ------- */
.yith-plugin-fw-text-array-table td {
    padding : 1px;
}

.yith-plugin-fw-text-array-table input[type=text],
.woocommerce table.form-table table.yith-plugin-fw-text-array-table input[type=text] {
    width : 100%;
}

/* ------- Image Gallery ------- */
.yith-plugin-fw .yith-plugin-fw-image-gallery ul li {
    display     : inline-block;
    width       : 80px;
    margin-left : 10px;
    position    : relative;
}

.yith-plugin-fw .yith-plugin-fw-image-gallery ul li img {
    width  : 80px;
    border : 1px solid #ccc;
}

.yith-plugin-fw .yith-plugin-fw-image-gallery ul li ul {
    position : absolute;
    top      : -6px;
    right    : -1px;
    width    : 20px;
    height   : 20px;
}

.yith-plugin-fw .yith-plugin-fw-image-gallery ul a.delete {
    background  : url(../images/x.png) no-repeat;
    width       : 20px;
    height      : 20px;
    display     : block;
    text-indent : -99999px;
}

/* ------- OnOff ------- */
.yith-plugin-fw-onoff-container {
    display    : inline-block;
    text-align : left;
}

.yith-plugin-fw-onoff-container input {
    display : none;
}

.yith-plugin-fw-onoff-container input + span {
    cursor        : pointer;
    text-indent   : -9999px;
    display       : block;
    width         : 36px;
    line-height   : 1;
    height        : 20px;
    background    : #a4a4a4;
    border-radius : 24px;
    position      : relative;
    transition    : all 0.3s;
}


.yith-plugin-fw-onoff-container input + span:before {
    content       : '';
    background    : #fff;
    width         : 16px;
    height        : 16px;
    border-radius : 50%;
    position      : absolute;
    top           : 2px;
    left          : 2px;
    transition    : all 0.3s;
}

.yith-plugin-fw-onoff-container input:checked + span,
.yith-plugin-fw-onoff-container input.onoffchecked + span {
    background : #0073aa;
}

.yith-plugin-fw-onoff-container input:checked + span:before,
.yith-plugin-fw-onoff-container input.onoffchecked + span:before {
    left : 18px;
}

/* ------- Preview ------- */
.yith-plugin-fw-preview-field {
    max-height : 200px;
}

/* ------- Radio ------- */
.yith-plugin-fw-radio input[type=radio] {
    margin : 0 3px 0 0;
}

.yith-plugin-fw-radio__row {
    margin-bottom : 8px;
}

/* ------- Checkbox Array ------- */
.yith-plugin-fw-checkbox-array input[type=checkbox] {
    margin : 2px 3px 0 0;
}

.yith-plugin-fw-checkbox-array__row {
    margin-bottom : 10px;
    display       : flex;
}

.yith-plugin-fw-checkbox-array__row label {
    display     : inline-block !important;
    width       : auto !important;
    float       : none !important;
    margin      : 0 0 0 10px !important;
    padding     : 0 !important;
    font-weight : 400 !important;
}

.yith-plugin-fw-checkbox-array__row label small {
    display     : block;
    line-height : 2em;
}

/* ------- Sidebar Layout ------- */
.yith-plugin-fw-sidebar-layout input[type="radio"] {
    display : none;
    width   : 0px;
}

.yith-plugin-fw-sidebar-layout input[type="radio"]:first-child {
    margin-right : -2px;
}

.yith-plugin-fw-sidebar-layout input[type="radio"] + img {
    border  : 2px solid #fff;
    padding : 1px;
}

.yith-plugin-fw-sidebar-layout input[type="radio"] + img:hover {
    cursor : pointer;
}

.yith-plugin-fw-sidebar-layout input[checked] + img {
    border  : 2px solid #f2ad35;
    padding : 1px;
}

.yith-plugin-fw-sidebar-layout select {
    vertical-align : 12px;
}

.yit-admin-panel-content-wrap .yith-plugin-fw-sidebar-layout label {
    font-weight : bold;
    width       : 200px;
    float       : left;
    line-height : 23px;
    margin-left : -230px;
}

/* ------- Slider ------- */
.yith-plugin-fw .slider {
    padding-top : 20px;
}

.yith-plugin-fw-slider-container .ui-slider .minCaption {
    position     : absolute;
    right        : 95%;
    top          : -6px;
    margin-right : 11px;
}

.yith-plugin-fw-slider-container .ui-slider .maxCaption {
    position    : absolute;
    left        : 95%;
    top         : -6px;
    margin-left : 20px;
}

.yith-plugin-fw-slider-container .ui-slider-horizontal .ui-slider-handle {
    background    : #fff;
    border        : 0px !important;
    top           : -12px !important;
    border-radius : 50%;
    width         : 27px !important;
    height        : 27px !important;
    box-sizing    : border-box;
    box-shadow    : 0 1px 7px -1px rgba(0, 0, 0, 0.5);
    font-size     : 13px;
    padding       : 6px 0;
    font-weight   : 600;
    color         : #555;
    text-align    : center;
}

.yith-plugin-fw-slider-container .ui-slider.ui-widget-content {
    background    : #ccc;
    border        : none !important;
    height        : 3px !important;
    border-radius : 3px !important;
    width         : 90%;
    margin        : 15px 5% 20px 5%;
}

.yith-plugin-fw-slider-container .ui-slider.ui-widget-content .ui-widget-header {
    background            : #4b93ff;
    left                  : 0px !important;
    -webkit-border-radius : 3px;
    -moz-border-radius    : 3px;
    -khtml-border-radius  : 3px;
    border-radius         : 3px;
}

/* ------- Select2 ------- */
.yith-plugin-fw-select2-wrapper {
    width : 400px;
}

/* ------- Textarea ------- */
.yith-plugin-fw-textarea-editor-field-wrapper {
    max-width : 1000px;
    clear     : both;
}

.yith-plugin-fw-textarea-editor-field-wrapper textarea.wp-editor-area,
.woocommerce table.form-table .yith-plugin-fw-textarea-editor-field-wrapper textarea.wp-editor-area {
    width : 100%;
}

/* ------- Buttons ------- */
.yith-plugin-fw-buttons-field-wrapper {
    margin-top : 7px;
}

/* ------- Select Images ------- */
.yith-plugin-fw-select-images__list {
    margin    : 0;
    display   : flex;
    flex-wrap : wrap;
}

.yith-plugin-fw-select-images__item {
    width           : calc(20% - 10px);
    min-width       : 150px;
    margin          : 0 10px 10px 0;
    padding         : 10px;
    box-sizing      : border-box;
    cursor          : pointer;
    transition      : all .3s;
    border          : 2px solid #eee;
    display         : flex;
    flex-direction  : column;
    justify-content : space-between;
}

.yith-plugin-fw-select-images__item:hover,
.yith-plugin-fw-select-images__item.yith-plugin-fw-select-images__item--selected {
    border-color : #07bcce;
}

.yith-plugin-fw-select-images__item__label {
    text-align    : center;
    font-weight   : 600;
    margin-bottom : 10px;
}

.yith-plugin-fw-select-images__item img {
    display   : block;
    max-width : 100%;
    margin    : 0 auto;
}

/****************
WordPress 5.3 with old panel fixes
 ****************/
.yith-plugin-fw-field-wrapper .select2-container .select2-selection--single {
    height      : auto;
    line-height : 1;
}

.yith-plugin-fw-field-wrapper .select2-container .select2-selection--single .select2-selection__arrow {
    height : 100%;
}

/****************
YITH UI
 ****************/
.yith-plugin-fw-banner {
    width         : 100%;
    height        : 45px;
    background    : #cfd9dd url(../images/banner-premium.png) no-repeat;
    display       : flex;
    align-items   : center;
    padding-right : -23px;
}

.yith-plugin-fw-banner h1 {
    text-transform : uppercase;
    color          : #0c5777;
    font-size      : 15px;
    padding        : 0 0 0 110px;
    font-weight    : 700;
    text-align     : left;
    display        : inline-block;
    box-sizing     : border-box;
}

.yith-plugin-fw-banner span {
    font-style     : italic;
    display        : block;
    font-size      : 15px;
    color          : #214249;
    text-transform : none;
    text-align     : right;
}

.yith-plugin-fw-banner a,
.yith-plugin-fw-banner a:focus {
    text-decoration : none;
    outline         : none;
    box-shadow      : none;
}

.yith-plugin-fw-rate {
    margin : 20px 0;
}

.yith-plugin-fw-rate .dashicons-star-filled {
    font-size : 12px;
    margin    : 4px -4px;
    color     : #0c5777;
}

.yith-plugin-fw-rate a {
    text-decoration : none;
}

.yith-plugin-ui a {
    outline    : none;
    box-shadow : none;
}

.yith-plugin-ui {
    font-family : -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
}

.yith-plugin-ui #wpwrap h2.nav-tab-wrapper, .yith-plugin-ui #wpbody-content h2.nav-tab-wrapper {
    margin-top : 10px;
}

.yith-plugin-ui .nav-tab {
    background-color : #336374;
    color            : #fff;
    font-size        : 13px;
    padding          : 7px 15px;
    border           : 0;
    margin           : 0 4px 0 0;
    transition       : background-color 0.3s linear;
    float            : none;
    display          : inline-block;
}

.yith-plugin-ui .nav-tab.yith-premium {
    background-color : #be421c;
}

.yith-plugin-ui .yith-nav-tab-wrapper .yith-plugin-fw-tab-element:hover > a.nav-tab:not(.nav-tab-active),
.yith-plugin-ui .yith-nav-tab-wrapper .nav-tab:not(.nav-tab-active):hover {
    background-color : #207a92;
}

.yith-plugin-ui .yith-nav-tab-wrapper .yith-plugin-fw-tab-element:hover > a.nav-tab.yith-premium:not(.nav-tab-active),
.yith-plugin-ui .yith-nav-tab-wrapper .nav-tab.yith-premium:not(.nav-tab-active):hover {
    background-color : #d24318;
}

.yith-plugin-ui .nav-tab-active,
.yith-plugin-ui .nav-tab-active:hover,
.yith-plugin-ui .nav-tab-active:focus,
.yith-plugin-ui .nav-tab-active:focus:active {
    background-color : #fff;
    color            : #336374;
    box-shadow       : 0 3px 0 -1px #fff;
    border-bottom    : 0;
}

.yith-plugin-ui .nav-tab-active.yith-premium,
.yith-plugin-ui .nav-tab-active.yith-premium:hover,
.yith-plugin-ui .nav-tab-active.yith-premium:focus,
.yith-plugin-ui .nav-tab-active.yith-premium:focus:active {
    background-color : #fff;
    color            : #be421c;
    box-shadow       : none;
}

.yith-plugin-ui .yith-plugin-fw-tab-element:first-child .nav-tab-active {
    border-left : 1px solid #ddd;
}

.yith-plugin-ui #plugin-fw-wc {
    padding-top : 0px;
}

.yith-plugin-ui .form-table td {
    padding : 15px 20px;
}

.yith-plugin-ui.metaboxes-tab label {
    color       : #33373b;
    font-size   : 14px;
    margin-left : -248px;
}

.yith-plugin-ui.metaboxes-tab .the-metabox {
    margin      : 40px 0 40px 290px;
    margin-left : 264px;
}

.yith-plugin-ui.metaboxes-tab label {
    width       : 200px;
    font-weight : 600;
}

.yith-plugin-ui span.description {
    color       : #716269;
    font-size   : 13px;
    font-style  : normal;
    font-weight : 400;
    margin-top  : 15px;
    margin-left : 0;
}

#wpwrap .yith-plugin-ui h2.nav-tab-wrapper, #wpbody-content .yith-plugin-ui h2.nav-tab-wrapper {
    border           : 0;
    border-bottom    : 1px solid #ddd;
    margin-right     : 15px;
    margin-bottom    : 0;
    background-color : transparent;
    padding-left     : 0;
}

.yith-plugin-ui #plugin-fw-wc table.form-table,
.yith-plugin-ui #yith-plugin-fw-panel table.form-table,
.yith-plugin-ui table.form-table {
    border        : 1px solid #d8d8d8;
    border-top    : 0;
    margin-bottom : 40px;
}

.yith-plugin-ui #wpwrap h2.nav-tab-wrapper, .yith-plugin-ui #wpbody-content h2.nav-tab-wrapper {
    background-color : transparent;
    border           : 0;
    text-transform   : none;
    border-bottom    : 1px solid #ddd;
}

.yith-plugin-ui h2 {
    color            : #2a8db0;
    font-size        : 16px;
    border           : 1px solid #d8d8d8;
    border-bottom    : 0;
    background-color : #fff;
    padding          : 35px 20px;
    margin           : 0px;
}

.yith-plugin-ui .yith-plugin-fw-custom-tab h2 {
    border       : 0px solid #d8d8d8;
    padding-left : 0;
}

.yith-plugin-ui #plugin-fw-wc table.form-table th, #yith-plugin-fw-panel table.form-table th.titledesc, .yith-plugin-ui #plugin-fw-wc table.form-table td, #yith-plugin-fw-panel table.form-table td.forminp {
    padding : 30px 20px;
}

.yith-plugin-ui #plugin-fw-wc table.form-table tr.toggle-element-fixed td, #yith-plugin-fw-panel table.form-table tr.toggle-element-fixed td.forminp {
    padding : 0 20px 30px;
}

.yith-plugin-ui #plugin-fw-wc h2 + div {
    background     : #fff;
    border-left    : 1px solid #d9d9d9;
    border-right   : 1px solid #d9d9d9;
    margin         : -20px 0 0 0;
    padding-left   : 20px;
    padding-bottom : 5px;
}

.yith-plugin-ui #plugin-fw-wc h2 + div p, .yith-plugin-ui p.info-box {
    background    : #f1f1f1;
    font-size     : 14px;
    font-weight   : 700;
    width         : 50%;
    display       : block;
    padding       : 15px;
    margin-bottom : 0;
}

.yith-plugin-ui #plugin-fw-wc h2 + div p:before, .yith-plugin-ui p.info-box:before {
    content      : "\e90e";
    font-family  : yith-icon;
    font-size    : 30px;
    font-weight  : normal;
    display      : block;
    margin-top   : -12px;
    float        : left;
    margin-right : 15px;
}

.yith-plugin-ui h2:first-child {
    border-top : 0;
}

/*** Icon ***/
.yith-plugin-ui .yith-icon--right-overlay {
    position    : relative;
    left        : -33px;
    line-height : 36px;
    width       : 0;
}

/*** General Input Style ***/

.yith-plugin-ui .yith-plugin-fw select,
.yith-plugin-ui textarea:not(.wp-editor-area),
.yith-plugin-ui input[type=number],
.yith-plugin-ui.metaboxes-tab input[type=number],
.yith-plugin-ui input[type=text],
.yith-plugin-ui input[type=email],
.yith-plugin-ui input[type=password],
.yith-plugin-ui .search-box input[name="s"],
.yith-plugin-fw.yith-plugin-ui input[type=text]:not(.select2-search__field),
.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text],
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text]:not(.select2-search__field),
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table select,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table textarea:not(.wp-editor-area),
.woocommerce .yith-plugin-fw table.form-table input[type=password],
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=number],
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text] {
    border        : 1px solid #d8d8d8;
    border-radius : 8px;
    padding       : 8px 10px;
    height        : 38px;
    min-width     : 90px;
    box-shadow    : none;
    color         : #716269;
}


.yith-plugin-ui input::-webkit-input-placeholder {
    color : #999999;
}

.yith-plugin-ui input:-ms-input-placeholder {
    color : #999999;
}

.yith-plugin-ui input::-ms-input-placeholder {
    color : #999999;
}

.yith-plugin-ui input::placeholder {
    color : #999999;
}

.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=number] {
    min-width : 70px;
}

.yith-plugin-ui textarea:not(.wp-editor-area),
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table textarea:not(.wp-editor-area) {
    height  : auto;
    padding : 8px;
}

.yith-plugin-ui input[type=text]:not(.select2-search__field).wp-color-picker,
.yith-plugin-ui table.form-table input[type=text].wp-color-picker,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text].wp-color-picker {
    border    : 0;
    height    : 30px;
    max-width : 121px;
    font-size : 11px;
    padding   : 0;
}

.yith-plugin-ui .search-box input[name="s"] {
    height : 35px;
}

.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text].small-text,
.yith-plugin-ui input[type=text].small-text {
    width  : 100px;
    height : 25px;
    border : 1px solid #d9d9d9;
}


.yith-plugin-ui select:focus,
.yith-plugin-ui .yith-plugin-fw select:focus,
.woocommerce .yith-plugin-ui table.form-table select:focus,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table select:focus,
.yith-plugin-ui input[type=text]:not(.select2-search__field):focus,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text]:not(.select2-search__field):focus,
.yith-plugin-ui input[type=number]:focus,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=number]:focus,
.yith-plugin-ui textarea:not(.wp-editor-area):focus,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table textarea:not(.wp-editor-area):focus {
    border-color : #a7d9ec;
}

.woocommerce .yith-plugin-ui table.form-table select,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table select,
.yith-plugin-ui select {
    -webkit-appearance : none;
    line-height        : 1.2em;
    min-height         : 35px;
    background         : #fff url("../images/arrow_down.svg") no-repeat right center;
    background-size    : 35px 13px;
    border-radius      : 8px;
    padding-right      : 30px;
}

.yith-plugin-ui .form-table th {
    width         : 200px;
    padding-right : 50px;
}

/** SELECT 2 **/
.yith-plugin-ui .select2-container,
.yith-plugin-ui .select2-selection--single {
    height  : auto;
    padding : 2px;
    outline : none;
}

.yith-plugin-ui .select2-container .select2-selection--single,
.yith-plugin-ui .select2-container .select2-selection--multiple,
.yith-plugin-ui .select2-container.select2-container--focus .select2-selection--single,
.yith-plugin-ui .select2-container.select2-container--focus .select2-selection--multiple,
.yith-plugin-ui .select2-container.select2-container--open .select2-selection--single,
.yith-plugin-ui .select2-container.select2-container--open .select2-selection--multiple {
    margin     : 0;
    border     : 0;
    box-shadow : none !important;
}

.yith-plugin-ui .select2-container .select2-selection--single .select2-selection__arrow {
    background : none;
}

/* old panel*/

.yith-plugin-fw-select2-container {
    z-index : 9999;
}

.yith-plugin-ui span.select2.select2-container.select2-container--default,
span.select2.select2-container.select2-container--default.yith-plugin-fw-select2-container {
    border        : 1px solid #d8d8d8;
    border-radius : 8px;
}

.yith-plugin-ui span.select2.select2-container.select2-container--default:hover,
.yith-plugin-ui span.select2.select2-container.select2-container--default:active,
.yith-plugin-ui span.select2.select2-container.select2-container--default:focus {
    border-color : #a7d9ec;
}

.yith-plugin-ui .select2-container--default .select2-selection--multiple .select2-selection__choice__remove {
    color : #fff;
}

.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text].select2-search__field,
.yith-plugin-ui .yith-plugin-fw table.form-table input.select2-search__field {
    border    : 0;
    padding   : 0;
    height    : initial;
    min-width : 380px;
}

.yith-plugin-ui .select2-container--default .select2-selection--multiple .select2-selection__choice,
.yith-plugin-ui .select2-selection__choice {
    color            : #fff;
    background-color : #4e8ba2;
    border-radius    : 12px;
    padding          : 3px 11px;
    border-color     : #4e8ba2;
    font-size        : 13px;
}

.wc-wp-version-gte-53 .yith-plugin-ui .select2-container .select2-selection--single .select2-selection__arrow {
    background-image : none;
}

.yith-plugin-ui .select2-container--default .select2-selection--single .select2-selection__arrow:before {
    content     : "\e900";
    font-family : 'yith-icon' !important;
    line-height : 36px;
    color       : #d9d9d9;
}

.yith-plugin-ui .select2-container--default .select2-selection--single .select2-selection__arrow b {
    display : none;
}

.yith-plugin-fw-select2-container.select2-container--open .select2-dropdown:after {
    display : none;
}

.yith-plugin-fw-select2-container.select2-container--open .select2-dropdown--below,
.yith-plugin-fw-select2-container.select2-container--open .select2-dropdown--above {
    margin-top    : 10px;
    border-radius : 8px;
    box-shadow    : 0 2px 4px 0 rgba(0, 0, 0, 0.12), 0 2px 10px 0 rgba(0, 0, 0, 0.12) !important;
    border        : 0;
}

.yith-plugin-fw-select2-container.select2-container--open .select2-dropdown--above {
    margin-top : -10px;
}

.yith-plugin-fw-select2-container.select2-container--default.select2-container--open.select2-container--below .select2-selection--single,
.yith-plugin-fw-select2-container.select2-container--default.select2-container--open.select2-container--above .select2-selection--single {
    border-radius : 8px;
}

.yith-plugin-fw-select2-container .select2-results .select2-results__group, .select2-results .select2-results__option:first-child {
    border-top-left-radius  : 8px;
    border-top-right-radius : 8px;
}

.yith-plugin-fw-select2-container .select2-results .select2-results__group,
.yith-plugin-fw-select2-container .select2-results .select2-results__option:last-child {
    border-bottom-left-radius  : 8px;
    border-bottom-right-radius : 8px;
}

.yith-plugin-fw-select2-container .select2-dropdown {
    border : 0;
}

.yith-plugin-fw-select2-container.select2-container--default .select2-results__option[data-selected=true] {
    background-color : #fff;
    outline          : none;
}

.yith-plugin-fw-select2-container.select2-container--default .select2-results__option--highlighted[aria-selected] {
    background-color : #e8eff1;
    color            : #4e8ba2;
    outline          : none;
}

.yith-plugin-ui .select2-container--default .select2-selection--multiple .select2-selection__choice__remove,
.yith-plugin-fw-panel .select2-container--default .select2-selection--multiple .select2-selection__choice__remove {
    float        : right;
    padding-left : 15px;
    text-indent  : -9999px;
}

.yith-plugin-fw-panel .select2-container--default .select2-selection--multiple .select2-selection__choice__remove:after,
.yith-plugin-ui .select2-container--default .select2-selection--multiple .select2-selection__choice__remove:after {
    float       : right;
    color       : #fff;
    content     : "\e906";
    font-family : 'yith-icon' !important;
    font-size   : 9px;
    line-height : 21px;
    text-indent : 0;
}

.yith-plugin-ui .select2-container .select2-selection--multiple,
.yith-plugin-ui .select2-container--default.select2-container--focus .select2-selection--multiple,
.wc-wp-version-gte-53 .yith-plugin-ui .select2-container.select2-container--open .select2-selection--multiple {
    border     : 0;
    box-shadow : none;
}

.yith-plugin-fw .select2-search input[type=text] {
    border  : 0;
    padding : 0;
    height  : auto;
}

.yith-plugin-ui .yith-plugin-fw-select-field-wrapper span.yith-icon.yith-icon-magnifier,
.yith-plugin-fw-panel .yith-plugin-fw-select-field-wrapper span.yith-icon.yith-icon-magnifier {
    position    : absolute;
    padding-top : 5px;
}

.yith-plugin-ui input[type=text].select2-search__field {
    border  : 0;
    padding : 0;
    height  : 30px;
}

.yith-plugin-ui .select2-container--default .select2-search--dropdown .select2-search__field,
.yith-plugin-fw-panel .select2-container--default .select2-search--dropdown .select2-search__field {
    border        : 1px solid #d9d9d9;
    border-radius : 8px;
    box-shadow    : none;
}

.yith-plugin-ui .select2-dropdown,
.yith-plugin-fw-panel .select2-dropdown {
    border : 1px solid #d9d9d9;
}

/** RADIO BUTTON **/
.yith-plugin-ui [type="radio"]:checked,
.yith-plugin-ui [type="radio"]:not(:checked) {
    position : absolute;
    left     : -9999px;
}

.yith-plugin-ui [type="radio"]:checked + label,
.yith-plugin-ui [type="radio"]:not(:checked) + label {
    position     : relative;
    padding-left : 35px;
    cursor       : pointer;
    line-height  : 20px;
    display      : inline-block;
    color        : #716269;
    font-size    : 14px;
}

.yith-plugin-ui [type="radio"]:checked + label:before,
.yith-plugin-ui [type="radio"]:not(:checked) + label:before {
    content       : '';
    position      : absolute;
    left          : 0;
    top           : 0;
    width         : 19px;
    height        : 19px;
    border        : 1px solid #d8d8d8;
    border-radius : 100%;
    background    : #fff;
}

.yith-plugin-ui [type="radio"]:checked + label:after,
.yith-plugin-ui [type="radio"]:not(:checked) + label:after {
    content            : '';
    width              : 13px;
    height             : 13px;
    background         : #4e8ba2;
    position           : absolute;
    top                : 4px;
    left               : 4px;
    border-radius      : 100%;
    -webkit-transition : all 0.2s ease;
    transition         : all 0.2s ease;
}

.yith-plugin-ui [type="radio"]:not(:checked) + label:after {
    opacity           : 0;
    -webkit-transform : scale(0);
    transform         : scale(0);
}

.yith-plugin-ui [type="radio"]:checked + label:after {
    opacity           : 1;
    -webkit-transform : scale(1);
    transform         : scale(1);
}

/** CHECKBOX **/
.yith-plugin-ui input[type="checkbox"] {
    -webkit-appearance : none;
    background-color   : #fff;
    border             : 1px solid #d8d8d8;
    box-shadow         : none;
    width              : 18px;
    height             : 18px;
    border-radius      : 3px;
    display            : inline-block;
    position           : relative;
}

.yith-plugin-ui input[type="checkbox"]:checked {
    background-color : #4e8ba2;
    border-color     : #488197;
}

.yith-plugin-ui .forminp-checkbox span.description.inline {
    margin-left : 10px;
}

.yith-plugin-ui input[type="checkbox"]:checked:before {
    display : none;
}

.yith-plugin-ui input[type="checkbox"]:checked:after {
    content     : "\e905";
    font-family : 'yith-icon';
    font-size   : 10px;
    position    : absolute;
    font-weight : 600;
    top         : 8px;
    left        : 3px;
    color       : #fff;
}

/** ONOFF **/
.yith-plugin-ui .yith-plugin-fw-onoff-container input[type="checkbox"] {
    display : none;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input + span {
    border           : 1px solid #d8d8d8;
    background-color : #fff;
    width            : 60px;
    height           : 24px;
    line-height      : 14px;
    padding          : 4px;
    box-sizing       : border-box;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input + span:before {
    background-color : #d8d8d8;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input + span:before {
    width            : 13px;
    height           : 13px;
    top              : 5px;
    left             : 6px;
    background-color : #d8d8d8;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input + span:after {
    content      : attr(data-text-off);
    height       : 14px;
    font-size    : 12px;
    font-weight  : 600;
    padding-left : 18px;
    color        : #979797;
    display      : block;
    white-space  : nowrap;
    text-align   : center;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input:checked + span,
.yith-plugin-ui .yith-plugin-fw-onoff-container input.onoffchecked + span {
    background-color : #fff;
    border-color     : #98aa36;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input:checked + span:before,
.yith-plugin-ui.yith-plugin-fw-onoff-container input.onoffchecked + span:before {
    background-color : #98aa36;
    left             : 38px;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input:checked + span:after,
.yith-plugin-ui.yith-plugin-fw-onoff-container input.onoffchecked + span:after {
    content       : attr(data-text-on);
    color         : #98aa36;
    height        : 14px;
    font-size     : 12px;
    font-weight   : 600;
    padding-left  : 0;
    padding-right : 18px;
    display       : block;
    white-space   : nowrap;
    text-align    : center;
}

.yith-plugin-ui .yith-plugin-fw-onoff-container input + span {
    text-indent : 0;
}


/**
DATEPICKER
 */
.yith-plugin-ui .yith-plugin-fw-datepicker + .yith-icon-calendar,
.yith-plugin-fw-datepicker + .yith-icon-calendar {
    color     : #ccc;
    font-size : 18px;
}

div#ui-datepicker-div.yith-plugin-fw-datepicker-div {
    border     : 0;
    box-shadow : 0 2px 4px 0 rgba(0, 0, 0, 0.12), 0 2px 10px 0 rgba(0, 0, 0, 0.12) !important;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-header.ui-widget-header {
    background  : #4e8ba2;
    color       : #fff;
    font-size   : 11px;
    line-height : 25px;
    border      : 0;
    min-height  : 25px;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-title {
    line-height : 25px;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker th {
    color : #716269;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-widget-header .ui-icon,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-widget-header .ui-state-hover,
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker .ui-datepicker-prev,
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker .ui-datepicker-next,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-icon.ui-icon-circle-triangle-w,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-icon.ui-icon-circle-triangle-e {
    background  : none;
    content     : "";
    border      : 0;
    text-indent : 0;
    width       : 15px;
    height      : 20px;
    display     : block;
    overflow    : hidden;
}

.yith-plugin-ui .yith-password-wrapper {
    display  : inline-block;
    width    : auto;
    position : relative;
}

.yith-plugin-ui .yith-password-wrapper .yith-password-eye-closed:before,
.yith-plugin-ui .yith-password-wrapper .yith-password-eye:before {
    content     : '\e911';
    color       : #999;
    font-family : 'yith-icon';
    font-size   : 20px;
    font-weight : normal;
    width       : 15px;
    display     : block;
    position    : absolute;
    right       : 13px;
    top         : 0;
    line-height : 38px;
    bottom      : 0;
    cursor      : pointer;
}

.yith-plugin-ui .yith-password-wrapper .yith-password-eye-closed:before {
    content : '\e912';
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-icon.ui-icon-circle-triangle-w:before {
    content     : "\e901";
    color       : #fff;
    font-family : 'yith-icon';
    font-size   : 12px;
    font-weight : normal;
    width       : 15px;
    display     : block;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-icon.ui-icon-circle-triangle-e:before {
    content     : "\e902";
    color       : #fff;
    text-indent : 0;
    font-family : 'yith-icon';
    font-size   : 12px;
    font-weight : normal;
    width       : 15px;
    display     : block;
}

/* arrow */
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker .ui-datepicker-prev,
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker .ui-datepicker-next,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-prev.ui-datepicker-prev-hover,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-prev.ui-datepicker-next-hover {
    top    : 1px;
    cursor : pointer;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-prev.ui-datepicker-prev,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-prev.ui-datepicker-prev-hover {
    left : 5px;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker .ui-datepicker-next,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-datepicker-next.ui-datepicker-next-hover {
    right : 1px;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-datepicker table {
    font-size : 10px;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-state-default,
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-widget-content .ui-state-default,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-widget-header .ui-state-default {
    background : #fff;
    border     : 1px solid #d9d9d9;

}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-state-hover,
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-widget-content .ui-state-hover,
#ui-datepicker-div.yith-plugin-fw-datepicker-div a.ui-state-default:focus,
#ui-datepicker-div.yith-plugin-fw-datepicker-div.ui-widget-content .ui-state-focus,
#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-widget-header .ui-state-focus {
    background : #ebf1f3;
    border     : 1px solid #bfd5dd;
    outline    : none;
    box-shadow : none;
}

#ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-widget-header .ui-state-hover,
.woocommerce #ui-datepicker-div.yith-plugin-fw-datepicker-div .ui-state-focus {
    background : none;
    border     : 0;
}

/** colorpicker **/
.yith-plugin-ui .iris-slider-offset.ui-slider.ui-widget-content {
    background : transparent !important;
    height     : auto !important;
}

.yith-plugin-ui .yith-plugin-fw-field-wrapper .yith-colorpicker-group:not(:first-child) {
    margin-top : 20px;
}

.yith-plugin-ui .yith-plugin-fw-field-wrapper .iris-picker,
.yith-plugin-ui .yith-plugin-fw-field-wrapper .wp-picker-holder {
    display : none !important;
}

.yith-plugin-ui .yith-plugin-fw-field-wrapper .wp-picker-input-wrap,
.yith-plugin-ui .yith-plugin-fw-field-wrapper .wp-picker-input-wrap > * {
    display        : inline-block !important;
    vertical-align : middle;
}

.yith-plugin-ui .yith-plugin-fw-field-wrapper .wp-picker-active .iris-picker,
.yith-plugin-ui .yith-plugin-fw-field-wrapper .wp-picker-active .wp-picker-holder {
    display : block !important;
}

.woocommerce .yith-plugin-ui .wp-picker-container .iris-picker, .yith-plugin-ui .wp-picker-container .iris-picker {
    z-index       : 100;
    position      : absolute;
    margin-top    : 20px;
    border        : 1px solid #d8d8d8;
    border-radius : 4px;
    box-shadow    : 0 1px 3px rgba(0, 0, 0, .2);
}

.yith-plugin-ui .yith-plugin-fw input[type=text].wp-color-picker {
    border  : 0;
    padding : 2px 5px;
    height  : 30px;
}

.yith-plugin-ui .wp-color-result-text {
    display : none;
}

.yith-plugin-ui .wp-picker-container .wp-color-result.button {
    width          : 30px;
    height         : 30px;
    box-shadow     : none;
    border         : 1px solid #d9d9d9;
    margin-bottom  : 0;
    padding        : 0;
    display        : inline-block;
    vertical-align : middle;
    box-sizing     : content-box;
}

.yith-plugin-ui .wp-picker-default-custom {
    cursor         : pointer;
    width          : 25px;
    padding        : 0 5px;
    display        : inline-block;
    vertical-align : middle;
    position       : relative;
}

.yith-plugin-ui .wp-picker-default-custom input.button {
    opacity  : 0;
    margin   : 0;
    padding  : 0;
    position : absolute;
    top      : 0;
    right    : 0;
    left     : 0;
    bottom   : 0;
    z-index  : 2;
}

.yith-plugin-ui .wp-picker-default-custom:before {
    content     : "\e91a";
    text-indent : 0;
    font-family : 'yith-icon';
    font-size   : 16px;
    color       : #d8d8d8;
    font-weight : normal;
    width       : 15px;
    display     : block;
    background  : none;
    line-height : 20px;
    margin-left : 8px;
}

.yith-plugin-ui .wp-picker-container {
    width         : auto;
    display       : inline-block;
    border        : 1px solid #d9d9d9;
    border-radius : 8px;
    padding       : 5px;
}

.yith-plugin-ui .iris-picker .iris-square-inner {
    box-shadow : 0 0 0 1px #d9d9d9 inset;
}

.yith-plugin-ui .iris-picker .iris-square-value {
    width      : 0;
    height     : 0;
    background : transparent;
    border     : 0;
}

.yith-plugin-ui .yith-single-colorpicker {
    display       : inline-block;
    width         : 220px;
    margin-bottom : 10px;
}

.yith-plugin-ui .yith-single-colorpicker > label {
    display        : block;
    font-size      : 11px;
    font-weight    : 600;
    margin         : 0 0 10px 0;
    text-transform : uppercase;
}

.yith-plugin-ui .the-metabox.multi-colorpicker span.description,
.yith-plugin-ui .yith-plugin-fw-panel-wc-row.multi-colorpicker span.description,
.yith-plugin-ui .yith-plugin-fw-multi-colorpicker-field-wrapper span.description {
    margin-top : 0;
}

/* RANGE SLIDER */
.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider-horizontal .ui-slider-handle {
    top : -45px !important;
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider-horizontal .ui-slider-handle {
    position           : absolute;
    top                : -33px;
    left               : 0;
    width              : auto !important;
    height             : auto !important;
    padding            : 2px 8px;
    margin-left        : -15px;
    font-size          : 12px;
    line-height        : 20px;
    color              : #4e8ba2;
    text-align         : center;
    background-color   : #fff;
    border             : 1px solid #fff;
    border-radius      : 3px;
    -webkit-transition : opacity .3s ease-in-out 0s;
    transition         : opacity .3s ease-in-out 0s;
    box-shadow         : 0px 1px 7px 0px rgba(1, 1, 1, 0.13);
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider-horizontal .ui-slider-handle:before {
    position          : absolute;
    bottom            : -3px;
    left              : 50%;
    display           : inline-block;
    width             : 6px;
    height            : 6px;
    margin-left       : -3px;
    content           : "";
    background-color  : #fff;
    -webkit-transform : rotate(-45deg);
    -ms-transform     : rotate(-45deg);
    transform         : rotate(-45deg);
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider.ui-widget-content .ui-widget-header {
    background : #4e8ba2;
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider-horizontal .ui-slider-handle:after {
    content       : '';
    width         : 17px;
    height        : 17px;
    display       : block;
    position      : absolute;
    background    : #fff;
    top           : 37px;
    border-radius : 50%;
    box-shadow    : 0px 1px 6px 0px rgba(1, 1, 1, 0.25);
    cursor        : grab;
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider-horizontal .ui-slider-handle:active:after {
    cursor : grabbing;
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider.ui-widget-content {
    height : 5px !important;
}

.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider .minCaption,
.yith-plugin-ui .yith-plugin-fw-slider-container .ui-slider .maxCaption {
    font-weight : 700;
    font-size   : 12px;
}


/* ADD_BOX AND TOGGLE */
.yith-plugin-ui .yith-add-box,
.yith-plugin-ui .yith-toggle-row {
    width         : 70%;
    border        : 1px solid #d9d9d9;
    box-shadow    : none;
    border-radius : 4px;
    margin        : 20px 0;
    padding       : 10px;
    box-sizing    : border-box;
    background    : white;
}

.yith-plugin-ui .yith-toggle-row {
    cursor     : pointer;
    transition : all ease 0.5s;
}

.yith-plugin-ui .yith-toggle-row.fixed {
    margin : 0;
}

.yith-plugin-ui .yith-toggle-row.highlight {
    transition : all ease 0.5s;
    /*box-shadow: 0 0 7px rgb(81, 117, 129);*/
    box-shadow : 0 0 7px rgb(30, 140, 190);

    display    : block;
}

.yith-plugin-ui .yith-add-box {
    padding : 25px;
    display : none;
}

.yith-plugin-ui .yith-toggle-title {
    position : relative;
}

.yith-plugin-ui #plugin-fw-wc .yith-toggle-title h3,
.yith-plugin-ui.metaboxes-tab h3,
.yith-plugin-ui .yith-toggle-title h3,
.metaboxes-tab.yith-plugin-ui .yith-toggle-title h3 {
    padding        : 0px 20px 0px 30px;
    margin         : 3px 0;
    display        : inline-block;
    position       : relative;
    min-width      : 173px;
    box-sizing     : border-box;
    font-weight    : 600;
    font-size      : 15px;
    color          : #23282d;
    text-transform : inherit;
}

.yith-plugin-ui .yith-toggle-title .subtitle {
    padding-left : 0;
    padding-top  : 8px;
    font-size    : 12px;
}

.yith-plugin-ui .yith-toggle {
    position        : absolute;
    width           : 25px;
    cursor          : pointer;
    left            : 0;
    top             : 0;
    bottom          : 0;
    margin          : auto;
    display         : flex;
    align-items     : center;
    box-sizing      : border-box;
    align-items     : center;
    justify-content : center;
}

.yith-plugin-ui .yith-toggle > span:before {
    color : #405e69;
}

.yith-plugin-ui .yith-toggle > span {
    position    : absolute;
    /* right       : 50%; */
    transition  : transform .3s;
    line-height : 1;
    font-size   : 13px;
    font-weight : 600;
}


.yith-plugin-ui .yith-toggle-row-opened .yith-toggle span {
    transform : rotateZ(90deg);
}

.yith-plugin-ui .yith-toggle-content {
    display  : none;
    position : relative;
    padding  : 30px 25px;
}

.yith-plugin-ui .yith-toggle-row-opened .yith-toggle-content {
    /*  display: block;*/
}

.yith-plugin-ui .yith-add-box-row,
.yith-plugin-ui .yith-toggle-content .yith-toggle-content-row {
    display       : table;
    margin-bottom : 40px;
}

.yith-plugin-ui .yith-add-box-row > label,
.yith-plugin-ui .yith-toggle-content .yith-toggle-content-row > label {
    display        : table-cell;
    min-width      : 180px;
    padding-right  : 20px;
    color          : #33373b;
    font-size      : 14px;
    vertical-align : top;
    font-weight    : 600;
}

.yith-plugin-ui .yith-add-box-row .yith-plugin-fw-field-wrapper:not(.yith-plugin-fw-title-field-wrapper),
.yith-plugin-ui .yith-plugin-fw-field-wrapper:not(.yith-plugin-fw-title-field-wrapper) {
    /* display: table-cell;*/
    width          : auto;
    float          : none;
    vertical-align : middle;
    position       : relative;
}

.yith-plugin-ui .yith-add-box-row span.yith-icon,
.yith-plugin-ui .yith-toggle-content .yith-toggle-content-row span.yith-icon {
    position : absolute;
    right    : 15px;
    top      : 0;
    bottom   : 0;
}

.yith-plugin-ui .yith-add-box-buttons,
.yith-plugin-ui .yith-toggle-content-buttons {
    text-align : right;
}

.yith-plugin-ui .yith-toggle-onoff {
    position : absolute;
    cursor   : pointer;
    right    : 0;
}

.yith-plugin-ui .yith-toggle_wrapper.ui-sortable .yith-toggle-onoff {
    right : 30px;
}

.yith-plugin-ui .yith-toggle-row .yith-icon-drag {
    position  : absolute;
    right     : 2px;
    top       : 50%;
    transform : translateY(-50%);
    font-size : 18px;
}

.yith-plugin-ui .yith-toggle-row.with-subtitle .yith-toggle-onoff {
    top : 25%;
}

/** SPINNER **/
.yith-plugin-ui .spinner {
    vertical-align : -5px;
    float          : none;
}

.yith-plugin-ui .spinner.show {
    visibility : visible;
}

/* LIST TABLE */
.yith-plugin-ui .form-table .list-table td {
    padding : 15px 0px;
}

.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table .bulkactions select {
    border     : 1px solid #d8d8d8;
    max-width  : 200px;
    max-height : 30px;
    padding    : 0 10px;
}

.yith-plugin-ui .form-table .list-table th {
    width : initial;
}

.yith-plugin-ui #plugin-fw-wc table.form-table th.check-column {
    padding-left : 3px;
}

.yith-plugin-ui #plugin-fw-wc table.wp-list-table tbody td {
    padding-left : 20px;
}

.yith-plugin-ui #plugin-fw-wc table.wp-list-table th,
.yith-plugin-ui #plugin-fw-wc table.wp-list-table td {
    padding : 15px 20px 15px 20px;
}

.yith-plugin-ui #plugin-fw-wc table.wp-list-table thead td,
.yith-plugin-ui #plugin-fw-wc table.wp-list-table thead th,
.yith-plugin-ui #plugin-fw-wc table.wp-list-table tfoot td,
.yith-plugin-ui #plugin-fw-wc table.wp-list-table tfoot th {
    padding : 3px 20px;
}

.yith-plugin-ui #plugin-fw-wc table.wp-list-table thead a,
.yith-plugin-ui #plugin-fw-wc table.wp-list-table tfoot a {
    padding-left : 0;
}

.yith-plugin-ui .yith-plugin-fw-list-table .list-table-title {
    margin-bottom : 20px;
    line-height   : 1.2;
}

.yith-plugin-ui .yith-plugin-fw-list-table .list-table-title > * {
    vertical-align : middle;
}

.yith-plugin-ui .yith-plugin-fw-list-table h2 {
    border       : 0;
    padding      : 0;
    display      : inline-block;
    margin-right : 10px;
    line-height  : inherit;
}

.yith-plugin-ui .form-table .yith-plugin-fw-list-table table th {
    vertical-align : middle;
}

.yith-plugin-ui #doaction, .yith-plugin-ui #doaction2,
.yith-plugin-ui #post-query-submit,
.yith-plugin-ui #search-submit,
.yith-plugin-ui .button.filter-button {
    line-height : 33px;
}

.yith-plugin-ui .tablenav {
    margin : 20px 0;
}

.yith-plugin-ui .form-table td p.yith-section-description {
    color         : #716269;
    margin-bottom : 20px;
}

/** Time picker width*/
.yith-plugin-ui input.ui-timepicker-input {
    width : 100px !important;
}

/** date format **/
.yith-plugin-ui .yith-plugin-fw-date-format code {
    margin-left : 30px;
}

.yith-plugin-ui .yith-plugin-fw-date-format input[type=text].small-text {
    margin-left : 45px;
}

/** checkbox columns **/
.yith-plugin-ui td.forminp-checkbox.two-cols fieldset {
    width         : calc(50% - 20px);
    padding-right : 20px;
    float         : left;
}

.yith-plugin-ui td.forminp-checkbox.two-cols fieldset:nth-child(2n+1) {
    clear : both;
}

.yith-plugin-ui td.forminp-checkbox.three-cols fieldset {
    width         : calc(33% - 10px);
    padding-right : 10px;
    float         : left;
}

.yith-plugin-ui td.forminp-checkbox.three-cols fieldset:nth-child(3n+1) {
    clear : both;
}

.yith-plugin-fw-checkbox-field-wrapper {
    display      : inline-block !important;
    margin-right : 10px;
}

.yith-plugin-fw-checkbox-field-wrapper + span.description {
    display        : inline-block;
    vertical-align : middle;
    margin         : 0;
}

.yith-plugin-ui span.description {
    margin-top  : 10px;
    line-height : 20px;
    max-width   : 40%;
    min-width   : min(100%, 360px);
}

.yith-plugin-fw-option-with-description {
    display        : table-cell;
    width          : 90%;
    vertical-align : top;
}


/**
 * Required
 */
.yith-plugin-fw-panel-wc-row.yith-plugin-fw--required th.titledesc > label:after,
.yith-plugin-fw-metabox-field-row.yith-plugin-fw--required label:first-child:after,
.yith-plugin-fw-toggle-element-field-wrapper .yith-plugin-fw--required > label:first-child:after {
    content     : '*';
    color       : #ea0034;
    font-weight : 800;
    margin-left : 4px;
}

/** YITH PANEL **/
.yith-plugin-ui #yith-plugin-fw-panel {
    margin-top : -10px;
}

.yith-plugin-ui #plugin-fw-wc, .yith-plugin-ui #yith-plugin-fw-panel {
    padding-top : 0;
}

.yith-plugin-ui .plugin-option tr {
    border : 0;
}

/** MULTI SELECT AND TEXT ARRAY INLINE STYLE **/
.yith-plugin-ui .yith-plugin-fw-multi-select .yith-single-select,
.yith-plugin-ui .yith-plugin-fw-text-array-inline .yith-single-text {
    display      : inline-block;
    width        : 30%;
    margin-right : 3%;
    max-width    : 400px;
}

.yith-plugin-ui .yith-plugin-fw-multi-select .yith-single-select label,
.yith-plugin-ui .yith-plugin-fw-text-array-inline .yith-single-text label {
    display     : block;
    font-size   : 11px;
    font-weight : 600;
    margin      : 0 0 10px 0;
}

.yith-plugin-ui .yith-plugin-fw-text-array-inline .yith-single-text input,
.yith-plugin-ui .yith-plugin-fw-multi-select .yith-single-select select,
.yith-plugin-ui .yith-plugin-fw-multi-select .yith-single-select span.select2 {
    width     : 100% !important;
    min-width : auto !important;
}

/**
 * Dimensions
 */

.yith-plugin-fw-dimensions {
    display     : flex;
    align-items : flex-end;
}

.yith-plugin-fw-dimensions__units {
    flex          : 1;
    display       : flex;
    margin-left   : 5px;
    width         : calc(100% - 105px);
    flex-wrap     : wrap;
    min-height    : 38px;
    margin-bottom : -8px;
}

.yith-plugin-fw-dimensions__unit {
    display     : inline-block;
    padding     : 0 3px;
    cursor      : pointer;
    font-weight : 700;
}

.yith-plugin-fw-dimensions__unit--selected {
    text-decoration : underline;
    color           : #007694;
}

.yith-plugin-fw-dimensions__unit--unique {
    text-decoration : none;
    cursor          : default;
}

.yith-plugin-fw-dimensions__dimensions {
    display     : flex;
    margin      : 0;
    align-items : flex-end;
    max-width   : 400px;
    width       : 400px;
}

.yith-plugin-fw-dimensions__dimension {
    text-align : center;
    flex       : 1;
}

.yith-plugin-fw-dimensions__dimension > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui .yith-plugin-fw-dimensions__dimension > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.yith-plugin-ui .yith-plugin-fw-dimensions__dimension > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.woocommerce .yith-plugin-fw table.form-table .yith-plugin-fw-dimensions__dimension > input[type=number].yith-plugin-fw-dimensions__dimension__number {
    width         : 100%;
    min-width     : 0;
    border-radius : 0;
    border        : 1px solid #d8d8d8;
    border-left   : none;
    height        : 38px;
    text-align    : center;
    padding-right : 0;
}

.yith-plugin-fw-dimensions__dimension:first-child > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui .yith-plugin-fw-dimensions__dimension:first-child > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.yith-plugin-ui .yith-plugin-fw-dimensions__dimension:first-child > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.woocommerce .yith-plugin-fw table.form-table .yith-plugin-fw-dimensions__dimension:first-child > input[type=number].yith-plugin-fw-dimensions__dimension__number {
    border-left   : 1px solid #d8d8d8;
    border-radius : 6px 0 0 6px;
}

.yith-plugin-fw-dimensions__dimension:last-child > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui .yith-plugin-fw-dimensions__dimension:last-child > input[type=number].yith-plugin-fw-dimensions__dimension__number,
.yith-plugin-ui .yith-plugin-fw-dimensions__dimension:last-child > input[type=number].yith-plugin-fw-dimensions__dimension__number {
    border-radius : 0 6px 6px 0;
}

.metaboxes-tab .yith-plugin-fw-dimensions__dimension__label,
.yith-plugin-ui.metaboxes-tab label.yith-plugin-fw-dimensions__dimension__label {
    float       : none;
    margin      : 0;
    width       : auto;
    font-weight : inherit;
    line-height : inherit;
}

.yith-plugin-fw-dimensions__dimension__label,
.metaboxes-tab .yith-plugin-fw-dimensions__dimension__label,
.yith-plugin-ui.metaboxes-tab label.yith-plugin-fw-dimensions__dimension__label {
    text-transform : uppercase;
    letter-spacing : 1px;
    font-size      : 9px;
    margin-bottom  : 5px;
    display        : inline-block;
}

.yith-plugin-fw-dimensions__linked {
    cursor : pointer;
}

.yith-plugin-fw-dimensions__linked span {
    border        : 1px solid #d8d8d8;
    border-left   : none;
    height        : 38px;
    border-radius : 0 6px 6px 0;
    box-sizing    : border-box;
    padding       : 8px 10px;
    width         : 100%;
}

.yith-plugin-fw-dimensions--linked-active .yith-plugin-fw-dimensions__linked span {
    color        : #fff;
    background   : #007694;
    border-color : #007694;
}

/**
 * Image dimensions
 */
.yith-plugin-ui .yith-plugin-fw-image-dimensions > div {
    display      : inline-block;
    width        : 30%;
    margin-right : 3%;
    max-width    : 100px;
}

.yith-plugin-ui .yith-plugin-fw-image-dimensions label {
    display        : block;
    font-size      : 11px;
    font-weight    : 600;
    text-transform : uppercase;
    margin         : 0 0 10px 0;
}

/**
 * Copy to clipboard
 */
.yith-plugin-fw-copy-to-clipboard {
    background    : #ffffff;
    border-radius : 8px;
    border        : 1px solid #d8d8d8;
    display       : flex;
    align-items   : center;
    padding       : 0 12px;
    max-width     : 400px;
    box-sizing    : border-box;
}

.yith-plugin-fw-copy-to-clipboard.yith-plugin-fw-copy-to-clipboard--readonly {
    background : #f1f1f1;
    border     : 1px dashed #cccccc;
}

.yith-plugin-fw-copy-to-clipboard__field-wrap {
    position : relative;
    flex     : 1;
}

.yith-plugin-fw-copy-to-clipboard__tip {
    display        : none;
    position       : absolute;
    background     : rgba(30, 30, 30, 0.9);
    color          : #ffffff;
    padding        : 7px 10px;
    border-radius  : 3px;
    right          : 8px;
    top            : 50%;
    transform      : translateY(-50%);
    pointer-events : none;
    font-size      : 12px;
    line-height    : 1;
}


.yith-plugin-fw-copy-to-clipboard input[type=text].yith-plugin-fw-copy-to-clipboard__field,
.yith-plugin-fw.yith-plugin-ui .yith-plugin-fw-copy-to-clipboard input[type=text].yith-plugin-fw-copy-to-clipboard__field,
.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text].yith-plugin-fw-copy-to-clipboard__field,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table input[type=text].yith-plugin-fw-copy-to-clipboard__field {
    height        : 38px;
    width         : 100%;
    background    : transparent;
    padding       : 0 10px 0 0;
    border        : 0;
    border-radius : 0;
    box-shadow    : none;
    outline       : none;
}

.yith-plugin-fw-copy-to-clipboard.yith-plugin-fw-copy-to-clipboard--readonly input[type=text].yith-plugin-fw-copy-to-clipboard__field,
.woocommerce.yith-plugin-fw-panel .yith-plugin-ui table.form-table .yith-plugin-fw-copy-to-clipboard--readonly input[type=text].yith-plugin-fw-copy-to-clipboard__field {
    color  : #999;
    cursor : default;
}

.yith-plugin-fw-copy-to-clipboard__copy {
    display       : flex;
    align-items   : center;
    cursor        : pointer;
    border-radius : 2px;
    padding       : 1px;
    user-select   : none;
    line-height   : 1;
    transition    : all .2s ease-in-out;
}

.yith-plugin-fw-copy-to-clipboard__copy:hover {
    box-shadow       : 0 0 0 4px rgba(15, 67, 60, .12);
    background-color : rgba(15, 67, 60, .12);
}

.yith-plugin-fw-copy-to-clipboard__copy__text {
    font-size   : 12px;
    font-weight : 600;
}

.yith-plugin-fw-copy-to-clipboard__copy__icon {
    margin-right : 4px;
    font-size    : 14px;
}

/** OVERRIDE WOOCOMMERCE CLASSES FOR 5.3 */
.branch-5-3 .yith-plugin-fw .select2-container .select2-selection--single .select2-selection__arrow {
    background : none;
}

.branch-5-3 .yith-plugin-fw .select2-container.select2-container--focus .select2-selection--single, .branch-5-3 .yith-plugin-fw .select2-container.select2-container--open .select2-selection--multiple, .branch-5-3 .yith-plugin-fw .select2-container.select2-container--open .select2-selection--single {
    box-shadow : none;
}

@media screen and (max-width : 1440px) {
    .yith-plugin-ui .yith-add-box,
    .yith-plugin-ui .yith-toggle-row:not(.fixed) {
        width : 80%;
    }


    .yith-plugin-ui .yith-add-box-row,
    .yith-plugin-ui .yith-toggle-content .yith-toggle-content-row,
    .yith-plugin-ui .yith-add-box-row label {
        display : block;
    }

    .yith-plugin-ui .yith-toggle-content .yith-toggle-content-row > label {
        display : block;
    }

    .yith-plugin-ui .yith-plugin-fw-option-with-description {
        width : 100%;
    }

    .yith-plugin-ui .yith-add-box-row,
    .yith-plugin-ui .yith-add-box-row label {
        display : block;
    }

    .yith-plugin-ui .yith-toggle-content .yith-plugin-fw-option-with-description,
    .yith-plugin-ui .yith-add-box-row .yith-plugin-fw-option-with-description {
        display     : block;
        padding-top : 20px;
    }


    .yith-plugin-ui .yith-add-box-row .yith-plugin-fw-option-with-description span.description {
        max-width : 100%;
    }
}

@media screen and (max-width : 1024px) {

    .yith-plugin-ui .yith-add-box,
    .yith-plugin-ui .yith-toggle-row:not(.fixed) {
        width : 90%;
    }

    .yith-plugin-ui td.forminp-checkbox.two-cols fieldset {
        width         : 100%;
        padding-right : 20px;
        float         : left;
    }

    .yith-plugin-ui td.forminp-checkbox.three-cols fieldset {
        width         : calc(50% - 10px);
        padding-right : 10px;
        float         : left;
    }

    .yith-plugin-ui td.forminp-checkbox.two-cols fieldset:nth-child(2n+1) {
        clear : both;
    }

    .yith-plugin-ui td.forminp-checkbox.three-cols fieldset:nth-child(3n+1) {
        clear : initial;
    }

}

@media screen and (max-width : 782px) {
    .yith-plugin-fw-banner h1 {
        line-height : 1.2em;
    }

    .yith-plugin-fw-banner h1 {
        font-size     : 12px;
        padding-right : 10px;
    }

    .yith-plugin-ui .yith-add-box,
    .yith-plugin-ui .yith-toggle-row {
        width : 100%;
    }

    .yith-plugin-ui td.forminp-checkbox.two-cols fieldset,
    .yith-plugin-ui td.forminp-checkbox.three-cols fieldset {
        width : 100%;
        clear : initial;
    }

    .yith-plugin-ui .wp-picker-container .wp-color-result.button {
        padding : 0;
    }

    .yith-plugin-ui #plugin-fw-wc table.form-table th,
    #yith-plugin-fw-panel table.form-table th.titledesc,
    .yith-plugin-ui #plugin-fw-wc table.form-table td,
    #yith-plugin-fw-panel table.form-table td.forminp {
        padding : 20px 20px;
    }

    .yith-plugin-ui #plugin-fw-wc table.form-table th.titledesc,
    #yith-plugin-fw-panel table.form-table th.titledesc {
        padding-bottom : 0;
    }
}

@media (max-width : 480px) {
    .yith-plugin-ui .nav-tab {
        font-size : 12px;
    }

    .yith-plugin-ui h2 {
        padding : 20px 20px;
    }

    .yith-plugin-ui [type="radio"]:checked + label:before, .yith-plugin-ui [type="radio"]:not(:checked) + label:before {
        width  : 14px;
        height : 14px;
    }

    .yith-plugin-ui [type="radio"]:checked + label:after, .yith-plugin-ui [type="radio"]:not(:checked) + label:after {
        width  : 10px;
        height : 10px;
        top    : 3px;
        left   : 3px;
    }

    .yith-plugin-ui [type="radio"]:checked + label, .yith-plugin-ui [type="radio"]:not(:checked) + label {
        padding-left : 25px;
    }

    .yith-plugin-ui [type="radio"]:checked + label, .yith-plugin-ui [type="radio"]:not(:checked) + label {
        font-size : 13px;
    }

    .yith-plugin-ui span.description {
        font-size : 12px;
    }

    .yith-plugin-ui #plugin-fw-wc table.form-table th,
    #yith-plugin-fw-panel table.form-table th.titledesc,
    .yith-plugin-ui #plugin-fw-wc table.form-table td,
    #yith-plugin-fw-panel table.form-table td.forminp {
        padding : 15px 20px;
    }

    .yith-plugin-ui .yith-plugin-fw-onoff-container input + span {
        width   : 48px;
        height  : 20px;
        padding : 2px;
    }

    .yith-plugin-ui .yith-plugin-fw-onoff-container input + span:before {
        top    : 4px;
        left   : 3px;
        width  : 10px;
        height : 10px;
    }

    .yith-plugin-ui .yith-plugin-fw-onoff-container input:checked + span:before, .yith-plugin-ui.yith-plugin-fw-onoff-container input.onoffchecked + span:before {
        left : 30px;
    }

    .yith-plugin-ui .yith-plugin-fw-onoff-container input:checked + span:after, .yith-plugin-ui.yith-plugin-fw-onoff-container input.onoffchecked + span:after {
        font-size     : 11px;
        padding-right : 12px;
        margin-top    : 0;
    }

    .yith-plugin-ui .yith-plugin-fw-onoff-container input + span:after {
        font-size    : 11px;
        padding-left : 12px;
        margin-top   : 0;
    }

    .yith-plugin-fw .select2-container .select2-selection--single .select2-selection__rendered {
        font-size : 14px;
    }

    .yith-plugin-ui input {
        font-size : 14px;
    }
}
