<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Symfony\\Component\\CssSelector\\' => array($vendorDir . '/symfony/css-selector'),
    'Psr\\Container\\' => array($vendorDir . '/psr/container/src'),
    'Pelago\\' => array($vendorDir . '/pelago/emogrifier/src'),
    'MaxMind\\Db\\' => array($vendorDir . '/maxmind-db/reader/src/MaxMind/Db'),
    'Composer\\Installers\\' => array($vendorDir . '/composer/installers/src/Composer/Installers'),
    'Automattic\\WooCommerce\\Vendor\\' => array($baseDir . '/lib/packages'),
    'Automattic\\WooCommerce\\Tests\\' => array($baseDir . '/tests/php/src'),
    'Automattic\\WooCommerce\\Testing\\Tools\\' => array($baseDir . '/tests/Tools'),
    'Automattic\\WooCommerce\\Blocks\\' => array($baseDir . '/packages/woocommerce-blocks/src'),
    'Automattic\\WooCommerce\\Admin\\' => array($baseDir . '/packages/woocommerce-admin/src'),
    'Automattic\\WooCommerce\\' => array($baseDir . '/src'),
    'Automattic\\Jetpack\\Autoloader\\' => array($vendorDir . '/automattic/jetpack-autoloader/src'),
);
