:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-20:#005a87}body.admin-color-light{--wp-admin-theme-color:#0085ba;--wp-admin-theme-color-darker-10:#0073a1;--wp-admin-theme-color-darker-20:#006187}body.admin-color-modern{--wp-admin-theme-color:#3858e9;--wp-admin-theme-color-darker-10:#2145e6;--wp-admin-theme-color-darker-20:#183ad6}body.admin-color-blue{--wp-admin-theme-color:#096484;--wp-admin-theme-color-darker-10:#07526c;--wp-admin-theme-color-darker-20:#064054}body.admin-color-coffee{--wp-admin-theme-color:#46403c;--wp-admin-theme-color-darker-10:#383330;--wp-admin-theme-color-darker-20:#2b2724}body.admin-color-ectoplasm{--wp-admin-theme-color:#523f6d;--wp-admin-theme-color-darker-10:#46365d;--wp-admin-theme-color-darker-20:#3a2c4d}body.admin-color-midnight{--wp-admin-theme-color:#e14d43;--wp-admin-theme-color-darker-10:#dd382d;--wp-admin-theme-color-darker-20:#d02c21}body.admin-color-ocean{--wp-admin-theme-color:#627c83;--wp-admin-theme-color-darker-10:#576e74;--wp-admin-theme-color-darker-20:#4c6066}body.admin-color-sunrise{--wp-admin-theme-color:#dd823b;--wp-admin-theme-color-darker-10:#d97426;--wp-admin-theme-color-darker-20:#c36922}.woocommerce-customer-effort-score__selection{margin:1em 0}.woocommerce-customer-effort-score__selection .components-base-control__field{display:flex;flex-direction:column;margin:0 auto;color:#007cba;color:var(--wp-admin-theme-color)}@media(min-width:601px){.woocommerce-customer-effort-score__selection .components-base-control__field{flex-direction:row}}.woocommerce-customer-effort-score__selection .components-radio-control__option:not(:last-child){margin-bottom:0;margin-left:4px}.woocommerce-customer-effort-score__selection .components-radio-control__option input{position:absolute;opacity:0}.woocommerce-customer-effort-score__selection .components-radio-control__option label{display:block;text-align:center;box-sizing:border-box;width:9em;height:100%;padding:1em .5em;font-size:.9em}.woocommerce-customer-effort-score__selection .components-radio-control__option label:hover{background-color:#e6f1f5}.woocommerce-customer-effort-score__selection .components-radio-control__option input:focus+label{outline:2px solid #006088;background-color:peach;color:#055d9c}.woocommerce-customer-effort-score__selection .components-radio-control__option input:checked+label{outline:2px solid #006088;background-color:#e6f1f5}.woocommerce-customer-effort-score__selection .components-radio-control__option label:before{display:block;font-size:24px;text-align:center;margin:1em 0;color:#006088}.woocommerce-customer-effort-score__selection .components-radio-control__option input[value="1"]+label:before{content:"😞"}.woocommerce-customer-effort-score__selection .components-radio-control__option input[value="2"]+label:before{content:"🙁"}.woocommerce-customer-effort-score__selection .components-radio-control__option input[value="3"]+label:before{content:"😐"}.woocommerce-customer-effort-score__selection .components-radio-control__option input[value="4"]+label:before{content:"🙂"}.woocommerce-customer-effort-score__selection .components-radio-control__option input[value="5"]+label:before{content:"😁"}.woocommerce-customer-effort-score__comments label{display:block;color:inherit;font-weight:700}.woocommerce-customer-effort-score__comments textarea{width:100%}.woocommerce-customer-effort-score__buttons{text-align:left}.woocommerce-customer-effort-score__buttons .components-button{margin-right:1em}