/**
 * Internal dependencies
 */
export const blockAttributes = {
	isPreview: {
		type: 'boolean',
		default: false,
		save: false,
	},
	/**
	 * The product ID to display.
	 */
	productId: {
		type: 'number',
	},
};

export default blockAttributes;
