
.wc-block-checkout__additional_fields {
	margin: 1.5em 0 -1.5em;
}
.wc-block-components-checkout-step__description-placeholder {
	opacity: 0.5;
}

.wc-block-components-checkout-step__title {
	display: flex;
	width: 100%;
}
