(window.webpackWcBlocksJsonp=window.webpackWcBlocksJsonp||[]).push([[26],{330:function(c,e,n){"use strict";var a=n(6),t=n.n(a);n(10),e.a=function(c){var e=c.children,n=c.className;return React.createElement("div",{className:t()("wc-block-components-main",n)},e)}},399:function(c,e,n){"use strict";n.r(e);var a=n(330);e.default=function(c){var e=c.children;return React.createElement(a.a,{className:"wc-block-cart__main"},e)}}}]);